// (c) Copyright 2023 Advanced Micro Devices, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of AMD and is protected under U.S. and international copyright
// and other intellectual property laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// AMD, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND AMD HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) AMD shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or AMD had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// AMD products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of AMD products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
//-----------------------------------------------------------------

`timescale 1ns/1fs

module my_trx_quad_top  # (
   parameter C_NO_OF_QUADS        = 1
) (
       input            gtwiz_freerun_clk,

       input            QUAD0_GTREFCLK0,
       input            QUAD0_GTREFCLK1,
       output           QUAD0_rxmarginreqack ,
       output [3:0]     QUAD0_rxmarginrescmd ,
       output [1:0]     QUAD0_rxmarginreslanenum ,
       output [7:0]     QUAD0_rxmarginrespayld ,
       output           QUAD0_rxmarginresreq ,
       input  [3:0]     QUAD0_rxmarginreqcmd ,
       input  [1:0]     QUAD0_rxmarginreqlanenum ,
       input  [7:0]     QUAD0_rxmarginreqpayld ,
       input            QUAD0_rxmarginreqreq ,
       input            QUAD0_rxmarginclk ,
       input            QUAD0_rxmarginresack ,
       input            QUAD0_pcielinkreachtarget ,
       input  [5:0]     QUAD0_pcieltssm ,
       input            QUAD0_bgbypassb ,
       input            QUAD0_bgmonitorenb ,
       input            QUAD0_bgpdb ,
       input  [4:0]     QUAD0_bgrcalovrd ,
       input            QUAD0_bgrcalovrdenb ,
       input            QUAD0_debugtraceready ,
       input            QUAD0_debugtraceclk ,
       input            QUAD0_rcalenb ,
       input            QUAD0_trigackout0 ,
       input            QUAD0_trigin0 ,
       input            QUAD0_ubenable ,
       input  [11:0]    QUAD0_ubintr ,
       input            QUAD0_ubiolmbrst ,
       input            QUAD0_ubmbrst ,
       input            QUAD0_ubrxuart ,
       input            QUAD0_refclk0_clktestsig ,
       input            QUAD0_refclk1_clktestsig ,
       output           QUAD0_correcterr ,
       output [15:0]    QUAD0_debugtracetdata ,
       output           QUAD0_debugtracetvalid ,
       output           QUAD0_refclk0_gtrefclkpdint ,
       output           QUAD0_refclk0_clktestsigint ,
       output           QUAD0_refclk1_gtrefclkpdint ,
       output           QUAD0_refclk1_clktestsigint ,
       output           QUAD0_trigackin0 ,
       output           QUAD0_trigout0 ,
       output           QUAD0_ubinterrupt ,
       output           QUAD0_ubtxuart ,
       output           QUAD0_uncorrecterr ,
       output [1:0]     QUAD0_hsclk0_rxrecclksel ,
       output [1:0]     QUAD0_hsclk1_rxrecclksel ,
       input  [15:0]    QUAD0_ctrlrsvdin0 ,
       input  [13:0]    QUAD0_ctrlrsvdin1 ,
       input  [15:0]    QUAD0_gpi ,
       output [31:0]    QUAD0_ctrlrsvdout ,
       output [15:0]    QUAD0_gpo ,
       output           QUAD0_ch0_bufgtce ,
       output           QUAD0_ch0_bufgtrst ,
       output [3:0]     QUAD0_ch0_bufgtcemask ,
       output [3:0]     QUAD0_ch0_bufgtrstmask ,
       output [11:0]    QUAD0_ch0_bufgtdiv ,
            
       input            QUAD0_ch0_iloreset ,
       output           QUAD0_ch0_iloresetdone ,
       output           QUAD0_ch0_phystatus ,
       input            QUAD0_ch0_clkrsvd0 ,
       input            QUAD0_ch0_clkrsvd1 ,
       input            QUAD0_ch0_dmonitorclk ,
       input            QUAD0_ch0_iloresetmask ,
       input  [2:0]     QUAD0_ch0_loopback ,
       input            QUAD0_ch0_dmonfiforeset,
       input  [15:0]    QUAD0_ch0_pcsrsvdin ,
       input  [15:0]    QUAD0_ch0_gtrsvd ,
       input  [19:0]    QUAD0_ch0_tstin ,
       output [15:0]    QUAD0_ch0_pcsrsvdout ,
       output [15:0]    QUAD0_ch0_pinrsvdas ,
       output           QUAD0_ch0_dmonitoroutclk,
       output           QUAD0_ch0_resetexception ,
       output [31:0]    QUAD0_ch0_dmonitorout,
       output           QUAD0_ch0_phyready ,
       input            QUAD0_ch0_phyesmadaptsave ,
       input            QUAD0_ch0_hsdppcsreset ,
       input            QUAD0_ch0_pcierstb ,
       output           QUAD0_ch1_bufgtce ,
       output           QUAD0_ch1_bufgtrst ,
       output [3:0]     QUAD0_ch1_bufgtcemask ,
       output [3:0]     QUAD0_ch1_bufgtrstmask ,
       output [11:0]    QUAD0_ch1_bufgtdiv ,
            
       input            QUAD0_ch1_iloreset ,
       output           QUAD0_ch1_iloresetdone ,
       output           QUAD0_ch1_phystatus ,
       input            QUAD0_ch1_clkrsvd0 ,
       input            QUAD0_ch1_clkrsvd1 ,
       input            QUAD0_ch1_dmonitorclk ,
       input            QUAD0_ch1_iloresetmask ,
       input  [2:0]     QUAD0_ch1_loopback ,
       input            QUAD0_ch1_dmonfiforeset,
       input  [15:0]    QUAD0_ch1_pcsrsvdin ,
       input  [15:0]    QUAD0_ch1_gtrsvd ,
       input  [19:0]    QUAD0_ch1_tstin ,
       output [15:0]    QUAD0_ch1_pcsrsvdout ,
       output [15:0]    QUAD0_ch1_pinrsvdas ,
       output           QUAD0_ch1_dmonitoroutclk,
       output           QUAD0_ch1_resetexception ,
       output [31:0]    QUAD0_ch1_dmonitorout,
       output           QUAD0_ch1_phyready ,
       input            QUAD0_ch1_phyesmadaptsave ,
       input            QUAD0_ch1_hsdppcsreset ,
       input            QUAD0_ch1_pcierstb ,
       output           QUAD0_ch2_bufgtce ,
       output           QUAD0_ch2_bufgtrst ,
       output [3:0]     QUAD0_ch2_bufgtcemask ,
       output [3:0]     QUAD0_ch2_bufgtrstmask ,
       output [11:0]    QUAD0_ch2_bufgtdiv ,
            
       input            QUAD0_ch2_iloreset ,
       output           QUAD0_ch2_iloresetdone ,
       output           QUAD0_ch2_phystatus ,
       input            QUAD0_ch2_clkrsvd0 ,
       input            QUAD0_ch2_clkrsvd1 ,
       input            QUAD0_ch2_dmonitorclk ,
       input            QUAD0_ch2_iloresetmask ,
       input  [2:0]     QUAD0_ch2_loopback ,
       input            QUAD0_ch2_dmonfiforeset,
       input  [15:0]    QUAD0_ch2_pcsrsvdin ,
       input  [15:0]    QUAD0_ch2_gtrsvd ,
       input  [19:0]    QUAD0_ch2_tstin ,
       output [15:0]    QUAD0_ch2_pcsrsvdout ,
       output [15:0]    QUAD0_ch2_pinrsvdas ,
       output           QUAD0_ch2_dmonitoroutclk,
       output           QUAD0_ch2_resetexception ,
       output [31:0]    QUAD0_ch2_dmonitorout,
       output           QUAD0_ch2_phyready ,
       input            QUAD0_ch2_phyesmadaptsave ,
       input            QUAD0_ch2_hsdppcsreset ,
       input            QUAD0_ch2_pcierstb ,
       output           QUAD0_ch3_bufgtce ,
       output           QUAD0_ch3_bufgtrst ,
       output [3:0]     QUAD0_ch3_bufgtcemask ,
       output [3:0]     QUAD0_ch3_bufgtrstmask ,
       output [11:0]    QUAD0_ch3_bufgtdiv ,
            
       input            QUAD0_ch3_iloreset ,
       output           QUAD0_ch3_iloresetdone ,
       output           QUAD0_ch3_phystatus ,
       input            QUAD0_ch3_clkrsvd0 ,
       input            QUAD0_ch3_clkrsvd1 ,
       input            QUAD0_ch3_dmonitorclk ,
       input            QUAD0_ch3_iloresetmask ,
       input  [2:0]     QUAD0_ch3_loopback ,
       input            QUAD0_ch3_dmonfiforeset,
       input  [15:0]    QUAD0_ch3_pcsrsvdin ,
       input  [15:0]    QUAD0_ch3_gtrsvd ,
       input  [19:0]    QUAD0_ch3_tstin ,
       output [15:0]    QUAD0_ch3_pcsrsvdout ,
       output [15:0]    QUAD0_ch3_pinrsvdas ,
       output           QUAD0_ch3_dmonitoroutclk,
       output           QUAD0_ch3_resetexception ,
       output [31:0]    QUAD0_ch3_dmonitorout,
       output           QUAD0_ch3_phyready ,
       input            QUAD0_ch3_phyesmadaptsave ,
       input            QUAD0_ch3_hsdppcsreset ,
       input            QUAD0_ch3_pcierstb ,
       input            QUAD0_refclk0_gtrefclkpd,
       input            QUAD0_hsclk0_rpllreset,
       input            QUAD0_hsclk0_lcpllreset,
       output           QUAD0_hsclk0_lcpllfbclklost ,
       output           QUAD0_hsclk0_lcplllock ,
       output           QUAD0_hsclk0_lcpllrefclklost ,
       output           QUAD0_hsclk0_lcpllrefclkmonitor ,
       input            QUAD0_hsclk0_lcpllclkrsvd0,
       input            QUAD0_hsclk0_lcpllclkrsvd1,
       input  [7:0]     QUAD0_hsclk0_lcpllrsvd0,
       input  [7:0]     QUAD0_hsclk0_lcpllrsvd1,
       output [7:0]     QUAD0_hsclk0_lcpllrsvdout ,
       input  [7:0]     QUAD0_hsclk0_lcpllfbdiv ,
       input            QUAD0_hsclk0_lcpllpd ,
       input  [2:0]     QUAD0_hsclk0_lcpllrefclksel ,
       input            QUAD0_hsclk0_lcpllresetbypassmode ,
       input  [1:0]     QUAD0_hsclk0_lcpllresetmask ,
       input  [25:0]    QUAD0_hsclk0_lcpllsdmdata ,
       input            QUAD0_hsclk0_lcpllsdmtoggle ,
       output           QUAD0_hsclk0_rpllfbclklost ,
       output           QUAD0_hsclk0_rplllock ,
       output           QUAD0_hsclk0_rpllrefclklost ,
       output           QUAD0_hsclk0_rpllrefclkmonitor ,
       input            QUAD0_hsclk0_rpllclkrsvd0,
       input            QUAD0_hsclk0_rpllclkrsvd1,
       input  [7:0]     QUAD0_hsclk0_rpllrsvd0 ,
       input  [7:0]     QUAD0_hsclk0_rpllrsvd1 ,
       output [7:0]     QUAD0_hsclk0_rpllrsvdout ,
       input  [7:0]     QUAD0_hsclk0_rpllfbdiv ,
       input            QUAD0_hsclk0_rpllpd ,
       input  [2:0]     QUAD0_hsclk0_rpllrefclksel ,
       input            QUAD0_hsclk0_rpllresetbypassmode ,
       input  [1:0]     QUAD0_hsclk0_rpllresetmask ,
       input  [25:0]    QUAD0_hsclk0_rpllsdmdata ,
       input            QUAD0_hsclk0_rpllsdmtoggle ,
       output           QUAD0_hsclk0_rxrecclkout0 ,
       output           QUAD0_hsclk0_rxrecclkout1 ,
       input            QUAD0_refclk1_gtrefclkpd,
       input            QUAD0_hsclk1_rpllreset,
       input            QUAD0_hsclk1_lcpllreset,
       output           QUAD0_hsclk1_lcpllfbclklost ,
       output           QUAD0_hsclk1_lcplllock ,
       output           QUAD0_hsclk1_lcpllrefclklost ,
       output           QUAD0_hsclk1_lcpllrefclkmonitor ,
       input            QUAD0_hsclk1_lcpllclkrsvd0,
       input            QUAD0_hsclk1_lcpllclkrsvd1,
       input  [7:0]     QUAD0_hsclk1_lcpllrsvd0,
       input  [7:0]     QUAD0_hsclk1_lcpllrsvd1,
       output [7:0]     QUAD0_hsclk1_lcpllrsvdout ,
       input  [7:0]     QUAD0_hsclk1_lcpllfbdiv ,
       input            QUAD0_hsclk1_lcpllpd ,
       input  [2:0]     QUAD0_hsclk1_lcpllrefclksel ,
       input            QUAD0_hsclk1_lcpllresetbypassmode ,
       input  [1:0]     QUAD0_hsclk1_lcpllresetmask ,
       input  [25:0]    QUAD0_hsclk1_lcpllsdmdata ,
       input            QUAD0_hsclk1_lcpllsdmtoggle ,
       output           QUAD0_hsclk1_rpllfbclklost ,
       output           QUAD0_hsclk1_rplllock ,
       output           QUAD0_hsclk1_rpllrefclklost ,
       output           QUAD0_hsclk1_rpllrefclkmonitor ,
       input            QUAD0_hsclk1_rpllclkrsvd0,
       input            QUAD0_hsclk1_rpllclkrsvd1,
       input  [7:0]     QUAD0_hsclk1_rpllrsvd0 ,
       input  [7:0]     QUAD0_hsclk1_rpllrsvd1 ,
       output [7:0]     QUAD0_hsclk1_rpllrsvdout ,
       input  [7:0]     QUAD0_hsclk1_rpllfbdiv ,
       input            QUAD0_hsclk1_rpllpd ,
       input  [2:0]     QUAD0_hsclk1_rpllrefclksel ,
       input            QUAD0_hsclk1_rpllresetbypassmode ,
       input  [1:0]     QUAD0_hsclk1_rpllresetmask ,
       input  [25:0]    QUAD0_hsclk1_rpllsdmdata ,
       input            QUAD0_hsclk1_rpllsdmtoggle ,
       output           QUAD0_hsclk1_rxrecclkout0 ,
       output           QUAD0_hsclk1_rxrecclkout1 ,
       input  [3:0]     QUAD0_rxp ,
       input  [3:0]     QUAD0_rxn ,
       output [3:0]     QUAD0_txp ,
       output [3:0]     QUAD0_txn ,

         //output         QUAD0_ch0_gtytxn,
         //output         QUAD0_ch0_gtytxp,
         //input          QUAD0_ch0_gtyrxn,
         //input          QUAD0_ch0_gtyrxp,
         //output         QUAD0_ch0_bufgtclr,
         //output         QUAD0_ch0_bufgtclrmask,
         output         QUAD0_RX0_cfokovrdrdy0,
         output         QUAD0_RX0_cfokovrdrdy1,
         input          QUAD0_RX0_cfokovrdfinish,
         input          QUAD0_RX0_cfokovrdpulse,
         input          QUAD0_RX0_cfokovrdstart,
         output         QUAD0_RX0_dccdone,
         output         QUAD0_RX0_osintstarted,
         output         QUAD0_RX0_osintstrobedone,
         output         QUAD0_RX0_osintstrobestarted,
         output [7:0]   QUAD0_RX0_dataextendrsvd,
         input  [7:0]   QUAD0_TX0_dataextendrsvd,
         output [2:0]   QUAD0_RX0_bufstatus,
         output         QUAD0_RX0_eyescandataerror,
         output [7:0]   QUAD0_RX0_10gstat,
         output         QUAD0_RX0_byteisaligned,
         output         QUAD0_RX0_byterealign,
         output         QUAD0_RX0_chanbondseq,
         output         QUAD0_RX0_chanisaligned,
         output         QUAD0_RX0_chanrealign,
         output [4:0]   QUAD0_RX0_chbondo,
         output [1:0]   QUAD0_RX0_clkcorcnt,
         output         QUAD0_RX0_cominitdet,
         output         QUAD0_RX0_commadet,
         output         QUAD0_RX0_comsasdet,
         output         QUAD0_RX0_comwakedet,
         output [15:0]  QUAD0_RX0_ctrl0,
         output [15:0]  QUAD0_RX0_ctrl1,
         output [7:0]   QUAD0_RX0_ctrl2,
         output [7:0]   QUAD0_RX0_ctrl3,
         output [1:0]   QUAD0_RX0_datavalid,
         output         QUAD0_RX0_dlyalignerr,
         output         QUAD0_RX0_dlyalignprog,
         output         QUAD0_RX0_elecidle,
         output         QUAD0_RX0_finealigndone,
         output [5:0]   QUAD0_RX0_header,
         output [1:0]   QUAD0_RX0_headervalid,
         output         QUAD0_RX0_osintdone,
         output         QUAD0_RX0_phaligndone,
         output         QUAD0_RX0_phalignerr,
         output         QUAD0_RX0_phdlyresetdone,
         output         QUAD0_RX0_phsetinitdone,
         output         QUAD0_RX0_phshift180done,
         output         QUAD0_RX0_sliderdy,
         output [1:0]   QUAD0_RX0_startofseq,
         output [2:0]   QUAD0_RX0_status,
         output         QUAD0_RX0_syncdone,
         output         QUAD0_RX0_valid,
         output         QUAD0_RX0_cdrlock,
         output         QUAD0_RX0_cdrphdone,
         output [127:0] QUAD0_RX0_data,
         output         QUAD0_RX0_outclk,
         output         QUAD0_RX0_pmaresetdone,
         output         QUAD0_RX0_prbserr,
         output         QUAD0_RX0_prbslocked,
         output         QUAD0_RX0_progdivresetdone,
         output         QUAD0_RX0_resetdone,

         output         QUAD0_RX0_mstresetdone,
         input          QUAD0_RX0_mstreset,
         input          QUAD0_RX0_mstdatapathreset,
         input          QUAD0_RX0_gtrxreset,
         input          QUAD0_RX0_cdrhold,
         input          QUAD0_RX0_cdrovrden,
         input          QUAD0_RX0_cdrreset,
         input          QUAD0_RX0_latclk,
         input          QUAD0_RX0_cdrfreqos,
         input          QUAD0_RX0_cdrincpctrl,
         input          QUAD0_RX0_cdrstepdir,
         input          QUAD0_RX0_cdrstepsq,
         input          QUAD0_RX0_cdrstepsx,
         input          QUAD0_RX0_eyescanreset,
         input          QUAD0_RX0_eyescantrigger,
         input  [4:0]   QUAD0_RX0_pcsresetmask,
         input          QUAD0_RX0_cdrbmcdrreq,
         input  [4:0]   QUAD0_RX0_chbondi,
         input          QUAD0_RX0_dapicodeovrden,
         input          QUAD0_RX0_dapicodereset,
         input          QUAD0_RX0_dlyalignreq,
         input          QUAD0_RX0_eqtraining,
         input          QUAD0_RX0_gearboxslip,
         input          QUAD0_RX0_lpmen,
         input          QUAD0_RX0_mldchaindone,
         input          QUAD0_RX0_mldchainreq,
         input          QUAD0_RX0_mlfinealignreq,
         input          QUAD0_RX0_oobreset,
         input          QUAD0_RX0_phalignreq,
         input  [1:0]   QUAD0_RX0_phalignresetmask,
         input          QUAD0_RX0_phdlypd,
         input          QUAD0_RX0_phdlyreset,
         input          QUAD0_RX0_phsetinitreq,
         input          QUAD0_RX0_phshift180,
         input          QUAD0_RX0_slide,
         input          QUAD0_RX0_syncallin,
         input          QUAD0_RX0_termination,
         input  [6:0]   QUAD0_RX0_pmaresetmask,
         input  [1:0]   QUAD0_RX0_pd,
         input          QUAD0_RX0_polarity,
         input          QUAD0_RX0_prbscntreset,
         input  [3:0]   QUAD0_RX0_prbssel,
         input          QUAD0_RX0_progdivreset,
         input  [7:0]   QUAD0_RX0_rate,
         input  [1:0]   QUAD0_RX0_resetmode,
         input          QUAD0_RX0_userrdy,
         input          QUAD0_RX0_usrclk,

         output         QUAD0_TX0_10gstat,
         output [1:0]   QUAD0_TX0_bufstatus,
         output         QUAD0_TX0_comfinish,
         output         QUAD0_TX0_dlyalignerr,
         output         QUAD0_TX0_dlyalignprog,
         output         QUAD0_TX0_phaligndone,
         output         QUAD0_TX0_phalignerr,
         output         QUAD0_TX0_phalignoutrsvd,
         output         QUAD0_TX0_phdlyresetdone,
         output         QUAD0_TX0_phsetinitdone,
         output         QUAD0_TX0_phshift180done,
         output         QUAD0_TX0_syncdone,
         output         QUAD0_TX0_dccdone,
         output         QUAD0_TX0_outclk,
         output         QUAD0_TX0_pmaresetdone,
         output         QUAD0_TX0_progdivresetdone,
         output         QUAD0_TX0_resetdone,
         output         QUAD0_TX0_mstresetdone,
         input          QUAD0_TX0_mstreset,
         input          QUAD0_TX0_mstdatapathreset,
         input          QUAD0_TX0_gttxreset,
         input  [127:0] QUAD0_TX0_data,
         input  [15:0]  QUAD0_TX0_ctrl0,
         input  [15:0]  QUAD0_TX0_ctrl1,
         input  [7:0]   QUAD0_TX0_ctrl2,
         input          QUAD0_TX0_cominit,
         input          QUAD0_TX0_comsas,
         input          QUAD0_TX0_comwake,
         input          QUAD0_TX0_dapicodeovrden,
         input          QUAD0_TX0_dapicodereset,
         input  [1:0]   QUAD0_TX0_deemph,
         input          QUAD0_TX0_detectrx,
         input  [4:0]   QUAD0_TX0_diffctrl,
         input          QUAD0_TX0_dlyalignreq,
         input          QUAD0_TX0_elecidle,
         input  [5:0]   QUAD0_TX0_header,
         input  [2:0]   QUAD0_TX0_margin,
         input          QUAD0_TX0_mldchaindone,
         input          QUAD0_TX0_mldchainreq,
         input          QUAD0_TX0_oneszeros,
         input          QUAD0_TX0_pausedelayalign,
         input          QUAD0_TX0_phalignreq,
         input  [1:0]   QUAD0_TX0_phalignresetmask,
         input          QUAD0_TX0_phdlypd,
         input          QUAD0_TX0_phdlyreset,
         input          QUAD0_TX0_phdlytstclk,
         input          QUAD0_TX0_phsetinitreq,
         input          QUAD0_TX0_phshift180,
         input          QUAD0_TX0_picodeovrden,
         input          QUAD0_TX0_picodereset,
         input          QUAD0_TX0_pippmen,
         input  [4:0]   QUAD0_TX0_pippmstepsize,
         input  [6:0]   QUAD0_TX0_sequence,
         input          QUAD0_TX0_swing,
         input          QUAD0_TX0_syncallin,
         input          QUAD0_TX0_inhibit,
         input          QUAD0_TX0_latclk,
         input  [6:0]   QUAD0_TX0_maincursor,
         input          QUAD0_TX0_pcsresetmask,
         input  [1:0]   QUAD0_TX0_pd,
         input          QUAD0_TX0_pisopd,
         input  [2:0]   QUAD0_TX0_pmaresetmask,
         input  [4:0]   QUAD0_TX0_postcursor,
         input  [4:0]   QUAD0_TX0_precursor,
         input          QUAD0_TX0_polarity,
         input          QUAD0_TX0_prbsforceerr,
         input  [3:0]   QUAD0_TX0_prbssel,
         input          QUAD0_TX0_progdivreset,
         input  [7:0]   QUAD0_TX0_rate,
         input  [1:0]   QUAD0_TX0_resetmode,
         input          QUAD0_TX0_userrdy,
         input          QUAD0_TX0_usrclk,
         //output         QUAD0_ch1_gtytxn,
         //output         QUAD0_ch1_gtytxp,
         //input          QUAD0_ch1_gtyrxn,
         //input          QUAD0_ch1_gtyrxp,
         //output         QUAD0_ch1_bufgtclr,
         //output         QUAD0_ch1_bufgtclrmask,
         output         QUAD0_RX1_cfokovrdrdy0,
         output         QUAD0_RX1_cfokovrdrdy1,
         input          QUAD0_RX1_cfokovrdfinish,
         input          QUAD0_RX1_cfokovrdpulse,
         input          QUAD0_RX1_cfokovrdstart,
         output         QUAD0_RX1_dccdone,
         output         QUAD0_RX1_osintstarted,
         output         QUAD0_RX1_osintstrobedone,
         output         QUAD0_RX1_osintstrobestarted,
         output [7:0]   QUAD0_RX1_dataextendrsvd,
         input  [7:0]   QUAD0_TX1_dataextendrsvd,
         output [2:0]   QUAD0_RX1_bufstatus,
         output         QUAD0_RX1_eyescandataerror,
         output [7:0]   QUAD0_RX1_10gstat,
         output         QUAD0_RX1_byteisaligned,
         output         QUAD0_RX1_byterealign,
         output         QUAD0_RX1_chanbondseq,
         output         QUAD0_RX1_chanisaligned,
         output         QUAD0_RX1_chanrealign,
         output [4:0]   QUAD0_RX1_chbondo,
         output [1:0]   QUAD0_RX1_clkcorcnt,
         output         QUAD0_RX1_cominitdet,
         output         QUAD0_RX1_commadet,
         output         QUAD0_RX1_comsasdet,
         output         QUAD0_RX1_comwakedet,
         output [15:0]  QUAD0_RX1_ctrl0,
         output [15:0]  QUAD0_RX1_ctrl1,
         output [7:0]   QUAD0_RX1_ctrl2,
         output [7:0]   QUAD0_RX1_ctrl3,
         output [1:0]   QUAD0_RX1_datavalid,
         output         QUAD0_RX1_dlyalignerr,
         output         QUAD0_RX1_dlyalignprog,
         output         QUAD0_RX1_elecidle,
         output         QUAD0_RX1_finealigndone,
         output [5:0]   QUAD0_RX1_header,
         output [1:0]   QUAD0_RX1_headervalid,
         output         QUAD0_RX1_osintdone,
         output         QUAD0_RX1_phaligndone,
         output         QUAD0_RX1_phalignerr,
         output         QUAD0_RX1_phdlyresetdone,
         output         QUAD0_RX1_phsetinitdone,
         output         QUAD0_RX1_phshift180done,
         output         QUAD0_RX1_sliderdy,
         output [1:0]   QUAD0_RX1_startofseq,
         output [2:0]   QUAD0_RX1_status,
         output         QUAD0_RX1_syncdone,
         output         QUAD0_RX1_valid,
         output         QUAD0_RX1_cdrlock,
         output         QUAD0_RX1_cdrphdone,
         output [127:0] QUAD0_RX1_data,
         output         QUAD0_RX1_outclk,
         output         QUAD0_RX1_pmaresetdone,
         output         QUAD0_RX1_prbserr,
         output         QUAD0_RX1_prbslocked,
         output         QUAD0_RX1_progdivresetdone,
         output         QUAD0_RX1_resetdone,

         output         QUAD0_RX1_mstresetdone,
         input          QUAD0_RX1_mstreset,
         input          QUAD0_RX1_mstdatapathreset,
         input          QUAD0_RX1_gtrxreset,
         input          QUAD0_RX1_cdrhold,
         input          QUAD0_RX1_cdrovrden,
         input          QUAD0_RX1_cdrreset,
         input          QUAD0_RX1_latclk,
         input          QUAD0_RX1_cdrfreqos,
         input          QUAD0_RX1_cdrincpctrl,
         input          QUAD0_RX1_cdrstepdir,
         input          QUAD0_RX1_cdrstepsq,
         input          QUAD0_RX1_cdrstepsx,
         input          QUAD0_RX1_eyescanreset,
         input          QUAD0_RX1_eyescantrigger,
         input  [4:0]   QUAD0_RX1_pcsresetmask,
         input          QUAD0_RX1_cdrbmcdrreq,
         input  [4:0]   QUAD0_RX1_chbondi,
         input          QUAD0_RX1_dapicodeovrden,
         input          QUAD0_RX1_dapicodereset,
         input          QUAD0_RX1_dlyalignreq,
         input          QUAD0_RX1_eqtraining,
         input          QUAD0_RX1_gearboxslip,
         input          QUAD0_RX1_lpmen,
         input          QUAD0_RX1_mldchaindone,
         input          QUAD0_RX1_mldchainreq,
         input          QUAD0_RX1_mlfinealignreq,
         input          QUAD0_RX1_oobreset,
         input          QUAD0_RX1_phalignreq,
         input  [1:0]   QUAD0_RX1_phalignresetmask,
         input          QUAD0_RX1_phdlypd,
         input          QUAD0_RX1_phdlyreset,
         input          QUAD0_RX1_phsetinitreq,
         input          QUAD0_RX1_phshift180,
         input          QUAD0_RX1_slide,
         input          QUAD0_RX1_syncallin,
         input          QUAD0_RX1_termination,
         input  [6:0]   QUAD0_RX1_pmaresetmask,
         input  [1:0]   QUAD0_RX1_pd,
         input          QUAD0_RX1_polarity,
         input          QUAD0_RX1_prbscntreset,
         input  [3:0]   QUAD0_RX1_prbssel,
         input          QUAD0_RX1_progdivreset,
         input  [7:0]   QUAD0_RX1_rate,
         input  [1:0]   QUAD0_RX1_resetmode,
         input          QUAD0_RX1_userrdy,
         input          QUAD0_RX1_usrclk,

         output         QUAD0_TX1_10gstat,
         output [1:0]   QUAD0_TX1_bufstatus,
         output         QUAD0_TX1_comfinish,
         output         QUAD0_TX1_dlyalignerr,
         output         QUAD0_TX1_dlyalignprog,
         output         QUAD0_TX1_phaligndone,
         output         QUAD0_TX1_phalignerr,
         output         QUAD0_TX1_phalignoutrsvd,
         output         QUAD0_TX1_phdlyresetdone,
         output         QUAD0_TX1_phsetinitdone,
         output         QUAD0_TX1_phshift180done,
         output         QUAD0_TX1_syncdone,
         output         QUAD0_TX1_dccdone,
         output         QUAD0_TX1_outclk,
         output         QUAD0_TX1_pmaresetdone,
         output         QUAD0_TX1_progdivresetdone,
         output         QUAD0_TX1_resetdone,
         output         QUAD0_TX1_mstresetdone,
         input          QUAD0_TX1_mstreset,
         input          QUAD0_TX1_mstdatapathreset,
         input          QUAD0_TX1_gttxreset,
         input  [127:0] QUAD0_TX1_data,
         input  [15:0]  QUAD0_TX1_ctrl0,
         input  [15:0]  QUAD0_TX1_ctrl1,
         input  [7:0]   QUAD0_TX1_ctrl2,
         input          QUAD0_TX1_cominit,
         input          QUAD0_TX1_comsas,
         input          QUAD0_TX1_comwake,
         input          QUAD0_TX1_dapicodeovrden,
         input          QUAD0_TX1_dapicodereset,
         input  [1:0]   QUAD0_TX1_deemph,
         input          QUAD0_TX1_detectrx,
         input  [4:0]   QUAD0_TX1_diffctrl,
         input          QUAD0_TX1_dlyalignreq,
         input          QUAD0_TX1_elecidle,
         input  [5:0]   QUAD0_TX1_header,
         input  [2:0]   QUAD0_TX1_margin,
         input          QUAD0_TX1_mldchaindone,
         input          QUAD0_TX1_mldchainreq,
         input          QUAD0_TX1_oneszeros,
         input          QUAD0_TX1_pausedelayalign,
         input          QUAD0_TX1_phalignreq,
         input  [1:0]   QUAD0_TX1_phalignresetmask,
         input          QUAD0_TX1_phdlypd,
         input          QUAD0_TX1_phdlyreset,
         input          QUAD0_TX1_phdlytstclk,
         input          QUAD0_TX1_phsetinitreq,
         input          QUAD0_TX1_phshift180,
         input          QUAD0_TX1_picodeovrden,
         input          QUAD0_TX1_picodereset,
         input          QUAD0_TX1_pippmen,
         input  [4:0]   QUAD0_TX1_pippmstepsize,
         input  [6:0]   QUAD0_TX1_sequence,
         input          QUAD0_TX1_swing,
         input          QUAD0_TX1_syncallin,
         input          QUAD0_TX1_inhibit,
         input          QUAD0_TX1_latclk,
         input  [6:0]   QUAD0_TX1_maincursor,
         input          QUAD0_TX1_pcsresetmask,
         input  [1:0]   QUAD0_TX1_pd,
         input          QUAD0_TX1_pisopd,
         input  [2:0]   QUAD0_TX1_pmaresetmask,
         input  [4:0]   QUAD0_TX1_postcursor,
         input  [4:0]   QUAD0_TX1_precursor,
         input          QUAD0_TX1_polarity,
         input          QUAD0_TX1_prbsforceerr,
         input  [3:0]   QUAD0_TX1_prbssel,
         input          QUAD0_TX1_progdivreset,
         input  [7:0]   QUAD0_TX1_rate,
         input  [1:0]   QUAD0_TX1_resetmode,
         input          QUAD0_TX1_userrdy,
         input          QUAD0_TX1_usrclk,
         //output         QUAD0_ch2_gtytxn,
         //output         QUAD0_ch2_gtytxp,
         //input          QUAD0_ch2_gtyrxn,
         //input          QUAD0_ch2_gtyrxp,
         //output         QUAD0_ch2_bufgtclr,
         //output         QUAD0_ch2_bufgtclrmask,
         output         QUAD0_RX2_cfokovrdrdy0,
         output         QUAD0_RX2_cfokovrdrdy1,
         input          QUAD0_RX2_cfokovrdfinish,
         input          QUAD0_RX2_cfokovrdpulse,
         input          QUAD0_RX2_cfokovrdstart,
         output         QUAD0_RX2_dccdone,
         output         QUAD0_RX2_osintstarted,
         output         QUAD0_RX2_osintstrobedone,
         output         QUAD0_RX2_osintstrobestarted,
         output [7:0]   QUAD0_RX2_dataextendrsvd,
         input  [7:0]   QUAD0_TX2_dataextendrsvd,
         output [2:0]   QUAD0_RX2_bufstatus,
         output         QUAD0_RX2_eyescandataerror,
         output [7:0]   QUAD0_RX2_10gstat,
         output         QUAD0_RX2_byteisaligned,
         output         QUAD0_RX2_byterealign,
         output         QUAD0_RX2_chanbondseq,
         output         QUAD0_RX2_chanisaligned,
         output         QUAD0_RX2_chanrealign,
         output [4:0]   QUAD0_RX2_chbondo,
         output [1:0]   QUAD0_RX2_clkcorcnt,
         output         QUAD0_RX2_cominitdet,
         output         QUAD0_RX2_commadet,
         output         QUAD0_RX2_comsasdet,
         output         QUAD0_RX2_comwakedet,
         output [15:0]  QUAD0_RX2_ctrl0,
         output [15:0]  QUAD0_RX2_ctrl1,
         output [7:0]   QUAD0_RX2_ctrl2,
         output [7:0]   QUAD0_RX2_ctrl3,
         output [1:0]   QUAD0_RX2_datavalid,
         output         QUAD0_RX2_dlyalignerr,
         output         QUAD0_RX2_dlyalignprog,
         output         QUAD0_RX2_elecidle,
         output         QUAD0_RX2_finealigndone,
         output [5:0]   QUAD0_RX2_header,
         output [1:0]   QUAD0_RX2_headervalid,
         output         QUAD0_RX2_osintdone,
         output         QUAD0_RX2_phaligndone,
         output         QUAD0_RX2_phalignerr,
         output         QUAD0_RX2_phdlyresetdone,
         output         QUAD0_RX2_phsetinitdone,
         output         QUAD0_RX2_phshift180done,
         output         QUAD0_RX2_sliderdy,
         output [1:0]   QUAD0_RX2_startofseq,
         output [2:0]   QUAD0_RX2_status,
         output         QUAD0_RX2_syncdone,
         output         QUAD0_RX2_valid,
         output         QUAD0_RX2_cdrlock,
         output         QUAD0_RX2_cdrphdone,
         output [127:0] QUAD0_RX2_data,
         output         QUAD0_RX2_outclk,
         output         QUAD0_RX2_pmaresetdone,
         output         QUAD0_RX2_prbserr,
         output         QUAD0_RX2_prbslocked,
         output         QUAD0_RX2_progdivresetdone,
         output         QUAD0_RX2_resetdone,

         output         QUAD0_RX2_mstresetdone,
         input          QUAD0_RX2_mstreset,
         input          QUAD0_RX2_mstdatapathreset,
         input          QUAD0_RX2_gtrxreset,
         input          QUAD0_RX2_cdrhold,
         input          QUAD0_RX2_cdrovrden,
         input          QUAD0_RX2_cdrreset,
         input          QUAD0_RX2_latclk,
         input          QUAD0_RX2_cdrfreqos,
         input          QUAD0_RX2_cdrincpctrl,
         input          QUAD0_RX2_cdrstepdir,
         input          QUAD0_RX2_cdrstepsq,
         input          QUAD0_RX2_cdrstepsx,
         input          QUAD0_RX2_eyescanreset,
         input          QUAD0_RX2_eyescantrigger,
         input  [4:0]   QUAD0_RX2_pcsresetmask,
         input          QUAD0_RX2_cdrbmcdrreq,
         input  [4:0]   QUAD0_RX2_chbondi,
         input          QUAD0_RX2_dapicodeovrden,
         input          QUAD0_RX2_dapicodereset,
         input          QUAD0_RX2_dlyalignreq,
         input          QUAD0_RX2_eqtraining,
         input          QUAD0_RX2_gearboxslip,
         input          QUAD0_RX2_lpmen,
         input          QUAD0_RX2_mldchaindone,
         input          QUAD0_RX2_mldchainreq,
         input          QUAD0_RX2_mlfinealignreq,
         input          QUAD0_RX2_oobreset,
         input          QUAD0_RX2_phalignreq,
         input  [1:0]   QUAD0_RX2_phalignresetmask,
         input          QUAD0_RX2_phdlypd,
         input          QUAD0_RX2_phdlyreset,
         input          QUAD0_RX2_phsetinitreq,
         input          QUAD0_RX2_phshift180,
         input          QUAD0_RX2_slide,
         input          QUAD0_RX2_syncallin,
         input          QUAD0_RX2_termination,
         input  [6:0]   QUAD0_RX2_pmaresetmask,
         input  [1:0]   QUAD0_RX2_pd,
         input          QUAD0_RX2_polarity,
         input          QUAD0_RX2_prbscntreset,
         input  [3:0]   QUAD0_RX2_prbssel,
         input          QUAD0_RX2_progdivreset,
         input  [7:0]   QUAD0_RX2_rate,
         input  [1:0]   QUAD0_RX2_resetmode,
         input          QUAD0_RX2_userrdy,
         input          QUAD0_RX2_usrclk,

         output         QUAD0_TX2_10gstat,
         output [1:0]   QUAD0_TX2_bufstatus,
         output         QUAD0_TX2_comfinish,
         output         QUAD0_TX2_dlyalignerr,
         output         QUAD0_TX2_dlyalignprog,
         output         QUAD0_TX2_phaligndone,
         output         QUAD0_TX2_phalignerr,
         output         QUAD0_TX2_phalignoutrsvd,
         output         QUAD0_TX2_phdlyresetdone,
         output         QUAD0_TX2_phsetinitdone,
         output         QUAD0_TX2_phshift180done,
         output         QUAD0_TX2_syncdone,
         output         QUAD0_TX2_dccdone,
         output         QUAD0_TX2_outclk,
         output         QUAD0_TX2_pmaresetdone,
         output         QUAD0_TX2_progdivresetdone,
         output         QUAD0_TX2_resetdone,
         output         QUAD0_TX2_mstresetdone,
         input          QUAD0_TX2_mstreset,
         input          QUAD0_TX2_mstdatapathreset,
         input          QUAD0_TX2_gttxreset,
         input  [127:0] QUAD0_TX2_data,
         input  [15:0]  QUAD0_TX2_ctrl0,
         input  [15:0]  QUAD0_TX2_ctrl1,
         input  [7:0]   QUAD0_TX2_ctrl2,
         input          QUAD0_TX2_cominit,
         input          QUAD0_TX2_comsas,
         input          QUAD0_TX2_comwake,
         input          QUAD0_TX2_dapicodeovrden,
         input          QUAD0_TX2_dapicodereset,
         input  [1:0]   QUAD0_TX2_deemph,
         input          QUAD0_TX2_detectrx,
         input  [4:0]   QUAD0_TX2_diffctrl,
         input          QUAD0_TX2_dlyalignreq,
         input          QUAD0_TX2_elecidle,
         input  [5:0]   QUAD0_TX2_header,
         input  [2:0]   QUAD0_TX2_margin,
         input          QUAD0_TX2_mldchaindone,
         input          QUAD0_TX2_mldchainreq,
         input          QUAD0_TX2_oneszeros,
         input          QUAD0_TX2_pausedelayalign,
         input          QUAD0_TX2_phalignreq,
         input  [1:0]   QUAD0_TX2_phalignresetmask,
         input          QUAD0_TX2_phdlypd,
         input          QUAD0_TX2_phdlyreset,
         input          QUAD0_TX2_phdlytstclk,
         input          QUAD0_TX2_phsetinitreq,
         input          QUAD0_TX2_phshift180,
         input          QUAD0_TX2_picodeovrden,
         input          QUAD0_TX2_picodereset,
         input          QUAD0_TX2_pippmen,
         input  [4:0]   QUAD0_TX2_pippmstepsize,
         input  [6:0]   QUAD0_TX2_sequence,
         input          QUAD0_TX2_swing,
         input          QUAD0_TX2_syncallin,
         input          QUAD0_TX2_inhibit,
         input          QUAD0_TX2_latclk,
         input  [6:0]   QUAD0_TX2_maincursor,
         input          QUAD0_TX2_pcsresetmask,
         input  [1:0]   QUAD0_TX2_pd,
         input          QUAD0_TX2_pisopd,
         input  [2:0]   QUAD0_TX2_pmaresetmask,
         input  [4:0]   QUAD0_TX2_postcursor,
         input  [4:0]   QUAD0_TX2_precursor,
         input          QUAD0_TX2_polarity,
         input          QUAD0_TX2_prbsforceerr,
         input  [3:0]   QUAD0_TX2_prbssel,
         input          QUAD0_TX2_progdivreset,
         input  [7:0]   QUAD0_TX2_rate,
         input  [1:0]   QUAD0_TX2_resetmode,
         input          QUAD0_TX2_userrdy,
         input          QUAD0_TX2_usrclk,
         //output         QUAD0_ch3_gtytxn,
         //output         QUAD0_ch3_gtytxp,
         //input          QUAD0_ch3_gtyrxn,
         //input          QUAD0_ch3_gtyrxp,
         //output         QUAD0_ch3_bufgtclr,
         //output         QUAD0_ch3_bufgtclrmask,
         output         QUAD0_RX3_cfokovrdrdy0,
         output         QUAD0_RX3_cfokovrdrdy1,
         input          QUAD0_RX3_cfokovrdfinish,
         input          QUAD0_RX3_cfokovrdpulse,
         input          QUAD0_RX3_cfokovrdstart,
         output         QUAD0_RX3_dccdone,
         output         QUAD0_RX3_osintstarted,
         output         QUAD0_RX3_osintstrobedone,
         output         QUAD0_RX3_osintstrobestarted,
         output [7:0]   QUAD0_RX3_dataextendrsvd,
         input  [7:0]   QUAD0_TX3_dataextendrsvd,
         output [2:0]   QUAD0_RX3_bufstatus,
         output         QUAD0_RX3_eyescandataerror,
         output [7:0]   QUAD0_RX3_10gstat,
         output         QUAD0_RX3_byteisaligned,
         output         QUAD0_RX3_byterealign,
         output         QUAD0_RX3_chanbondseq,
         output         QUAD0_RX3_chanisaligned,
         output         QUAD0_RX3_chanrealign,
         output [4:0]   QUAD0_RX3_chbondo,
         output [1:0]   QUAD0_RX3_clkcorcnt,
         output         QUAD0_RX3_cominitdet,
         output         QUAD0_RX3_commadet,
         output         QUAD0_RX3_comsasdet,
         output         QUAD0_RX3_comwakedet,
         output [15:0]  QUAD0_RX3_ctrl0,
         output [15:0]  QUAD0_RX3_ctrl1,
         output [7:0]   QUAD0_RX3_ctrl2,
         output [7:0]   QUAD0_RX3_ctrl3,
         output [1:0]   QUAD0_RX3_datavalid,
         output         QUAD0_RX3_dlyalignerr,
         output         QUAD0_RX3_dlyalignprog,
         output         QUAD0_RX3_elecidle,
         output         QUAD0_RX3_finealigndone,
         output [5:0]   QUAD0_RX3_header,
         output [1:0]   QUAD0_RX3_headervalid,
         output         QUAD0_RX3_osintdone,
         output         QUAD0_RX3_phaligndone,
         output         QUAD0_RX3_phalignerr,
         output         QUAD0_RX3_phdlyresetdone,
         output         QUAD0_RX3_phsetinitdone,
         output         QUAD0_RX3_phshift180done,
         output         QUAD0_RX3_sliderdy,
         output [1:0]   QUAD0_RX3_startofseq,
         output [2:0]   QUAD0_RX3_status,
         output         QUAD0_RX3_syncdone,
         output         QUAD0_RX3_valid,
         output         QUAD0_RX3_cdrlock,
         output         QUAD0_RX3_cdrphdone,
         output [127:0] QUAD0_RX3_data,
         output         QUAD0_RX3_outclk,
         output         QUAD0_RX3_pmaresetdone,
         output         QUAD0_RX3_prbserr,
         output         QUAD0_RX3_prbslocked,
         output         QUAD0_RX3_progdivresetdone,
         output         QUAD0_RX3_resetdone,

         output         QUAD0_RX3_mstresetdone,
         input          QUAD0_RX3_mstreset,
         input          QUAD0_RX3_mstdatapathreset,
         input          QUAD0_RX3_gtrxreset,
         input          QUAD0_RX3_cdrhold,
         input          QUAD0_RX3_cdrovrden,
         input          QUAD0_RX3_cdrreset,
         input          QUAD0_RX3_latclk,
         input          QUAD0_RX3_cdrfreqos,
         input          QUAD0_RX3_cdrincpctrl,
         input          QUAD0_RX3_cdrstepdir,
         input          QUAD0_RX3_cdrstepsq,
         input          QUAD0_RX3_cdrstepsx,
         input          QUAD0_RX3_eyescanreset,
         input          QUAD0_RX3_eyescantrigger,
         input  [4:0]   QUAD0_RX3_pcsresetmask,
         input          QUAD0_RX3_cdrbmcdrreq,
         input  [4:0]   QUAD0_RX3_chbondi,
         input          QUAD0_RX3_dapicodeovrden,
         input          QUAD0_RX3_dapicodereset,
         input          QUAD0_RX3_dlyalignreq,
         input          QUAD0_RX3_eqtraining,
         input          QUAD0_RX3_gearboxslip,
         input          QUAD0_RX3_lpmen,
         input          QUAD0_RX3_mldchaindone,
         input          QUAD0_RX3_mldchainreq,
         input          QUAD0_RX3_mlfinealignreq,
         input          QUAD0_RX3_oobreset,
         input          QUAD0_RX3_phalignreq,
         input  [1:0]   QUAD0_RX3_phalignresetmask,
         input          QUAD0_RX3_phdlypd,
         input          QUAD0_RX3_phdlyreset,
         input          QUAD0_RX3_phsetinitreq,
         input          QUAD0_RX3_phshift180,
         input          QUAD0_RX3_slide,
         input          QUAD0_RX3_syncallin,
         input          QUAD0_RX3_termination,
         input  [6:0]   QUAD0_RX3_pmaresetmask,
         input  [1:0]   QUAD0_RX3_pd,
         input          QUAD0_RX3_polarity,
         input          QUAD0_RX3_prbscntreset,
         input  [3:0]   QUAD0_RX3_prbssel,
         input          QUAD0_RX3_progdivreset,
         input  [7:0]   QUAD0_RX3_rate,
         input  [1:0]   QUAD0_RX3_resetmode,
         input          QUAD0_RX3_userrdy,
         input          QUAD0_RX3_usrclk,

         output         QUAD0_TX3_10gstat,
         output [1:0]   QUAD0_TX3_bufstatus,
         output         QUAD0_TX3_comfinish,
         output         QUAD0_TX3_dlyalignerr,
         output         QUAD0_TX3_dlyalignprog,
         output         QUAD0_TX3_phaligndone,
         output         QUAD0_TX3_phalignerr,
         output         QUAD0_TX3_phalignoutrsvd,
         output         QUAD0_TX3_phdlyresetdone,
         output         QUAD0_TX3_phsetinitdone,
         output         QUAD0_TX3_phshift180done,
         output         QUAD0_TX3_syncdone,
         output         QUAD0_TX3_dccdone,
         output         QUAD0_TX3_outclk,
         output         QUAD0_TX3_pmaresetdone,
         output         QUAD0_TX3_progdivresetdone,
         output         QUAD0_TX3_resetdone,
         output         QUAD0_TX3_mstresetdone,
         input          QUAD0_TX3_mstreset,
         input          QUAD0_TX3_mstdatapathreset,
         input          QUAD0_TX3_gttxreset,
         input  [127:0] QUAD0_TX3_data,
         input  [15:0]  QUAD0_TX3_ctrl0,
         input  [15:0]  QUAD0_TX3_ctrl1,
         input  [7:0]   QUAD0_TX3_ctrl2,
         input          QUAD0_TX3_cominit,
         input          QUAD0_TX3_comsas,
         input          QUAD0_TX3_comwake,
         input          QUAD0_TX3_dapicodeovrden,
         input          QUAD0_TX3_dapicodereset,
         input  [1:0]   QUAD0_TX3_deemph,
         input          QUAD0_TX3_detectrx,
         input  [4:0]   QUAD0_TX3_diffctrl,
         input          QUAD0_TX3_dlyalignreq,
         input          QUAD0_TX3_elecidle,
         input  [5:0]   QUAD0_TX3_header,
         input  [2:0]   QUAD0_TX3_margin,
         input          QUAD0_TX3_mldchaindone,
         input          QUAD0_TX3_mldchainreq,
         input          QUAD0_TX3_oneszeros,
         input          QUAD0_TX3_pausedelayalign,
         input          QUAD0_TX3_phalignreq,
         input  [1:0]   QUAD0_TX3_phalignresetmask,
         input          QUAD0_TX3_phdlypd,
         input          QUAD0_TX3_phdlyreset,
         input          QUAD0_TX3_phdlytstclk,
         input          QUAD0_TX3_phsetinitreq,
         input          QUAD0_TX3_phshift180,
         input          QUAD0_TX3_picodeovrden,
         input          QUAD0_TX3_picodereset,
         input          QUAD0_TX3_pippmen,
         input  [4:0]   QUAD0_TX3_pippmstepsize,
         input  [6:0]   QUAD0_TX3_sequence,
         input          QUAD0_TX3_swing,
         input          QUAD0_TX3_syncallin,
         input          QUAD0_TX3_inhibit,
         input          QUAD0_TX3_latclk,
         input  [6:0]   QUAD0_TX3_maincursor,
         input          QUAD0_TX3_pcsresetmask,
         input  [1:0]   QUAD0_TX3_pd,
         input          QUAD0_TX3_pisopd,
         input  [2:0]   QUAD0_TX3_pmaresetmask,
         input  [4:0]   QUAD0_TX3_postcursor,
         input  [4:0]   QUAD0_TX3_precursor,
         input          QUAD0_TX3_polarity,
         input          QUAD0_TX3_prbsforceerr,
         input  [3:0]   QUAD0_TX3_prbssel,
         input          QUAD0_TX3_progdivreset,
         input  [7:0]   QUAD0_TX3_rate,
         input  [1:0]   QUAD0_TX3_resetmode,
         input          QUAD0_TX3_userrdy,
         input          QUAD0_TX3_usrclk,

         output         QUAD0_gtpowergood

);

/*       input  */ wire           QUAD_altclk = 1'b0;
//Northout_Southin_intf for last quad
/*       output */ wire [1:0]   QUAD0_resetdone_northout;
/*       input  */ wire [1:0]   QUAD0_resetdone_southin = 2'b0;
/*       output */ wire [3:0]   QUAD0_txpinorthout;
/*       output */ wire [3:0]   QUAD0_rxpinorthout;
/*       input  */ wire [3:0]   QUAD0_txpisouthin = 4'b0;
/*       input  */ wire [3:0]   QUAD0_rxpisouthin = 4'b0;
/*       output */ wire [5:0]   QUAD0_pipenorthout;
/*       input  */ wire [5:0]   QUAD0_pipesouthin = 6'b0;
//Northin_Southout_intf for first quad
/*       input  */ wire [1:0]   QUAD0_resetdone_northin = 2'b0;
/*       output */ wire [1:0]   QUAD0_resetdone_southout;
/*       input  */ wire [3:0]   QUAD0_txpinorthin = 4'b0;
/*       input  */ wire [3:0]   QUAD0_rxpinorthin = 4'b0;
/*       output */ wire [3:0]   QUAD0_txpisouthout;
/*       output */ wire [3:0]   QUAD0_rxpisouthout;
/*       input  */ wire [5:0]   QUAD0_pipenorthin = 6'b0;
/*       output */ wire [5:0]   QUAD0_pipesouthout;





my_trx_gt_quad_base_0 gt_quad_base_0_inst (
         .GT_REFCLK0          ( QUAD0_GTREFCLK0) ,
         .GT_REFCLK1          ( QUAD0_GTREFCLK1) ,
         .altclk                      ( QUAD_altclk) ,
         .rxp                         ( QUAD0_rxp) ,
         .rxn                         ( QUAD0_rxn) ,
         .txp                         ( QUAD0_txp) ,
         .txn                         ( QUAD0_txn) ,
         .apb3clk                     ( gtwiz_freerun_clk) ,
         .apb3presetn                 ( 1'b1) ,
         .apb3paddr                   ( 16'b0),
         .apb3penable                 ( 1'b0),
         .apb3psel                    ( 1'b0),
         .apb3pwdata                  ( 32'b0),
         .apb3pwrite                  ( 1'b0),
         .apb3prdata                  ( ),
         .apb3pslverr                 ( ),
         .apb3pready                  ( ),

         .rxmarginreqack              ( QUAD0_rxmarginreqack) ,
         .rxmarginrescmd              ( QUAD0_rxmarginrescmd) ,
         .rxmarginreslanenum          ( QUAD0_rxmarginreslanenum) ,
         .rxmarginrespayld            ( QUAD0_rxmarginrespayld) ,
         .rxmarginresreq              ( QUAD0_rxmarginresreq) ,
         .rxmarginreqcmd              ( QUAD0_rxmarginreqcmd) ,
         .rxmarginreqlanenum          ( QUAD0_rxmarginreqlanenum) ,
         .rxmarginreqpayld            ( QUAD0_rxmarginreqpayld) ,
         .rxmarginreqreq              ( QUAD0_rxmarginreqreq) ,
         .rxmarginclk                 ( QUAD0_rxmarginclk) ,
         .rxmarginresack              ( QUAD0_rxmarginresack) ,
         .pcielinkreachtarget         ( QUAD0_pcielinkreachtarget) ,
         .pcieltssm                   ( QUAD0_pcieltssm) ,
         .bgbypassb                   ( QUAD0_bgbypassb) ,
         .bgmonitorenb                ( QUAD0_bgmonitorenb) ,
         .bgpdb                       ( QUAD0_bgpdb) ,
         .bgrcalovrd                  ( QUAD0_bgrcalovrd) ,
         .bgrcalovrdenb               ( QUAD0_bgrcalovrdenb) ,
         .debugtraceready             ( QUAD0_debugtraceready) ,
         .debugtraceclk               ( QUAD0_debugtraceclk) ,
         .rcalenb                     ( QUAD0_rcalenb) ,
         .trigackout0                 ( QUAD0_trigackout0) ,
         .trigin0                     ( QUAD0_trigin0) ,
         .ubenable                    ( QUAD0_ubenable) ,
         .ubintr                      ( QUAD0_ubintr) ,
         .ubiolmbrst                  ( QUAD0_ubiolmbrst) ,
         .ubmbrst                     ( QUAD0_ubmbrst) ,
         .ubrxuart                    ( QUAD0_ubrxuart) ,
         .ctrlrsvdin0                 ( QUAD0_ctrlrsvdin0) ,
         .ctrlrsvdin1                 ( QUAD0_ctrlrsvdin1) ,
         .gpi                         ( QUAD0_gpi) ,
         .refclk0_clktestsig          ( QUAD0_refclk0_clktestsig) ,
         .refclk1_clktestsig          ( QUAD0_refclk1_clktestsig) ,
         .correcterr                  ( QUAD0_correcterr) ,
         .debugtracetdata             ( QUAD0_debugtracetdata) ,
         .debugtracetvalid            ( QUAD0_debugtracetvalid) ,
         .refclk0_gtrefclkpdint       ( QUAD0_refclk0_gtrefclkpdint) ,
         .refclk0_clktestsigint       ( QUAD0_refclk0_clktestsigint) ,
         .refclk1_gtrefclkpdint       ( QUAD0_refclk1_gtrefclkpdint) ,
         .refclk1_clktestsigint       ( QUAD0_refclk1_clktestsigint) ,
         .trigackin0                  ( QUAD0_trigackin0) ,
         .trigout0                    ( QUAD0_trigout0) ,
         .ubinterrupt                 ( QUAD0_ubinterrupt) ,
         .ubtxuart                    ( QUAD0_ubtxuart) ,
         .uncorrecterr                ( QUAD0_uncorrecterr) ,
         .ctrlrsvdout                 ( QUAD0_ctrlrsvdout) ,
         .gpo                         ( QUAD0_gpo) ,
         .resetdone_northin           ( QUAD0_resetdone_northin) ,
         .resetdone_southin           ( QUAD0_resetdone_southin) ,
         .resetdone_northout          ( QUAD0_resetdone_northout) ,
         .resetdone_southout          ( QUAD0_resetdone_southout) ,
         .txpinorthout                ( QUAD0_txpinorthout) ,
         .txpisouthout                ( QUAD0_txpisouthout) ,
         .rxpinorthout                ( QUAD0_rxpinorthout) ,
         .rxpisouthout                ( QUAD0_rxpisouthout) ,
         .pipenorthout                ( QUAD0_pipenorthout) ,
         .pipesouthout                ( QUAD0_pipesouthout) ,
         .pipenorthin                 ( QUAD0_pipenorthin) ,
         .pipesouthin                 ( QUAD0_pipesouthin) ,
         .rxpinorthin                 ( QUAD0_rxpinorthin) ,
         .rxpisouthin                 ( QUAD0_rxpisouthin) ,
         .txpinorthin                 ( QUAD0_txpinorthin) ,
         .txpisouthin                 ( QUAD0_txpisouthin) ,
         .hsclk0_rxrecclksel          ( QUAD0_hsclk0_rxrecclksel) ,
         .hsclk1_rxrecclksel          ( QUAD0_hsclk1_rxrecclksel) ,
         .refclk0_gtrefclkpd          ( QUAD0_refclk0_gtrefclkpd) ,
         .hsclk0_rpllreset            ( QUAD0_hsclk0_rpllreset) ,
         .hsclk0_lcpllreset           ( QUAD0_hsclk0_lcpllreset) ,
         .hsclk0_lcpllfbclklost       ( QUAD0_hsclk0_lcpllfbclklost) ,
         .hsclk0_lcplllock            ( QUAD0_hsclk0_lcplllock) ,
         .hsclk0_lcpllrefclklost      ( QUAD0_hsclk0_lcpllrefclklost) ,
         .hsclk0_lcpllrefclkmonitor   ( QUAD0_hsclk0_lcpllrefclkmonitor) ,
         .hsclk0_lcpllclkrsvd0        ( QUAD0_hsclk0_lcpllclkrsvd0),
         .hsclk0_lcpllclkrsvd1        ( QUAD0_hsclk0_lcpllclkrsvd1),
         .hsclk0_lcpllrsvd0           ( QUAD0_hsclk0_lcpllrsvd0),
         .hsclk0_lcpllrsvd1           ( QUAD0_hsclk0_lcpllrsvd1),
         .hsclk0_lcpllrsvdout         ( QUAD0_hsclk0_lcpllrsvdout) ,
         .hsclk0_lcpllfbdiv           ( QUAD0_hsclk0_lcpllfbdiv) ,
//         .hsclk0_lcpllgtgrefclk       ( QUAD0_hsclk0_lcpllgtgrefclk),
         .hsclk0_lcpllpd              ( QUAD0_hsclk0_lcpllpd) ,
         .hsclk0_lcpllrefclksel       ( QUAD0_hsclk0_lcpllrefclksel) ,
         .hsclk0_lcpllresetbypassmode ( QUAD0_hsclk0_lcpllresetbypassmode) ,
         .hsclk0_lcpllresetmask       ( QUAD0_hsclk0_lcpllresetmask) ,
         .hsclk0_lcpllsdmdata         ( QUAD0_hsclk0_lcpllsdmdata) ,
         .hsclk0_lcpllsdmtoggle       ( QUAD0_hsclk0_lcpllsdmtoggle) ,
         .hsclk0_rpllfbclklost        ( QUAD0_hsclk0_rpllfbclklost) ,
         .hsclk0_rplllock             ( QUAD0_hsclk0_rplllock) ,
         .hsclk0_rpllrefclklost       ( QUAD0_hsclk0_rpllrefclklost) ,
         .hsclk0_rpllrefclkmonitor    ( QUAD0_hsclk0_rpllrefclkmonitor) ,
         .hsclk0_rpllclkrsvd0         ( QUAD0_hsclk0_rpllclkrsvd0),
         .hsclk0_rpllclkrsvd1         ( QUAD0_hsclk0_rpllclkrsvd1),
         .hsclk0_rpllrsvd0            ( QUAD0_hsclk0_rpllrsvd0) ,
         .hsclk0_rpllrsvd1            ( QUAD0_hsclk0_rpllrsvd1) ,
         .hsclk0_rpllrsvdout          ( QUAD0_hsclk0_rpllrsvdout) ,
         .hsclk0_rpllfbdiv            ( QUAD0_hsclk0_rpllfbdiv) ,
//         .hsclk0_rpllgtgrefclk        ( QUAD0_hsclk0_rpllgtgrefclk),
         .hsclk0_rpllpd               ( QUAD0_hsclk0_rpllpd) ,
         .hsclk0_rpllrefclksel        ( QUAD0_hsclk0_rpllrefclksel) ,
         .hsclk0_rpllresetbypassmode  ( QUAD0_hsclk0_rpllresetbypassmode) ,
         .hsclk0_rpllresetmask        ( QUAD0_hsclk0_rpllresetmask) ,
         .hsclk0_rpllsdmdata          ( QUAD0_hsclk0_rpllsdmdata) ,
         .hsclk0_rpllsdmtoggle        ( QUAD0_hsclk0_rpllsdmtoggle) ,
         .hsclk0_rxrecclkout0         ( QUAD0_hsclk0_rxrecclkout0) ,
         .hsclk0_rxrecclkout1         ( QUAD0_hsclk0_rxrecclkout1) ,
         .refclk1_gtrefclkpd          ( QUAD0_refclk1_gtrefclkpd) ,
         .hsclk1_rpllreset            ( QUAD0_hsclk1_rpllreset) ,
         .hsclk1_lcpllreset           ( QUAD0_hsclk1_lcpllreset) ,
         .hsclk1_lcpllfbclklost       ( QUAD0_hsclk1_lcpllfbclklost) ,
         .hsclk1_lcplllock            ( QUAD0_hsclk1_lcplllock) ,
         .hsclk1_lcpllrefclklost      ( QUAD0_hsclk1_lcpllrefclklost) ,
         .hsclk1_lcpllrefclkmonitor   ( QUAD0_hsclk1_lcpllrefclkmonitor) ,
         .hsclk1_lcpllclkrsvd0        ( QUAD0_hsclk1_lcpllclkrsvd0),
         .hsclk1_lcpllclkrsvd1        ( QUAD0_hsclk1_lcpllclkrsvd1),
         .hsclk1_lcpllrsvd0           ( QUAD0_hsclk1_lcpllrsvd0),
         .hsclk1_lcpllrsvd1           ( QUAD0_hsclk1_lcpllrsvd1),
         .hsclk1_lcpllrsvdout         ( QUAD0_hsclk1_lcpllrsvdout) ,
         .hsclk1_lcpllfbdiv           ( QUAD0_hsclk1_lcpllfbdiv) ,
//         .hsclk1_lcpllgtgrefclk       ( QUAD0_hsclk1_lcpllgtgrefclk),
         .hsclk1_lcpllpd              ( QUAD0_hsclk1_lcpllpd) ,
         .hsclk1_lcpllrefclksel       ( QUAD0_hsclk1_lcpllrefclksel) ,
         .hsclk1_lcpllresetbypassmode ( QUAD0_hsclk1_lcpllresetbypassmode) ,
         .hsclk1_lcpllresetmask       ( QUAD0_hsclk1_lcpllresetmask) ,
         .hsclk1_lcpllsdmdata         ( QUAD0_hsclk1_lcpllsdmdata) ,
         .hsclk1_lcpllsdmtoggle       ( QUAD0_hsclk1_lcpllsdmtoggle) ,
         .hsclk1_rpllfbclklost        ( QUAD0_hsclk1_rpllfbclklost) ,
         .hsclk1_rplllock             ( QUAD0_hsclk1_rplllock) ,
         .hsclk1_rpllrefclklost       ( QUAD0_hsclk1_rpllrefclklost) ,
         .hsclk1_rpllrefclkmonitor    ( QUAD0_hsclk1_rpllrefclkmonitor) ,
         .hsclk1_rpllclkrsvd0         ( QUAD0_hsclk1_rpllclkrsvd0),
         .hsclk1_rpllclkrsvd1         ( QUAD0_hsclk1_rpllclkrsvd1),
         .hsclk1_rpllrsvd0            ( QUAD0_hsclk1_rpllrsvd0) ,
         .hsclk1_rpllrsvd1            ( QUAD0_hsclk1_rpllrsvd1) ,
         .hsclk1_rpllrsvdout          ( QUAD0_hsclk1_rpllrsvdout) ,
         .hsclk1_rpllfbdiv            ( QUAD0_hsclk1_rpllfbdiv) ,
//         .hsclk1_rpllgtgrefclk        ( QUAD0_hsclk1_rpllgtgrefclk),
         .hsclk1_rpllpd               ( QUAD0_hsclk1_rpllpd) ,
         .hsclk1_rpllrefclksel        ( QUAD0_hsclk1_rpllrefclksel) ,
         .hsclk1_rpllresetbypassmode  ( QUAD0_hsclk1_rpllresetbypassmode) ,
         .hsclk1_rpllresetmask        ( QUAD0_hsclk1_rpllresetmask) ,
         .hsclk1_rpllsdmdata          ( QUAD0_hsclk1_rpllsdmdata) ,
         .hsclk1_rpllsdmtoggle        ( QUAD0_hsclk1_rpllsdmtoggle) ,
         .hsclk1_rxrecclkout0         ( QUAD0_hsclk1_rxrecclkout0) ,
         .hsclk1_rxrecclkout1         ( QUAD0_hsclk1_rxrecclkout1) ,
         .ch0_bufgtce              ( QUAD0_ch0_bufgtce),
         .ch0_bufgtcemask          ( QUAD0_ch0_bufgtcemask),
         .ch0_bufgtdiv             ( QUAD0_ch0_bufgtdiv),
         .ch0_bufgtrst             ( QUAD0_ch0_bufgtrst),
         .ch0_bufgtrstmask         ( QUAD0_ch0_bufgtrstmask),
         //.ch0_gtytxn ( ),
         //.ch0_gtytxp ( ),
         .ch0_iloreset             ( QUAD0_ch0_iloreset),
         .ch0_iloresetdone         ( QUAD0_ch0_iloresetdone),
         .ch0_phystatus            ( QUAD0_ch0_phystatus),
         .ch0_clkrsvd0             ( QUAD0_ch0_clkrsvd0),
         .ch0_clkrsvd1             ( QUAD0_ch0_clkrsvd1),
         .ch0_dmonitorclk          ( QUAD0_ch0_dmonitorclk),
         .ch0_phyesmadaptsave      ( QUAD0_ch0_phyesmadaptsave),
         .ch0_iloresetmask         ( QUAD0_ch0_iloresetmask),
         .ch0_loopback             ( QUAD0_ch0_loopback),
         .ch0_dmonfiforeset        ( QUAD0_ch0_dmonfiforeset),
         .ch0_pcsrsvdin            ( QUAD0_ch0_pcsrsvdin),
         .ch0_gtrsvd               ( QUAD0_ch0_gtrsvd),
         .ch0_tstin                ( QUAD0_ch0_tstin),
         .ch0_pcsrsvdout           ( QUAD0_ch0_pcsrsvdout),
         .ch0_pinrsvdas            ( QUAD0_ch0_pinrsvdas),
         .ch0_dmonitoroutclk       ( QUAD0_ch0_dmonitoroutclk),
         .ch0_resetexception       ( QUAD0_ch0_resetexception),
         .ch0_dmonitorout          ( QUAD0_ch0_dmonitorout),
         .ch0_phyready             ( QUAD0_ch0_phyready),
         .ch0_hsdppcsreset         ( QUAD0_ch0_hsdppcsreset),
         .ch0_pcierstb             ( QUAD0_ch0_pcierstb),

         .ch0_eyescandataerror     ( QUAD0_RX0_eyescandataerror), 
         .ch0_cdrbmcdrreq          ( QUAD0_RX0_cdrbmcdrreq),
         .ch0_cdrfreqos            ( QUAD0_RX0_cdrfreqos),
         .ch0_cdrincpctrl          ( QUAD0_RX0_cdrincpctrl),
         .ch0_cdrstepdir           ( QUAD0_RX0_cdrstepdir),
         .ch0_cdrstepsq            ( QUAD0_RX0_cdrstepsq),
         .ch0_cdrstepsx            ( QUAD0_RX0_cdrstepsx),
         .ch0_eyescanreset         ( QUAD0_RX0_eyescanreset),
         .ch0_eyescantrigger       ( QUAD0_RX0_eyescantrigger),
         .ch0_gtrxreset            ( QUAD0_RX0_gtrxreset),
         .ch0_gttxreset            ( QUAD0_TX0_gttxreset),
         //.ch0_gtyrxn               ( QUAD0_ch0_gtyrxn),
         //.ch0_gtyrxp               ( QUAD0_ch0_gtyrxp),
         .ch0_cfokovrdrdy0         ( QUAD0_RX0_cfokovrdrdy0),       
         .ch0_cfokovrdrdy1         ( QUAD0_RX0_cfokovrdrdy1),
         .ch0_cfokovrdfinish       ( QUAD0_RX0_cfokovrdfinish),
         .ch0_cfokovrdpulse        ( QUAD0_RX0_cfokovrdpulse),
         .ch0_cfokovrdstart        ( QUAD0_RX0_cfokovrdstart),
         .ch0_rxdccdone            ( QUAD0_RX0_dccdone),
         .ch0_rxosintstarted       ( QUAD0_RX0_osintstarted),
         .ch0_rxosintstrobedone    ( QUAD0_RX0_osintstrobedone),
         .ch0_rxosintstrobestarted ( QUAD0_RX0_osintstrobestarted),
         .ch0_rxdataextendrsvd     ( QUAD0_RX0_dataextendrsvd),
         .ch0_txdataextendrsvd     ( QUAD0_TX0_dataextendrsvd),
         .ch0_rx10gstat            ( QUAD0_RX0_10gstat),
         .ch0_rxbufstatus          ( QUAD0_RX0_bufstatus),
         .ch0_rxbyteisaligned      ( QUAD0_RX0_byteisaligned),
         .ch0_rxbyterealign        ( QUAD0_RX0_byterealign),
         .ch0_rxcdrlock            ( QUAD0_RX0_cdrlock),
         .ch0_rxcdrphdone          ( QUAD0_RX0_cdrphdone),
         .ch0_rxchanbondseq        ( QUAD0_RX0_chanbondseq),
         .ch0_rxchanisaligned      ( QUAD0_RX0_chanisaligned),
         .ch0_rxchanrealign        ( QUAD0_RX0_chanrealign),
         .ch0_rxchbondo            ( QUAD0_RX0_chbondo),
         .ch0_rxclkcorcnt          ( QUAD0_RX0_clkcorcnt),
         .ch0_rxcominitdet         ( QUAD0_RX0_cominitdet),
         .ch0_rxcommadet           ( QUAD0_RX0_commadet),
         .ch0_rxcomsasdet          ( QUAD0_RX0_comsasdet),
         .ch0_rxcomwakedet         ( QUAD0_RX0_comwakedet),
         .ch0_rxctrl0              ( QUAD0_RX0_ctrl0),
         .ch0_rxctrl1              ( QUAD0_RX0_ctrl1),
         .ch0_rxctrl2              ( QUAD0_RX0_ctrl2),
         .ch0_rxctrl3              ( QUAD0_RX0_ctrl3),
         .ch0_rxdata               ( QUAD0_RX0_data),
         .ch0_rxdatavalid          ( QUAD0_RX0_datavalid),
         .ch0_rxdlyalignerr        ( QUAD0_RX0_dlyalignerr),
         .ch0_rxdlyalignprog       ( QUAD0_RX0_dlyalignprog),
         .ch0_rxelecidle           ( QUAD0_RX0_elecidle),
         .ch0_rxfinealigndone      ( QUAD0_RX0_finealigndone),
         .ch0_rxheader             ( QUAD0_RX0_header),
         .ch0_rxheadervalid        ( QUAD0_RX0_headervalid),
         .ch0_rxosintdone          ( QUAD0_RX0_osintdone),
         .ch0_rxoutclk             ( QUAD0_RX0_outclk),
         .ch0_rxphaligndone        ( QUAD0_RX0_phaligndone),
         .ch0_rxphalignerr         ( QUAD0_RX0_phalignerr),
         .ch0_rxphdlyresetdone     ( QUAD0_RX0_phdlyresetdone),
         .ch0_rxphsetinitdone      ( QUAD0_RX0_phsetinitdone),
         .ch0_rxphshift180done     ( QUAD0_RX0_phshift180done),
         .ch0_rxpmaresetdone       ( QUAD0_RX0_pmaresetdone),
         .ch0_rxprbserr            ( QUAD0_RX0_prbserr),
         .ch0_rxprbslocked         ( QUAD0_RX0_prbslocked),
         .ch0_rxprogdivresetdone   ( QUAD0_RX0_progdivresetdone),
         .ch0_rxresetdone          ( QUAD0_RX0_resetdone),
         .ch0_rxsliderdy           ( QUAD0_RX0_sliderdy),
         .ch0_rxstartofseq         ( QUAD0_RX0_startofseq),
         .ch0_rxstatus             ( QUAD0_RX0_status),
         .ch0_rxsyncdone           ( QUAD0_RX0_syncdone),
         .ch0_rxvalid              ( QUAD0_RX0_valid),
         .ch0_rxcdrhold            ( QUAD0_RX0_cdrhold),
         .ch0_rxcdrovrden          ( QUAD0_RX0_cdrovrden),
         .ch0_rxcdrreset           ( QUAD0_RX0_cdrreset),
         .ch0_rxchbondi            ( QUAD0_RX0_chbondi),
         .ch0_rxdapicodeovrden     ( QUAD0_RX0_dapicodeovrden),
         .ch0_rxdapicodereset      ( QUAD0_RX0_dapicodereset),
         .ch0_rxdlyalignreq        ( QUAD0_RX0_dlyalignreq),
         .ch0_rxeqtraining         ( QUAD0_RX0_eqtraining),
         .ch0_rxgearboxslip        ( QUAD0_RX0_gearboxslip),
         .ch0_rxlatclk             ( QUAD0_RX0_latclk),
         .ch0_rxlpmen              ( QUAD0_RX0_lpmen),
         .ch0_rxmldchaindone       ( QUAD0_RX0_mldchaindone),
         .ch0_rxmldchainreq        ( QUAD0_RX0_mldchainreq),
         .ch0_rxmlfinealignreq     ( QUAD0_RX0_mlfinealignreq),
         .ch0_rxoobreset           ( QUAD0_RX0_oobreset),
         .ch0_rxpcsresetmask       ( QUAD0_RX0_pcsresetmask),
         .ch0_rxpd                 ( QUAD0_RX0_pd),
         .ch0_rxphalignreq         ( QUAD0_RX0_phalignreq),
         .ch0_rxphalignresetmask   ( QUAD0_RX0_phalignresetmask),
         .ch0_rxphdlypd            ( QUAD0_RX0_phdlypd),
         .ch0_rxphdlyreset         ( QUAD0_RX0_phdlyreset),
         .ch0_rxphsetinitreq       ( QUAD0_RX0_phsetinitreq),
         .ch0_rxphshift180         ( QUAD0_RX0_phshift180),
         .ch0_rxpmaresetmask       ( QUAD0_RX0_pmaresetmask),
         .ch0_rxpolarity           ( QUAD0_RX0_polarity),
         .ch0_rxprbscntreset       ( QUAD0_RX0_prbscntreset),
         .ch0_rxprbssel            ( QUAD0_RX0_prbssel),
         .ch0_rxprogdivreset       ( QUAD0_RX0_progdivreset),
         .ch0_rxrate               ( QUAD0_RX0_rate),
         .ch0_rxresetmode          ( QUAD0_RX0_resetmode),
         .ch0_rxslide              ( QUAD0_RX0_slide),
         .ch0_rxsyncallin          ( QUAD0_RX0_syncallin),
         .ch0_rxtermination        ( QUAD0_RX0_termination),
         .ch0_rxuserrdy            ( QUAD0_RX0_userrdy),
         .ch0_rxusrclk             ( QUAD0_RX0_usrclk),
         .ch0_rxmstresetdone       ( QUAD0_RX0_mstresetdone),
         .ch0_rxmstreset           ( QUAD0_RX0_mstreset),
         .ch0_rxmstdatapathreset   ( QUAD0_RX0_mstdatapathreset),
         .ch0_tx10gstat            ( QUAD0_TX0_10gstat),
         .ch0_txbufstatus          ( QUAD0_TX0_bufstatus),
         .ch0_txcomfinish          ( QUAD0_TX0_comfinish),
         .ch0_txdccdone            ( QUAD0_TX0_dccdone),
         .ch0_txdlyalignerr        ( QUAD0_TX0_dlyalignerr),
         .ch0_txdlyalignprog       ( QUAD0_TX0_dlyalignprog),
         .ch0_txoutclk             ( QUAD0_TX0_outclk),
         .ch0_txphaligndone        ( QUAD0_TX0_phaligndone),
         .ch0_txphalignerr         ( QUAD0_TX0_phalignerr),
         .ch0_txphalignoutrsvd     ( QUAD0_TX0_phalignoutrsvd),
         .ch0_txphdlyresetdone     ( QUAD0_TX0_phdlyresetdone),
         .ch0_txphsetinitdone      ( QUAD0_TX0_phsetinitdone),
         .ch0_txphshift180done     ( QUAD0_TX0_phshift180done),
         .ch0_txpmaresetdone       ( QUAD0_TX0_pmaresetdone),
         .ch0_txprogdivresetdone   ( QUAD0_TX0_progdivresetdone),
         .ch0_txresetdone          ( QUAD0_TX0_resetdone),
         .ch0_txsyncdone           ( QUAD0_TX0_syncdone),
         .ch0_txcominit            ( QUAD0_TX0_cominit),
         .ch0_txcomsas             ( QUAD0_TX0_comsas),
         .ch0_txcomwake            ( QUAD0_TX0_comwake),
         .ch0_txctrl0              ( QUAD0_TX0_ctrl0),
         .ch0_txctrl1              ( QUAD0_TX0_ctrl1),
         .ch0_txctrl2              ( QUAD0_TX0_ctrl2),
         .ch0_txdapicodeovrden     ( QUAD0_TX0_dapicodeovrden),
         .ch0_txdapicodereset      ( QUAD0_TX0_dapicodereset),
         .ch0_txdata               ( QUAD0_TX0_data),
         .ch0_txdeemph             ( QUAD0_TX0_deemph),
         .ch0_txdetectrx           ( QUAD0_TX0_detectrx),
         .ch0_txdiffctrl           ( QUAD0_TX0_diffctrl),
         .ch0_txdlyalignreq        ( QUAD0_TX0_dlyalignreq),
         .ch0_txelecidle           ( QUAD0_TX0_elecidle),
         .ch0_txheader             ( QUAD0_TX0_header),
         .ch0_txinhibit            ( QUAD0_TX0_inhibit),
         .ch0_txlatclk             ( QUAD0_TX0_latclk),
         .ch0_txmaincursor         ( QUAD0_TX0_maincursor),
         .ch0_txmargin             ( QUAD0_TX0_margin),
         .ch0_txmldchaindone       ( QUAD0_TX0_mldchaindone),
         .ch0_txmldchainreq        ( QUAD0_TX0_mldchainreq),
         .ch0_txoneszeros          ( QUAD0_TX0_oneszeros),
         .ch0_txpausedelayalign    ( QUAD0_TX0_pausedelayalign),
         .ch0_txpcsresetmask       ( QUAD0_TX0_pcsresetmask),
         .ch0_txpd                 ( QUAD0_TX0_pd),
         .ch0_txphalignreq         ( QUAD0_TX0_phalignreq),
         .ch0_txphalignresetmask   ( QUAD0_TX0_phalignresetmask),
         .ch0_txphdlypd            ( QUAD0_TX0_phdlypd),
         .ch0_txphdlyreset         ( QUAD0_TX0_phdlyreset),
         .ch0_txphdlytstclk        ( QUAD0_TX0_phdlytstclk),
         .ch0_txphsetinitreq       ( QUAD0_TX0_phsetinitreq),
         .ch0_txphshift180         ( QUAD0_TX0_phshift180),
         .ch0_txpicodeovrden       ( QUAD0_TX0_picodeovrden),
         .ch0_txpicodereset        ( QUAD0_TX0_picodereset),
         .ch0_txpippmen            ( QUAD0_TX0_pippmen),
         .ch0_txpippmstepsize      ( QUAD0_TX0_pippmstepsize),
         .ch0_txpisopd             ( QUAD0_TX0_pisopd),
         .ch0_txpmaresetmask       ( QUAD0_TX0_pmaresetmask),
         .ch0_txpolarity           ( QUAD0_TX0_polarity),
         .ch0_txpostcursor         ( QUAD0_TX0_postcursor),
         .ch0_txprbsforceerr       ( QUAD0_TX0_prbsforceerr),
         .ch0_txprbssel            ( QUAD0_TX0_prbssel),
         .ch0_txprecursor          ( QUAD0_TX0_precursor),
         .ch0_txprogdivreset       ( QUAD0_TX0_progdivreset),
         .ch0_txrate               ( QUAD0_TX0_rate),
         .ch0_txresetmode          ( QUAD0_TX0_resetmode),
         .ch0_txsequence           ( QUAD0_TX0_sequence),
         .ch0_txswing              ( QUAD0_TX0_swing),
         .ch0_txsyncallin          ( QUAD0_TX0_syncallin),
         .ch0_txuserrdy            ( QUAD0_TX0_userrdy),
         .ch0_txusrclk             ( QUAD0_TX0_usrclk),
         .ch0_txmstresetdone       ( QUAD0_TX0_mstresetdone),
         .ch0_txmstreset           ( QUAD0_TX0_mstreset),
         .ch0_txmstdatapathreset   ( QUAD0_TX0_mstdatapathreset),
         .ch1_bufgtce              ( QUAD0_ch1_bufgtce),
         .ch1_bufgtcemask          ( QUAD0_ch1_bufgtcemask),
         .ch1_bufgtdiv             ( QUAD0_ch1_bufgtdiv),
         .ch1_bufgtrst             ( QUAD0_ch1_bufgtrst),
         .ch1_bufgtrstmask         ( QUAD0_ch1_bufgtrstmask),
         //.ch1_gtytxn ( ),
         //.ch1_gtytxp ( ),
         .ch1_iloreset             ( QUAD0_ch1_iloreset),
         .ch1_iloresetdone         ( QUAD0_ch1_iloresetdone),
         .ch1_phystatus            ( QUAD0_ch1_phystatus),
         .ch1_clkrsvd0             ( QUAD0_ch1_clkrsvd0),
         .ch1_clkrsvd1             ( QUAD0_ch1_clkrsvd1),
         .ch1_dmonitorclk          ( QUAD0_ch1_dmonitorclk),
         .ch1_phyesmadaptsave      ( QUAD0_ch1_phyesmadaptsave),
         .ch1_iloresetmask         ( QUAD0_ch1_iloresetmask),
         .ch1_loopback             ( QUAD0_ch1_loopback),
         .ch1_dmonfiforeset        ( QUAD0_ch1_dmonfiforeset),
         .ch1_pcsrsvdin            ( QUAD0_ch1_pcsrsvdin),
         .ch1_gtrsvd               ( QUAD0_ch1_gtrsvd),
         .ch1_tstin                ( QUAD0_ch1_tstin),
         .ch1_pcsrsvdout           ( QUAD0_ch1_pcsrsvdout),
         .ch1_pinrsvdas            ( QUAD0_ch1_pinrsvdas),
         .ch1_dmonitoroutclk       ( QUAD0_ch1_dmonitoroutclk),
         .ch1_resetexception       ( QUAD0_ch1_resetexception),
         .ch1_dmonitorout          ( QUAD0_ch1_dmonitorout),
         .ch1_phyready             ( QUAD0_ch1_phyready),
         .ch1_hsdppcsreset         ( QUAD0_ch1_hsdppcsreset),
         .ch1_pcierstb             ( QUAD0_ch1_pcierstb),

         .ch1_eyescandataerror     ( QUAD0_RX1_eyescandataerror), 
         .ch1_cdrbmcdrreq          ( QUAD0_RX1_cdrbmcdrreq),
         .ch1_cdrfreqos            ( QUAD0_RX1_cdrfreqos),
         .ch1_cdrincpctrl          ( QUAD0_RX1_cdrincpctrl),
         .ch1_cdrstepdir           ( QUAD0_RX1_cdrstepdir),
         .ch1_cdrstepsq            ( QUAD0_RX1_cdrstepsq),
         .ch1_cdrstepsx            ( QUAD0_RX1_cdrstepsx),
         .ch1_eyescanreset         ( QUAD0_RX1_eyescanreset),
         .ch1_eyescantrigger       ( QUAD0_RX1_eyescantrigger),
         .ch1_gtrxreset            ( QUAD0_RX1_gtrxreset),
         .ch1_gttxreset            ( QUAD0_TX1_gttxreset),
         //.ch1_gtyrxn               ( QUAD0_ch1_gtyrxn),
         //.ch1_gtyrxp               ( QUAD0_ch1_gtyrxp),
         .ch1_cfokovrdrdy0         ( QUAD0_RX1_cfokovrdrdy0),       
         .ch1_cfokovrdrdy1         ( QUAD0_RX1_cfokovrdrdy1),
         .ch1_cfokovrdfinish       ( QUAD0_RX1_cfokovrdfinish),
         .ch1_cfokovrdpulse        ( QUAD0_RX1_cfokovrdpulse),
         .ch1_cfokovrdstart        ( QUAD0_RX1_cfokovrdstart),
         .ch1_rxdccdone            ( QUAD0_RX1_dccdone),
         .ch1_rxosintstarted       ( QUAD0_RX1_osintstarted),
         .ch1_rxosintstrobedone    ( QUAD0_RX1_osintstrobedone),
         .ch1_rxosintstrobestarted ( QUAD0_RX1_osintstrobestarted),
         .ch1_rxdataextendrsvd     ( QUAD0_RX1_dataextendrsvd),
         .ch1_txdataextendrsvd     ( QUAD0_TX1_dataextendrsvd),
         .ch1_rx10gstat            ( QUAD0_RX1_10gstat),
         .ch1_rxbufstatus          ( QUAD0_RX1_bufstatus),
         .ch1_rxbyteisaligned      ( QUAD0_RX1_byteisaligned),
         .ch1_rxbyterealign        ( QUAD0_RX1_byterealign),
         .ch1_rxcdrlock            ( QUAD0_RX1_cdrlock),
         .ch1_rxcdrphdone          ( QUAD0_RX1_cdrphdone),
         .ch1_rxchanbondseq        ( QUAD0_RX1_chanbondseq),
         .ch1_rxchanisaligned      ( QUAD0_RX1_chanisaligned),
         .ch1_rxchanrealign        ( QUAD0_RX1_chanrealign),
         .ch1_rxchbondo            ( QUAD0_RX1_chbondo),
         .ch1_rxclkcorcnt          ( QUAD0_RX1_clkcorcnt),
         .ch1_rxcominitdet         ( QUAD0_RX1_cominitdet),
         .ch1_rxcommadet           ( QUAD0_RX1_commadet),
         .ch1_rxcomsasdet          ( QUAD0_RX1_comsasdet),
         .ch1_rxcomwakedet         ( QUAD0_RX1_comwakedet),
         .ch1_rxctrl0              ( QUAD0_RX1_ctrl0),
         .ch1_rxctrl1              ( QUAD0_RX1_ctrl1),
         .ch1_rxctrl2              ( QUAD0_RX1_ctrl2),
         .ch1_rxctrl3              ( QUAD0_RX1_ctrl3),
         .ch1_rxdata               ( QUAD0_RX1_data),
         .ch1_rxdatavalid          ( QUAD0_RX1_datavalid),
         .ch1_rxdlyalignerr        ( QUAD0_RX1_dlyalignerr),
         .ch1_rxdlyalignprog       ( QUAD0_RX1_dlyalignprog),
         .ch1_rxelecidle           ( QUAD0_RX1_elecidle),
         .ch1_rxfinealigndone      ( QUAD0_RX1_finealigndone),
         .ch1_rxheader             ( QUAD0_RX1_header),
         .ch1_rxheadervalid        ( QUAD0_RX1_headervalid),
         .ch1_rxosintdone          ( QUAD0_RX1_osintdone),
         .ch1_rxoutclk             ( QUAD0_RX1_outclk),
         .ch1_rxphaligndone        ( QUAD0_RX1_phaligndone),
         .ch1_rxphalignerr         ( QUAD0_RX1_phalignerr),
         .ch1_rxphdlyresetdone     ( QUAD0_RX1_phdlyresetdone),
         .ch1_rxphsetinitdone      ( QUAD0_RX1_phsetinitdone),
         .ch1_rxphshift180done     ( QUAD0_RX1_phshift180done),
         .ch1_rxpmaresetdone       ( QUAD0_RX1_pmaresetdone),
         .ch1_rxprbserr            ( QUAD0_RX1_prbserr),
         .ch1_rxprbslocked         ( QUAD0_RX1_prbslocked),
         .ch1_rxprogdivresetdone   ( QUAD0_RX1_progdivresetdone),
         .ch1_rxresetdone          ( QUAD0_RX1_resetdone),
         .ch1_rxsliderdy           ( QUAD0_RX1_sliderdy),
         .ch1_rxstartofseq         ( QUAD0_RX1_startofseq),
         .ch1_rxstatus             ( QUAD0_RX1_status),
         .ch1_rxsyncdone           ( QUAD0_RX1_syncdone),
         .ch1_rxvalid              ( QUAD0_RX1_valid),
         .ch1_rxcdrhold            ( QUAD0_RX1_cdrhold),
         .ch1_rxcdrovrden          ( QUAD0_RX1_cdrovrden),
         .ch1_rxcdrreset           ( QUAD0_RX1_cdrreset),
         .ch1_rxchbondi            ( QUAD0_RX1_chbondi),
         .ch1_rxdapicodeovrden     ( QUAD0_RX1_dapicodeovrden),
         .ch1_rxdapicodereset      ( QUAD0_RX1_dapicodereset),
         .ch1_rxdlyalignreq        ( QUAD0_RX1_dlyalignreq),
         .ch1_rxeqtraining         ( QUAD0_RX1_eqtraining),
         .ch1_rxgearboxslip        ( QUAD0_RX1_gearboxslip),
         .ch1_rxlatclk             ( QUAD0_RX1_latclk),
         .ch1_rxlpmen              ( QUAD0_RX1_lpmen),
         .ch1_rxmldchaindone       ( QUAD0_RX1_mldchaindone),
         .ch1_rxmldchainreq        ( QUAD0_RX1_mldchainreq),
         .ch1_rxmlfinealignreq     ( QUAD0_RX1_mlfinealignreq),
         .ch1_rxoobreset           ( QUAD0_RX1_oobreset),
         .ch1_rxpcsresetmask       ( QUAD0_RX1_pcsresetmask),
         .ch1_rxpd                 ( QUAD0_RX1_pd),
         .ch1_rxphalignreq         ( QUAD0_RX1_phalignreq),
         .ch1_rxphalignresetmask   ( QUAD0_RX1_phalignresetmask),
         .ch1_rxphdlypd            ( QUAD0_RX1_phdlypd),
         .ch1_rxphdlyreset         ( QUAD0_RX1_phdlyreset),
         .ch1_rxphsetinitreq       ( QUAD0_RX1_phsetinitreq),
         .ch1_rxphshift180         ( QUAD0_RX1_phshift180),
         .ch1_rxpmaresetmask       ( QUAD0_RX1_pmaresetmask),
         .ch1_rxpolarity           ( QUAD0_RX1_polarity),
         .ch1_rxprbscntreset       ( QUAD0_RX1_prbscntreset),
         .ch1_rxprbssel            ( QUAD0_RX1_prbssel),
         .ch1_rxprogdivreset       ( QUAD0_RX1_progdivreset),
         .ch1_rxrate               ( QUAD0_RX1_rate),
         .ch1_rxresetmode          ( QUAD0_RX1_resetmode),
         .ch1_rxslide              ( QUAD0_RX1_slide),
         .ch1_rxsyncallin          ( QUAD0_RX1_syncallin),
         .ch1_rxtermination        ( QUAD0_RX1_termination),
         .ch1_rxuserrdy            ( QUAD0_RX1_userrdy),
         .ch1_rxusrclk             ( QUAD0_RX1_usrclk),
         .ch1_rxmstresetdone       ( QUAD0_RX1_mstresetdone),
         .ch1_rxmstreset           ( QUAD0_RX1_mstreset),
         .ch1_rxmstdatapathreset   ( QUAD0_RX1_mstdatapathreset),
         .ch1_tx10gstat            ( QUAD0_TX1_10gstat),
         .ch1_txbufstatus          ( QUAD0_TX1_bufstatus),
         .ch1_txcomfinish          ( QUAD0_TX1_comfinish),
         .ch1_txdccdone            ( QUAD0_TX1_dccdone),
         .ch1_txdlyalignerr        ( QUAD0_TX1_dlyalignerr),
         .ch1_txdlyalignprog       ( QUAD0_TX1_dlyalignprog),
         .ch1_txoutclk             ( QUAD0_TX1_outclk),
         .ch1_txphaligndone        ( QUAD0_TX1_phaligndone),
         .ch1_txphalignerr         ( QUAD0_TX1_phalignerr),
         .ch1_txphalignoutrsvd     ( QUAD0_TX1_phalignoutrsvd),
         .ch1_txphdlyresetdone     ( QUAD0_TX1_phdlyresetdone),
         .ch1_txphsetinitdone      ( QUAD0_TX1_phsetinitdone),
         .ch1_txphshift180done     ( QUAD0_TX1_phshift180done),
         .ch1_txpmaresetdone       ( QUAD0_TX1_pmaresetdone),
         .ch1_txprogdivresetdone   ( QUAD0_TX1_progdivresetdone),
         .ch1_txresetdone          ( QUAD0_TX1_resetdone),
         .ch1_txsyncdone           ( QUAD0_TX1_syncdone),
         .ch1_txcominit            ( QUAD0_TX1_cominit),
         .ch1_txcomsas             ( QUAD0_TX1_comsas),
         .ch1_txcomwake            ( QUAD0_TX1_comwake),
         .ch1_txctrl0              ( QUAD0_TX1_ctrl0),
         .ch1_txctrl1              ( QUAD0_TX1_ctrl1),
         .ch1_txctrl2              ( QUAD0_TX1_ctrl2),
         .ch1_txdapicodeovrden     ( QUAD0_TX1_dapicodeovrden),
         .ch1_txdapicodereset      ( QUAD0_TX1_dapicodereset),
         .ch1_txdata               ( QUAD0_TX1_data),
         .ch1_txdeemph             ( QUAD0_TX1_deemph),
         .ch1_txdetectrx           ( QUAD0_TX1_detectrx),
         .ch1_txdiffctrl           ( QUAD0_TX1_diffctrl),
         .ch1_txdlyalignreq        ( QUAD0_TX1_dlyalignreq),
         .ch1_txelecidle           ( QUAD0_TX1_elecidle),
         .ch1_txheader             ( QUAD0_TX1_header),
         .ch1_txinhibit            ( QUAD0_TX1_inhibit),
         .ch1_txlatclk             ( QUAD0_TX1_latclk),
         .ch1_txmaincursor         ( QUAD0_TX1_maincursor),
         .ch1_txmargin             ( QUAD0_TX1_margin),
         .ch1_txmldchaindone       ( QUAD0_TX1_mldchaindone),
         .ch1_txmldchainreq        ( QUAD0_TX1_mldchainreq),
         .ch1_txoneszeros          ( QUAD0_TX1_oneszeros),
         .ch1_txpausedelayalign    ( QUAD0_TX1_pausedelayalign),
         .ch1_txpcsresetmask       ( QUAD0_TX1_pcsresetmask),
         .ch1_txpd                 ( QUAD0_TX1_pd),
         .ch1_txphalignreq         ( QUAD0_TX1_phalignreq),
         .ch1_txphalignresetmask   ( QUAD0_TX1_phalignresetmask),
         .ch1_txphdlypd            ( QUAD0_TX1_phdlypd),
         .ch1_txphdlyreset         ( QUAD0_TX1_phdlyreset),
         .ch1_txphdlytstclk        ( QUAD0_TX1_phdlytstclk),
         .ch1_txphsetinitreq       ( QUAD0_TX1_phsetinitreq),
         .ch1_txphshift180         ( QUAD0_TX1_phshift180),
         .ch1_txpicodeovrden       ( QUAD0_TX1_picodeovrden),
         .ch1_txpicodereset        ( QUAD0_TX1_picodereset),
         .ch1_txpippmen            ( QUAD0_TX1_pippmen),
         .ch1_txpippmstepsize      ( QUAD0_TX1_pippmstepsize),
         .ch1_txpisopd             ( QUAD0_TX1_pisopd),
         .ch1_txpmaresetmask       ( QUAD0_TX1_pmaresetmask),
         .ch1_txpolarity           ( QUAD0_TX1_polarity),
         .ch1_txpostcursor         ( QUAD0_TX1_postcursor),
         .ch1_txprbsforceerr       ( QUAD0_TX1_prbsforceerr),
         .ch1_txprbssel            ( QUAD0_TX1_prbssel),
         .ch1_txprecursor          ( QUAD0_TX1_precursor),
         .ch1_txprogdivreset       ( QUAD0_TX1_progdivreset),
         .ch1_txrate               ( QUAD0_TX1_rate),
         .ch1_txresetmode          ( QUAD0_TX1_resetmode),
         .ch1_txsequence           ( QUAD0_TX1_sequence),
         .ch1_txswing              ( QUAD0_TX1_swing),
         .ch1_txsyncallin          ( QUAD0_TX1_syncallin),
         .ch1_txuserrdy            ( QUAD0_TX1_userrdy),
         .ch1_txusrclk             ( QUAD0_TX1_usrclk),
         .ch1_txmstresetdone       ( QUAD0_TX1_mstresetdone),
         .ch1_txmstreset           ( QUAD0_TX1_mstreset),
         .ch1_txmstdatapathreset   ( QUAD0_TX1_mstdatapathreset),
         .ch2_bufgtce              ( QUAD0_ch2_bufgtce),
         .ch2_bufgtcemask          ( QUAD0_ch2_bufgtcemask),
         .ch2_bufgtdiv             ( QUAD0_ch2_bufgtdiv),
         .ch2_bufgtrst             ( QUAD0_ch2_bufgtrst),
         .ch2_bufgtrstmask         ( QUAD0_ch2_bufgtrstmask),
         //.ch2_gtytxn ( ),
         //.ch2_gtytxp ( ),
         .ch2_iloreset             ( QUAD0_ch2_iloreset),
         .ch2_iloresetdone         ( QUAD0_ch2_iloresetdone),
         .ch2_phystatus            ( QUAD0_ch2_phystatus),
         .ch2_clkrsvd0             ( QUAD0_ch2_clkrsvd0),
         .ch2_clkrsvd1             ( QUAD0_ch2_clkrsvd1),
         .ch2_dmonitorclk          ( QUAD0_ch2_dmonitorclk),
         .ch2_phyesmadaptsave      ( QUAD0_ch2_phyesmadaptsave),
         .ch2_iloresetmask         ( QUAD0_ch2_iloresetmask),
         .ch2_loopback             ( QUAD0_ch2_loopback),
         .ch2_dmonfiforeset        ( QUAD0_ch2_dmonfiforeset),
         .ch2_pcsrsvdin            ( QUAD0_ch2_pcsrsvdin),
         .ch2_gtrsvd               ( QUAD0_ch2_gtrsvd),
         .ch2_tstin                ( QUAD0_ch2_tstin),
         .ch2_pcsrsvdout           ( QUAD0_ch2_pcsrsvdout),
         .ch2_pinrsvdas            ( QUAD0_ch2_pinrsvdas),
         .ch2_dmonitoroutclk       ( QUAD0_ch2_dmonitoroutclk),
         .ch2_resetexception       ( QUAD0_ch2_resetexception),
         .ch2_dmonitorout          ( QUAD0_ch2_dmonitorout),
         .ch2_phyready             ( QUAD0_ch2_phyready),
         .ch2_hsdppcsreset         ( QUAD0_ch2_hsdppcsreset),
         .ch2_pcierstb             ( QUAD0_ch2_pcierstb),

         .ch2_eyescandataerror     ( QUAD0_RX2_eyescandataerror), 
         .ch2_cdrbmcdrreq          ( QUAD0_RX2_cdrbmcdrreq),
         .ch2_cdrfreqos            ( QUAD0_RX2_cdrfreqos),
         .ch2_cdrincpctrl          ( QUAD0_RX2_cdrincpctrl),
         .ch2_cdrstepdir           ( QUAD0_RX2_cdrstepdir),
         .ch2_cdrstepsq            ( QUAD0_RX2_cdrstepsq),
         .ch2_cdrstepsx            ( QUAD0_RX2_cdrstepsx),
         .ch2_eyescanreset         ( QUAD0_RX2_eyescanreset),
         .ch2_eyescantrigger       ( QUAD0_RX2_eyescantrigger),
         .ch2_gtrxreset            ( QUAD0_RX2_gtrxreset),
         .ch2_gttxreset            ( QUAD0_TX2_gttxreset),
         //.ch2_gtyrxn               ( QUAD0_ch2_gtyrxn),
         //.ch2_gtyrxp               ( QUAD0_ch2_gtyrxp),
         .ch2_cfokovrdrdy0         ( QUAD0_RX2_cfokovrdrdy0),       
         .ch2_cfokovrdrdy1         ( QUAD0_RX2_cfokovrdrdy1),
         .ch2_cfokovrdfinish       ( QUAD0_RX2_cfokovrdfinish),
         .ch2_cfokovrdpulse        ( QUAD0_RX2_cfokovrdpulse),
         .ch2_cfokovrdstart        ( QUAD0_RX2_cfokovrdstart),
         .ch2_rxdccdone            ( QUAD0_RX2_dccdone),
         .ch2_rxosintstarted       ( QUAD0_RX2_osintstarted),
         .ch2_rxosintstrobedone    ( QUAD0_RX2_osintstrobedone),
         .ch2_rxosintstrobestarted ( QUAD0_RX2_osintstrobestarted),
         .ch2_rxdataextendrsvd     ( QUAD0_RX2_dataextendrsvd),
         .ch2_txdataextendrsvd     ( QUAD0_TX2_dataextendrsvd),
         .ch2_rx10gstat            ( QUAD0_RX2_10gstat),
         .ch2_rxbufstatus          ( QUAD0_RX2_bufstatus),
         .ch2_rxbyteisaligned      ( QUAD0_RX2_byteisaligned),
         .ch2_rxbyterealign        ( QUAD0_RX2_byterealign),
         .ch2_rxcdrlock            ( QUAD0_RX2_cdrlock),
         .ch2_rxcdrphdone          ( QUAD0_RX2_cdrphdone),
         .ch2_rxchanbondseq        ( QUAD0_RX2_chanbondseq),
         .ch2_rxchanisaligned      ( QUAD0_RX2_chanisaligned),
         .ch2_rxchanrealign        ( QUAD0_RX2_chanrealign),
         .ch2_rxchbondo            ( QUAD0_RX2_chbondo),
         .ch2_rxclkcorcnt          ( QUAD0_RX2_clkcorcnt),
         .ch2_rxcominitdet         ( QUAD0_RX2_cominitdet),
         .ch2_rxcommadet           ( QUAD0_RX2_commadet),
         .ch2_rxcomsasdet          ( QUAD0_RX2_comsasdet),
         .ch2_rxcomwakedet         ( QUAD0_RX2_comwakedet),
         .ch2_rxctrl0              ( QUAD0_RX2_ctrl0),
         .ch2_rxctrl1              ( QUAD0_RX2_ctrl1),
         .ch2_rxctrl2              ( QUAD0_RX2_ctrl2),
         .ch2_rxctrl3              ( QUAD0_RX2_ctrl3),
         .ch2_rxdata               ( QUAD0_RX2_data),
         .ch2_rxdatavalid          ( QUAD0_RX2_datavalid),
         .ch2_rxdlyalignerr        ( QUAD0_RX2_dlyalignerr),
         .ch2_rxdlyalignprog       ( QUAD0_RX2_dlyalignprog),
         .ch2_rxelecidle           ( QUAD0_RX2_elecidle),
         .ch2_rxfinealigndone      ( QUAD0_RX2_finealigndone),
         .ch2_rxheader             ( QUAD0_RX2_header),
         .ch2_rxheadervalid        ( QUAD0_RX2_headervalid),
         .ch2_rxosintdone          ( QUAD0_RX2_osintdone),
         .ch2_rxoutclk             ( QUAD0_RX2_outclk),
         .ch2_rxphaligndone        ( QUAD0_RX2_phaligndone),
         .ch2_rxphalignerr         ( QUAD0_RX2_phalignerr),
         .ch2_rxphdlyresetdone     ( QUAD0_RX2_phdlyresetdone),
         .ch2_rxphsetinitdone      ( QUAD0_RX2_phsetinitdone),
         .ch2_rxphshift180done     ( QUAD0_RX2_phshift180done),
         .ch2_rxpmaresetdone       ( QUAD0_RX2_pmaresetdone),
         .ch2_rxprbserr            ( QUAD0_RX2_prbserr),
         .ch2_rxprbslocked         ( QUAD0_RX2_prbslocked),
         .ch2_rxprogdivresetdone   ( QUAD0_RX2_progdivresetdone),
         .ch2_rxresetdone          ( QUAD0_RX2_resetdone),
         .ch2_rxsliderdy           ( QUAD0_RX2_sliderdy),
         .ch2_rxstartofseq         ( QUAD0_RX2_startofseq),
         .ch2_rxstatus             ( QUAD0_RX2_status),
         .ch2_rxsyncdone           ( QUAD0_RX2_syncdone),
         .ch2_rxvalid              ( QUAD0_RX2_valid),
         .ch2_rxcdrhold            ( QUAD0_RX2_cdrhold),
         .ch2_rxcdrovrden          ( QUAD0_RX2_cdrovrden),
         .ch2_rxcdrreset           ( QUAD0_RX2_cdrreset),
         .ch2_rxchbondi            ( QUAD0_RX2_chbondi),
         .ch2_rxdapicodeovrden     ( QUAD0_RX2_dapicodeovrden),
         .ch2_rxdapicodereset      ( QUAD0_RX2_dapicodereset),
         .ch2_rxdlyalignreq        ( QUAD0_RX2_dlyalignreq),
         .ch2_rxeqtraining         ( QUAD0_RX2_eqtraining),
         .ch2_rxgearboxslip        ( QUAD0_RX2_gearboxslip),
         .ch2_rxlatclk             ( QUAD0_RX2_latclk),
         .ch2_rxlpmen              ( QUAD0_RX2_lpmen),
         .ch2_rxmldchaindone       ( QUAD0_RX2_mldchaindone),
         .ch2_rxmldchainreq        ( QUAD0_RX2_mldchainreq),
         .ch2_rxmlfinealignreq     ( QUAD0_RX2_mlfinealignreq),
         .ch2_rxoobreset           ( QUAD0_RX2_oobreset),
         .ch2_rxpcsresetmask       ( QUAD0_RX2_pcsresetmask),
         .ch2_rxpd                 ( QUAD0_RX2_pd),
         .ch2_rxphalignreq         ( QUAD0_RX2_phalignreq),
         .ch2_rxphalignresetmask   ( QUAD0_RX2_phalignresetmask),
         .ch2_rxphdlypd            ( QUAD0_RX2_phdlypd),
         .ch2_rxphdlyreset         ( QUAD0_RX2_phdlyreset),
         .ch2_rxphsetinitreq       ( QUAD0_RX2_phsetinitreq),
         .ch2_rxphshift180         ( QUAD0_RX2_phshift180),
         .ch2_rxpmaresetmask       ( QUAD0_RX2_pmaresetmask),
         .ch2_rxpolarity           ( QUAD0_RX2_polarity),
         .ch2_rxprbscntreset       ( QUAD0_RX2_prbscntreset),
         .ch2_rxprbssel            ( QUAD0_RX2_prbssel),
         .ch2_rxprogdivreset       ( QUAD0_RX2_progdivreset),
         .ch2_rxrate               ( QUAD0_RX2_rate),
         .ch2_rxresetmode          ( QUAD0_RX2_resetmode),
         .ch2_rxslide              ( QUAD0_RX2_slide),
         .ch2_rxsyncallin          ( QUAD0_RX2_syncallin),
         .ch2_rxtermination        ( QUAD0_RX2_termination),
         .ch2_rxuserrdy            ( QUAD0_RX2_userrdy),
         .ch2_rxusrclk             ( QUAD0_RX2_usrclk),
         .ch2_rxmstresetdone       ( QUAD0_RX2_mstresetdone),
         .ch2_rxmstreset           ( QUAD0_RX2_mstreset),
         .ch2_rxmstdatapathreset   ( QUAD0_RX2_mstdatapathreset),
         .ch2_tx10gstat            ( QUAD0_TX2_10gstat),
         .ch2_txbufstatus          ( QUAD0_TX2_bufstatus),
         .ch2_txcomfinish          ( QUAD0_TX2_comfinish),
         .ch2_txdccdone            ( QUAD0_TX2_dccdone),
         .ch2_txdlyalignerr        ( QUAD0_TX2_dlyalignerr),
         .ch2_txdlyalignprog       ( QUAD0_TX2_dlyalignprog),
         .ch2_txoutclk             ( QUAD0_TX2_outclk),
         .ch2_txphaligndone        ( QUAD0_TX2_phaligndone),
         .ch2_txphalignerr         ( QUAD0_TX2_phalignerr),
         .ch2_txphalignoutrsvd     ( QUAD0_TX2_phalignoutrsvd),
         .ch2_txphdlyresetdone     ( QUAD0_TX2_phdlyresetdone),
         .ch2_txphsetinitdone      ( QUAD0_TX2_phsetinitdone),
         .ch2_txphshift180done     ( QUAD0_TX2_phshift180done),
         .ch2_txpmaresetdone       ( QUAD0_TX2_pmaresetdone),
         .ch2_txprogdivresetdone   ( QUAD0_TX2_progdivresetdone),
         .ch2_txresetdone          ( QUAD0_TX2_resetdone),
         .ch2_txsyncdone           ( QUAD0_TX2_syncdone),
         .ch2_txcominit            ( QUAD0_TX2_cominit),
         .ch2_txcomsas             ( QUAD0_TX2_comsas),
         .ch2_txcomwake            ( QUAD0_TX2_comwake),
         .ch2_txctrl0              ( QUAD0_TX2_ctrl0),
         .ch2_txctrl1              ( QUAD0_TX2_ctrl1),
         .ch2_txctrl2              ( QUAD0_TX2_ctrl2),
         .ch2_txdapicodeovrden     ( QUAD0_TX2_dapicodeovrden),
         .ch2_txdapicodereset      ( QUAD0_TX2_dapicodereset),
         .ch2_txdata               ( QUAD0_TX2_data),
         .ch2_txdeemph             ( QUAD0_TX2_deemph),
         .ch2_txdetectrx           ( QUAD0_TX2_detectrx),
         .ch2_txdiffctrl           ( QUAD0_TX2_diffctrl),
         .ch2_txdlyalignreq        ( QUAD0_TX2_dlyalignreq),
         .ch2_txelecidle           ( QUAD0_TX2_elecidle),
         .ch2_txheader             ( QUAD0_TX2_header),
         .ch2_txinhibit            ( QUAD0_TX2_inhibit),
         .ch2_txlatclk             ( QUAD0_TX2_latclk),
         .ch2_txmaincursor         ( QUAD0_TX2_maincursor),
         .ch2_txmargin             ( QUAD0_TX2_margin),
         .ch2_txmldchaindone       ( QUAD0_TX2_mldchaindone),
         .ch2_txmldchainreq        ( QUAD0_TX2_mldchainreq),
         .ch2_txoneszeros          ( QUAD0_TX2_oneszeros),
         .ch2_txpausedelayalign    ( QUAD0_TX2_pausedelayalign),
         .ch2_txpcsresetmask       ( QUAD0_TX2_pcsresetmask),
         .ch2_txpd                 ( QUAD0_TX2_pd),
         .ch2_txphalignreq         ( QUAD0_TX2_phalignreq),
         .ch2_txphalignresetmask   ( QUAD0_TX2_phalignresetmask),
         .ch2_txphdlypd            ( QUAD0_TX2_phdlypd),
         .ch2_txphdlyreset         ( QUAD0_TX2_phdlyreset),
         .ch2_txphdlytstclk        ( QUAD0_TX2_phdlytstclk),
         .ch2_txphsetinitreq       ( QUAD0_TX2_phsetinitreq),
         .ch2_txphshift180         ( QUAD0_TX2_phshift180),
         .ch2_txpicodeovrden       ( QUAD0_TX2_picodeovrden),
         .ch2_txpicodereset        ( QUAD0_TX2_picodereset),
         .ch2_txpippmen            ( QUAD0_TX2_pippmen),
         .ch2_txpippmstepsize      ( QUAD0_TX2_pippmstepsize),
         .ch2_txpisopd             ( QUAD0_TX2_pisopd),
         .ch2_txpmaresetmask       ( QUAD0_TX2_pmaresetmask),
         .ch2_txpolarity           ( QUAD0_TX2_polarity),
         .ch2_txpostcursor         ( QUAD0_TX2_postcursor),
         .ch2_txprbsforceerr       ( QUAD0_TX2_prbsforceerr),
         .ch2_txprbssel            ( QUAD0_TX2_prbssel),
         .ch2_txprecursor          ( QUAD0_TX2_precursor),
         .ch2_txprogdivreset       ( QUAD0_TX2_progdivreset),
         .ch2_txrate               ( QUAD0_TX2_rate),
         .ch2_txresetmode          ( QUAD0_TX2_resetmode),
         .ch2_txsequence           ( QUAD0_TX2_sequence),
         .ch2_txswing              ( QUAD0_TX2_swing),
         .ch2_txsyncallin          ( QUAD0_TX2_syncallin),
         .ch2_txuserrdy            ( QUAD0_TX2_userrdy),
         .ch2_txusrclk             ( QUAD0_TX2_usrclk),
         .ch2_txmstresetdone       ( QUAD0_TX2_mstresetdone),
         .ch2_txmstreset           ( QUAD0_TX2_mstreset),
         .ch2_txmstdatapathreset   ( QUAD0_TX2_mstdatapathreset),
         .ch3_bufgtce              ( QUAD0_ch3_bufgtce),
         .ch3_bufgtcemask          ( QUAD0_ch3_bufgtcemask),
         .ch3_bufgtdiv             ( QUAD0_ch3_bufgtdiv),
         .ch3_bufgtrst             ( QUAD0_ch3_bufgtrst),
         .ch3_bufgtrstmask         ( QUAD0_ch3_bufgtrstmask),
         //.ch3_gtytxn ( ),
         //.ch3_gtytxp ( ),
         .ch3_iloreset             ( QUAD0_ch3_iloreset),
         .ch3_iloresetdone         ( QUAD0_ch3_iloresetdone),
         .ch3_phystatus            ( QUAD0_ch3_phystatus),
         .ch3_clkrsvd0             ( QUAD0_ch3_clkrsvd0),
         .ch3_clkrsvd1             ( QUAD0_ch3_clkrsvd1),
         .ch3_dmonitorclk          ( QUAD0_ch3_dmonitorclk),
         .ch3_phyesmadaptsave      ( QUAD0_ch3_phyesmadaptsave),
         .ch3_iloresetmask         ( QUAD0_ch3_iloresetmask),
         .ch3_loopback             ( QUAD0_ch3_loopback),
         .ch3_dmonfiforeset        ( QUAD0_ch3_dmonfiforeset),
         .ch3_pcsrsvdin            ( QUAD0_ch3_pcsrsvdin),
         .ch3_gtrsvd               ( QUAD0_ch3_gtrsvd),
         .ch3_tstin                ( QUAD0_ch3_tstin),
         .ch3_pcsrsvdout           ( QUAD0_ch3_pcsrsvdout),
         .ch3_pinrsvdas            ( QUAD0_ch3_pinrsvdas),
         .ch3_dmonitoroutclk       ( QUAD0_ch3_dmonitoroutclk),
         .ch3_resetexception       ( QUAD0_ch3_resetexception),
         .ch3_dmonitorout          ( QUAD0_ch3_dmonitorout),
         .ch3_phyready             ( QUAD0_ch3_phyready),
         .ch3_hsdppcsreset         ( QUAD0_ch3_hsdppcsreset),
         .ch3_pcierstb             ( QUAD0_ch3_pcierstb),

         .ch3_eyescandataerror     ( QUAD0_RX3_eyescandataerror), 
         .ch3_cdrbmcdrreq          ( QUAD0_RX3_cdrbmcdrreq),
         .ch3_cdrfreqos            ( QUAD0_RX3_cdrfreqos),
         .ch3_cdrincpctrl          ( QUAD0_RX3_cdrincpctrl),
         .ch3_cdrstepdir           ( QUAD0_RX3_cdrstepdir),
         .ch3_cdrstepsq            ( QUAD0_RX3_cdrstepsq),
         .ch3_cdrstepsx            ( QUAD0_RX3_cdrstepsx),
         .ch3_eyescanreset         ( QUAD0_RX3_eyescanreset),
         .ch3_eyescantrigger       ( QUAD0_RX3_eyescantrigger),
         .ch3_gtrxreset            ( QUAD0_RX3_gtrxreset),
         .ch3_gttxreset            ( QUAD0_TX3_gttxreset),
         //.ch3_gtyrxn               ( QUAD0_ch3_gtyrxn),
         //.ch3_gtyrxp               ( QUAD0_ch3_gtyrxp),
         .ch3_cfokovrdrdy0         ( QUAD0_RX3_cfokovrdrdy0),       
         .ch3_cfokovrdrdy1         ( QUAD0_RX3_cfokovrdrdy1),
         .ch3_cfokovrdfinish       ( QUAD0_RX3_cfokovrdfinish),
         .ch3_cfokovrdpulse        ( QUAD0_RX3_cfokovrdpulse),
         .ch3_cfokovrdstart        ( QUAD0_RX3_cfokovrdstart),
         .ch3_rxdccdone            ( QUAD0_RX3_dccdone),
         .ch3_rxosintstarted       ( QUAD0_RX3_osintstarted),
         .ch3_rxosintstrobedone    ( QUAD0_RX3_osintstrobedone),
         .ch3_rxosintstrobestarted ( QUAD0_RX3_osintstrobestarted),
         .ch3_rxdataextendrsvd     ( QUAD0_RX3_dataextendrsvd),
         .ch3_txdataextendrsvd     ( QUAD0_TX3_dataextendrsvd),
         .ch3_rx10gstat            ( QUAD0_RX3_10gstat),
         .ch3_rxbufstatus          ( QUAD0_RX3_bufstatus),
         .ch3_rxbyteisaligned      ( QUAD0_RX3_byteisaligned),
         .ch3_rxbyterealign        ( QUAD0_RX3_byterealign),
         .ch3_rxcdrlock            ( QUAD0_RX3_cdrlock),
         .ch3_rxcdrphdone          ( QUAD0_RX3_cdrphdone),
         .ch3_rxchanbondseq        ( QUAD0_RX3_chanbondseq),
         .ch3_rxchanisaligned      ( QUAD0_RX3_chanisaligned),
         .ch3_rxchanrealign        ( QUAD0_RX3_chanrealign),
         .ch3_rxchbondo            ( QUAD0_RX3_chbondo),
         .ch3_rxclkcorcnt          ( QUAD0_RX3_clkcorcnt),
         .ch3_rxcominitdet         ( QUAD0_RX3_cominitdet),
         .ch3_rxcommadet           ( QUAD0_RX3_commadet),
         .ch3_rxcomsasdet          ( QUAD0_RX3_comsasdet),
         .ch3_rxcomwakedet         ( QUAD0_RX3_comwakedet),
         .ch3_rxctrl0              ( QUAD0_RX3_ctrl0),
         .ch3_rxctrl1              ( QUAD0_RX3_ctrl1),
         .ch3_rxctrl2              ( QUAD0_RX3_ctrl2),
         .ch3_rxctrl3              ( QUAD0_RX3_ctrl3),
         .ch3_rxdata               ( QUAD0_RX3_data),
         .ch3_rxdatavalid          ( QUAD0_RX3_datavalid),
         .ch3_rxdlyalignerr        ( QUAD0_RX3_dlyalignerr),
         .ch3_rxdlyalignprog       ( QUAD0_RX3_dlyalignprog),
         .ch3_rxelecidle           ( QUAD0_RX3_elecidle),
         .ch3_rxfinealigndone      ( QUAD0_RX3_finealigndone),
         .ch3_rxheader             ( QUAD0_RX3_header),
         .ch3_rxheadervalid        ( QUAD0_RX3_headervalid),
         .ch3_rxosintdone          ( QUAD0_RX3_osintdone),
         .ch3_rxoutclk             ( QUAD0_RX3_outclk),
         .ch3_rxphaligndone        ( QUAD0_RX3_phaligndone),
         .ch3_rxphalignerr         ( QUAD0_RX3_phalignerr),
         .ch3_rxphdlyresetdone     ( QUAD0_RX3_phdlyresetdone),
         .ch3_rxphsetinitdone      ( QUAD0_RX3_phsetinitdone),
         .ch3_rxphshift180done     ( QUAD0_RX3_phshift180done),
         .ch3_rxpmaresetdone       ( QUAD0_RX3_pmaresetdone),
         .ch3_rxprbserr            ( QUAD0_RX3_prbserr),
         .ch3_rxprbslocked         ( QUAD0_RX3_prbslocked),
         .ch3_rxprogdivresetdone   ( QUAD0_RX3_progdivresetdone),
         .ch3_rxresetdone          ( QUAD0_RX3_resetdone),
         .ch3_rxsliderdy           ( QUAD0_RX3_sliderdy),
         .ch3_rxstartofseq         ( QUAD0_RX3_startofseq),
         .ch3_rxstatus             ( QUAD0_RX3_status),
         .ch3_rxsyncdone           ( QUAD0_RX3_syncdone),
         .ch3_rxvalid              ( QUAD0_RX3_valid),
         .ch3_rxcdrhold            ( QUAD0_RX3_cdrhold),
         .ch3_rxcdrovrden          ( QUAD0_RX3_cdrovrden),
         .ch3_rxcdrreset           ( QUAD0_RX3_cdrreset),
         .ch3_rxchbondi            ( QUAD0_RX3_chbondi),
         .ch3_rxdapicodeovrden     ( QUAD0_RX3_dapicodeovrden),
         .ch3_rxdapicodereset      ( QUAD0_RX3_dapicodereset),
         .ch3_rxdlyalignreq        ( QUAD0_RX3_dlyalignreq),
         .ch3_rxeqtraining         ( QUAD0_RX3_eqtraining),
         .ch3_rxgearboxslip        ( QUAD0_RX3_gearboxslip),
         .ch3_rxlatclk             ( QUAD0_RX3_latclk),
         .ch3_rxlpmen              ( QUAD0_RX3_lpmen),
         .ch3_rxmldchaindone       ( QUAD0_RX3_mldchaindone),
         .ch3_rxmldchainreq        ( QUAD0_RX3_mldchainreq),
         .ch3_rxmlfinealignreq     ( QUAD0_RX3_mlfinealignreq),
         .ch3_rxoobreset           ( QUAD0_RX3_oobreset),
         .ch3_rxpcsresetmask       ( QUAD0_RX3_pcsresetmask),
         .ch3_rxpd                 ( QUAD0_RX3_pd),
         .ch3_rxphalignreq         ( QUAD0_RX3_phalignreq),
         .ch3_rxphalignresetmask   ( QUAD0_RX3_phalignresetmask),
         .ch3_rxphdlypd            ( QUAD0_RX3_phdlypd),
         .ch3_rxphdlyreset         ( QUAD0_RX3_phdlyreset),
         .ch3_rxphsetinitreq       ( QUAD0_RX3_phsetinitreq),
         .ch3_rxphshift180         ( QUAD0_RX3_phshift180),
         .ch3_rxpmaresetmask       ( QUAD0_RX3_pmaresetmask),
         .ch3_rxpolarity           ( QUAD0_RX3_polarity),
         .ch3_rxprbscntreset       ( QUAD0_RX3_prbscntreset),
         .ch3_rxprbssel            ( QUAD0_RX3_prbssel),
         .ch3_rxprogdivreset       ( QUAD0_RX3_progdivreset),
         .ch3_rxrate               ( QUAD0_RX3_rate),
         .ch3_rxresetmode          ( QUAD0_RX3_resetmode),
         .ch3_rxslide              ( QUAD0_RX3_slide),
         .ch3_rxsyncallin          ( QUAD0_RX3_syncallin),
         .ch3_rxtermination        ( QUAD0_RX3_termination),
         .ch3_rxuserrdy            ( QUAD0_RX3_userrdy),
         .ch3_rxusrclk             ( QUAD0_RX3_usrclk),
         .ch3_rxmstresetdone       ( QUAD0_RX3_mstresetdone),
         .ch3_rxmstreset           ( QUAD0_RX3_mstreset),
         .ch3_rxmstdatapathreset   ( QUAD0_RX3_mstdatapathreset),
         .ch3_tx10gstat            ( QUAD0_TX3_10gstat),
         .ch3_txbufstatus          ( QUAD0_TX3_bufstatus),
         .ch3_txcomfinish          ( QUAD0_TX3_comfinish),
         .ch3_txdccdone            ( QUAD0_TX3_dccdone),
         .ch3_txdlyalignerr        ( QUAD0_TX3_dlyalignerr),
         .ch3_txdlyalignprog       ( QUAD0_TX3_dlyalignprog),
         .ch3_txoutclk             ( QUAD0_TX3_outclk),
         .ch3_txphaligndone        ( QUAD0_TX3_phaligndone),
         .ch3_txphalignerr         ( QUAD0_TX3_phalignerr),
         .ch3_txphalignoutrsvd     ( QUAD0_TX3_phalignoutrsvd),
         .ch3_txphdlyresetdone     ( QUAD0_TX3_phdlyresetdone),
         .ch3_txphsetinitdone      ( QUAD0_TX3_phsetinitdone),
         .ch3_txphshift180done     ( QUAD0_TX3_phshift180done),
         .ch3_txpmaresetdone       ( QUAD0_TX3_pmaresetdone),
         .ch3_txprogdivresetdone   ( QUAD0_TX3_progdivresetdone),
         .ch3_txresetdone          ( QUAD0_TX3_resetdone),
         .ch3_txsyncdone           ( QUAD0_TX3_syncdone),
         .ch3_txcominit            ( QUAD0_TX3_cominit),
         .ch3_txcomsas             ( QUAD0_TX3_comsas),
         .ch3_txcomwake            ( QUAD0_TX3_comwake),
         .ch3_txctrl0              ( QUAD0_TX3_ctrl0),
         .ch3_txctrl1              ( QUAD0_TX3_ctrl1),
         .ch3_txctrl2              ( QUAD0_TX3_ctrl2),
         .ch3_txdapicodeovrden     ( QUAD0_TX3_dapicodeovrden),
         .ch3_txdapicodereset      ( QUAD0_TX3_dapicodereset),
         .ch3_txdata               ( QUAD0_TX3_data),
         .ch3_txdeemph             ( QUAD0_TX3_deemph),
         .ch3_txdetectrx           ( QUAD0_TX3_detectrx),
         .ch3_txdiffctrl           ( QUAD0_TX3_diffctrl),
         .ch3_txdlyalignreq        ( QUAD0_TX3_dlyalignreq),
         .ch3_txelecidle           ( QUAD0_TX3_elecidle),
         .ch3_txheader             ( QUAD0_TX3_header),
         .ch3_txinhibit            ( QUAD0_TX3_inhibit),
         .ch3_txlatclk             ( QUAD0_TX3_latclk),
         .ch3_txmaincursor         ( QUAD0_TX3_maincursor),
         .ch3_txmargin             ( QUAD0_TX3_margin),
         .ch3_txmldchaindone       ( QUAD0_TX3_mldchaindone),
         .ch3_txmldchainreq        ( QUAD0_TX3_mldchainreq),
         .ch3_txoneszeros          ( QUAD0_TX3_oneszeros),
         .ch3_txpausedelayalign    ( QUAD0_TX3_pausedelayalign),
         .ch3_txpcsresetmask       ( QUAD0_TX3_pcsresetmask),
         .ch3_txpd                 ( QUAD0_TX3_pd),
         .ch3_txphalignreq         ( QUAD0_TX3_phalignreq),
         .ch3_txphalignresetmask   ( QUAD0_TX3_phalignresetmask),
         .ch3_txphdlypd            ( QUAD0_TX3_phdlypd),
         .ch3_txphdlyreset         ( QUAD0_TX3_phdlyreset),
         .ch3_txphdlytstclk        ( QUAD0_TX3_phdlytstclk),
         .ch3_txphsetinitreq       ( QUAD0_TX3_phsetinitreq),
         .ch3_txphshift180         ( QUAD0_TX3_phshift180),
         .ch3_txpicodeovrden       ( QUAD0_TX3_picodeovrden),
         .ch3_txpicodereset        ( QUAD0_TX3_picodereset),
         .ch3_txpippmen            ( QUAD0_TX3_pippmen),
         .ch3_txpippmstepsize      ( QUAD0_TX3_pippmstepsize),
         .ch3_txpisopd             ( QUAD0_TX3_pisopd),
         .ch3_txpmaresetmask       ( QUAD0_TX3_pmaresetmask),
         .ch3_txpolarity           ( QUAD0_TX3_polarity),
         .ch3_txpostcursor         ( QUAD0_TX3_postcursor),
         .ch3_txprbsforceerr       ( QUAD0_TX3_prbsforceerr),
         .ch3_txprbssel            ( QUAD0_TX3_prbssel),
         .ch3_txprecursor          ( QUAD0_TX3_precursor),
         .ch3_txprogdivreset       ( QUAD0_TX3_progdivreset),
         .ch3_txrate               ( QUAD0_TX3_rate),
         .ch3_txresetmode          ( QUAD0_TX3_resetmode),
         .ch3_txsequence           ( QUAD0_TX3_sequence),
         .ch3_txswing              ( QUAD0_TX3_swing),
         .ch3_txsyncallin          ( QUAD0_TX3_syncallin),
         .ch3_txuserrdy            ( QUAD0_TX3_userrdy),
         .ch3_txusrclk             ( QUAD0_TX3_usrclk),
         .ch3_txmstresetdone       ( QUAD0_TX3_mstresetdone),
         .ch3_txmstreset           ( QUAD0_TX3_mstreset),
         .ch3_txmstdatapathreset   ( QUAD0_TX3_mstdatapathreset),
         .gtpowergood                      ( QUAD0_gtpowergood)
);

endmodule
