// (c) Copyright 2023 Advanced Micro Devices, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of AMD and is protected under U.S. and international copyright
// and other intellectual property laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// AMD, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND AMD HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) AMD shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or AMD had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// AMD products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of AMD products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
//-----------------------------------------------------------------
`timescale 1ns / 1fs
module my_gtm_exdes_vio  
(
			input QUAD0_GTREFCLK0_in_p ,
			input QUAD0_GTREFCLK0_in_n ,
			input QUAD0_GTREFCLK1_in_p ,
			input QUAD0_GTREFCLK1_in_n ,
			input    [3:0]  QUAD0_rxp ,
			input    [3:0]  QUAD0_rxn ,
			output   [3:0]  QUAD0_txp ,
			output   [3:0]  QUAD0_txn ,
			input wire      gtwiz_freerun_clock

);
wire       INTF0_link_status_out;
wire       INTF0_rst_tx_done_out;
wire       gtwiz_INTF0_rst_tx_datapath_in;
wire       INTF0_TX_usrclk;
wire       INTF0_rst_rx_done_out;
wire       gtwiz_INTF0_rst_rx_datapath_in ;
wire       INTF0_RX_usrclk;
wire [3:0] intf0_rate_sel;
wire       intf0_gpio_enable;
wire       gtwiz_INTF0_rst_all_in ;
wire       INTF1_link_status_out;
wire       INTF1_rst_tx_done_out;
wire       gtwiz_INTF1_rst_tx_datapath_in;
wire       INTF1_TX_usrclk;
wire       INTF1_rst_rx_done_out;
wire       gtwiz_INTF1_rst_rx_datapath_in ;
wire       INTF1_RX_usrclk;
wire [3:0] intf1_rate_sel;
wire       intf1_gpio_enable;
wire       gtwiz_INTF1_rst_all_in ;
wire       gt_powergood;
wire [2:0] gt_loopback;


my_gtm_example_top exdes_inst ( 
			.QUAD0_GTREFCLK0_in_p(QUAD0_GTREFCLK0_in_p),
			.QUAD0_GTREFCLK0_in_n(QUAD0_GTREFCLK0_in_n),
			.QUAD0_GTREFCLK1_in_p(QUAD0_GTREFCLK1_in_p),
			.QUAD0_GTREFCLK1_in_n(QUAD0_GTREFCLK1_in_n),
.QUAD0_rxp(QUAD0_rxp) ,
.QUAD0_rxn(QUAD0_rxn) ,
.QUAD0_txp(QUAD0_txp) ,
.QUAD0_txn(QUAD0_txn) ,
.INTF0_link_status_out(INTF0_link_status_out),
.INTF0_rst_tx_done_out(INTF0_rst_tx_done_out),
.gtwiz_INTF0_rst_tx_datapath_in(gtwiz_INTF0_rst_tx_datapath_in),
.INTF0_TX_usrclk				(INTF0_TX_usrclk				)	,
.INTF0_rst_rx_done_out(INTF0_rst_rx_done_out),
.gtwiz_INTF0_rst_rx_datapath_in(gtwiz_INTF0_rst_rx_datapath_in),
.INTF0_RX_usrclk				(INTF0_RX_usrclk				)	,
.intf0_rate_sel  				(intf0_rate_sel  				)	,
.intf0_gpio_enable			(intf0_gpio_enable			)	,
.gtwiz_INTF0_rst_all_in (gtwiz_INTF0_rst_all_in )	,
.INTF1_link_status_out(INTF1_link_status_out),
.INTF1_rst_tx_done_out(INTF1_rst_tx_done_out),
.gtwiz_INTF1_rst_tx_datapath_in(gtwiz_INTF1_rst_tx_datapath_in),
.INTF1_TX_usrclk				(INTF1_TX_usrclk				)	,
.INTF1_rst_rx_done_out(INTF1_rst_rx_done_out),
.gtwiz_INTF1_rst_rx_datapath_in(gtwiz_INTF1_rst_rx_datapath_in),
.INTF1_RX_usrclk				(INTF1_RX_usrclk				)	,
.intf1_rate_sel  				(intf1_rate_sel  				)	,
.intf1_gpio_enable			(intf1_gpio_enable			)	,
.gtwiz_INTF1_rst_all_in (gtwiz_INTF1_rst_all_in )	,
.gtwiz_freerun_clock(gtwiz_freerun_clock),
.gt_powergood(gt_powergood)
     );
      

//INTERFACE 0 vio instance
intf0_axis_vio_top axis_vio_intf_0_inst (
.probe_in0(gt_powergood),   
.probe_in1(INTF0_link_status_out),  
.probe_out0(gtwiz_INTF0_rst_all_in),  
.probe_out1(intf0_rate_sel),  
.probe_out2(intf0_gpio_enable),  
.probe_in2(INTF0_rst_rx_done_out),    
.probe_in3(INTF0_rst_tx_done_out),    
.probe_out3(gtwiz_INTF0_rst_rx_datapath_in),  
.probe_out4(gtwiz_INTF0_rst_tx_datapath_in),  
.clk(gtwiz_freerun_clock)                
);
//INTERFACE 1 vio instance
intf1_axis_vio_top axis_vio_intf_1_inst (
.probe_in0(gt_powergood),   
.probe_in1(INTF1_link_status_out),  
.probe_out0(gtwiz_INTF1_rst_all_in),  
.probe_out1(intf1_rate_sel),  
.probe_out2(intf1_gpio_enable),  
.probe_in2(INTF1_rst_rx_done_out),    
.probe_in3(INTF1_rst_tx_done_out),    
.probe_out3(gtwiz_INTF1_rst_rx_datapath_in),  
.probe_out4(gtwiz_INTF1_rst_tx_datapath_in),  
.clk(gtwiz_freerun_clock)                
);




//As design generated with different top file cips will not be added to current hierarchy. So adding cips manually to current hierachy
design_0_wrapper design_0_i();


endmodule
