// (c) Copyright 2023 Advanced Micro Devices, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of AMD and is protected under U.S. and international copyright
// and other intellectual property laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// AMD, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND AMD HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) AMD shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or AMD had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// AMD products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of AMD products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
//-----------------------------------------------------------------

`timescale 1ns/1fs

module my_gtm_quad_top  # (
   parameter C_NO_OF_QUADS        = 1
) (
       input            gtwiz_freerun_clk,

       input            QUAD0_GTREFCLK0,
       input            QUAD0_GTREFCLK1,
       output           QUAD0_rxmarginreqack ,
       output [3:0]     QUAD0_rxmarginrescmd ,
       output [1:0]     QUAD0_rxmarginreslanenum ,
       output [7:0]     QUAD0_rxmarginrespayld ,
       output           QUAD0_rxmarginresreq ,
       input  [3:0]     QUAD0_rxmarginreqcmd ,
       input  [1:0]     QUAD0_rxmarginreqlanenum ,
       input  [7:0]     QUAD0_rxmarginreqpayld ,
       input            QUAD0_rxmarginreqreq ,
       input            QUAD0_rxmarginclk ,
       input            QUAD0_rxmarginresack ,
       input            QUAD0_pcielinkreachtarget ,
       input  [5:0]     QUAD0_pcieltssm ,
       input  [31:0]    QUAD0_gpi ,
       output [31:0]    QUAD0_gpo ,
       output [1:0]     QUAD0_hsclk0_rxrecclksel ,
       output [1:0]     QUAD0_hsclk1_rxrecclksel ,
       output           QUAD0_correcterr ,
       output           QUAD0_debugtracetvalid ,
       output [15:0]    QUAD0_debugtracetdata ,
       output           QUAD0_refclk0_gtrefclkpdint ,
       output           QUAD0_refclk0_clktestsigint ,
       output           QUAD0_refclk1_gtrefclkpdint ,
       output           QUAD0_refclk1_clktestsigint ,
       output           QUAD0_trigackin0 ,
       output           QUAD0_trigout0 ,
       output           QUAD0_ubinterrupt ,
       output           QUAD0_ubtxuart ,
       output           QUAD0_uncorrecterr ,
       output [7:0]     QUAD0_ctrlrsvdout ,
       input            QUAD0_bgbypassb ,
       input            QUAD0_bgmonitorenb ,
       input            QUAD0_bgpdb ,
       input            QUAD0_bgrcalovrdenb ,
       input  [4:0]     QUAD0_bgrcalovrd ,
       input            QUAD0_debugtraceready ,
       input            QUAD0_debugtraceclk ,
       input            QUAD0_trigackout0 ,
       input            QUAD0_trigin0 ,
       input            QUAD0_ubenable ,
       input            QUAD0_ubiolmbrst ,
       input            QUAD0_ubmbrst ,
       input  [11:0]    QUAD0_ubintr ,
       input            QUAD0_ubrxuart ,
       input            QUAD0_coestatusdebug ,
       input  [7:0]     QUAD0_ctrlrsvdin ,
       input            QUAD0_refclk0_clktestsig ,
       input            QUAD0_refclk1_clktestsig ,
       input            QUAD0_m0_axis_tready,
       output [31:0]    QUAD0_m0_axis_tdata,
       output           QUAD0_m0_axis_tlast,
       output           QUAD0_m0_axis_tvalid,
       output           QUAD0_s0_axis_tready,
       input  [31:0]    QUAD0_s0_axis_tdata,
       input            QUAD0_s0_axis_tlast,
       input            QUAD0_s0_axis_tvalid,
       output [1:0]     QUAD0_ch0_refdebugout,
       output           QUAD0_ch0_eyescandataerror,
       input            QUAD0_ch0_cdrfreqos,
       input            QUAD0_ch0_cdrincpctrl,
       input            QUAD0_ch0_cdrstepdir,
       input            QUAD0_ch0_cdrstepsq,
       input            QUAD0_ch0_cdrstepsx,
       input            QUAD0_ch0_eyescanreset,
       input            QUAD0_ch0_eyescantrigger,
            
       input            QUAD0_ch0_iloreset ,
       output           QUAD0_ch0_iloresetdone ,
       output           QUAD0_ch0_phystatus ,
       input            QUAD0_ch0_clkrsvd0 ,
       input            QUAD0_ch0_clkrsvd1 ,
       input            QUAD0_ch0_dmonitorclk ,
       input            QUAD0_ch0_iloresetmask ,
       input  [2:0]     QUAD0_ch0_loopback ,
       input            QUAD0_ch0_dmonfiforeset,
       input  [15:0]    QUAD0_ch0_pcsrsvdin ,
       input  [15:0]    QUAD0_ch0_gtrsvd ,
       output [15:0]    QUAD0_ch0_pcsrsvdout ,
       output [15:0]    QUAD0_ch0_pinrsvdas ,
       output           QUAD0_ch0_dmonitoroutclk,
       output           QUAD0_ch0_resetexception ,
       output [31:0]    QUAD0_ch0_dmonitorout,
       input            QUAD0_ch0_hsdppcsreset ,
       input            QUAD0_ch0_pcierstb ,
       input            QUAD0_m1_axis_tready,
       input  [31:0]    QUAD0_s1_axis_tdata,
       input            QUAD0_s1_axis_tlast,
       input            QUAD0_s1_axis_tvalid,
       output           QUAD0_s1_axis_tready,
       output [31:0]    QUAD0_m1_axis_tdata,
       output           QUAD0_m1_axis_tlast,
       output           QUAD0_m1_axis_tvalid,
       output [1:0]     QUAD0_ch1_refdebugout,
       output           QUAD0_ch1_eyescandataerror,
       input            QUAD0_ch1_cdrfreqos,
       input            QUAD0_ch1_cdrincpctrl,
       input            QUAD0_ch1_cdrstepdir,
       input            QUAD0_ch1_cdrstepsq,
       input            QUAD0_ch1_cdrstepsx,
       input            QUAD0_ch1_eyescanreset,
       input            QUAD0_ch1_eyescantrigger,
            
       input            QUAD0_ch1_iloreset ,
       output           QUAD0_ch1_iloresetdone ,
       output           QUAD0_ch1_phystatus ,
       input            QUAD0_ch1_clkrsvd0 ,
       input            QUAD0_ch1_clkrsvd1 ,
       input            QUAD0_ch1_dmonitorclk ,
       input            QUAD0_ch1_iloresetmask ,
       input  [2:0]     QUAD0_ch1_loopback ,
       input            QUAD0_ch1_dmonfiforeset,
       input  [15:0]    QUAD0_ch1_pcsrsvdin ,
       input  [15:0]    QUAD0_ch1_gtrsvd ,
       output [15:0]    QUAD0_ch1_pcsrsvdout ,
       output [15:0]    QUAD0_ch1_pinrsvdas ,
       output           QUAD0_ch1_dmonitoroutclk,
       output           QUAD0_ch1_resetexception ,
       output [31:0]    QUAD0_ch1_dmonitorout,
       input            QUAD0_ch1_hsdppcsreset ,
       input            QUAD0_ch1_pcierstb ,
       input  [31:0]    QUAD0_s2_axis_tdata,
       input            QUAD0_s2_axis_tlast,
       input            QUAD0_s2_axis_tvalid,
       output           QUAD0_s2_axis_tready,
       output [31:0]    QUAD0_m2_axis_tdata,
       output           QUAD0_m2_axis_tlast,
       output           QUAD0_m2_axis_tvalid,
       input            QUAD0_m2_axis_tready,
       output [1:0]     QUAD0_ch2_refdebugout,
       output           QUAD0_ch2_eyescandataerror,
       input            QUAD0_ch2_cdrfreqos,
       input            QUAD0_ch2_cdrincpctrl,
       input            QUAD0_ch2_cdrstepdir,
       input            QUAD0_ch2_cdrstepsq,
       input            QUAD0_ch2_cdrstepsx,
       input            QUAD0_ch2_eyescanreset,
       input            QUAD0_ch2_eyescantrigger,
            
       input            QUAD0_ch2_iloreset ,
       output           QUAD0_ch2_iloresetdone ,
       output           QUAD0_ch2_phystatus ,
       input            QUAD0_ch2_clkrsvd0 ,
       input            QUAD0_ch2_clkrsvd1 ,
       input            QUAD0_ch2_dmonitorclk ,
       input            QUAD0_ch2_iloresetmask ,
       input  [2:0]     QUAD0_ch2_loopback ,
       input            QUAD0_ch2_dmonfiforeset,
       input  [15:0]    QUAD0_ch2_pcsrsvdin ,
       input  [15:0]    QUAD0_ch2_gtrsvd ,
       output [15:0]    QUAD0_ch2_pcsrsvdout ,
       output [15:0]    QUAD0_ch2_pinrsvdas ,
       output           QUAD0_ch2_dmonitoroutclk,
       output           QUAD0_ch2_resetexception ,
       output [31:0]    QUAD0_ch2_dmonitorout,
       input            QUAD0_ch2_hsdppcsreset ,
       input            QUAD0_ch2_pcierstb ,
       input  [31:0]    QUAD0_s3_axis_tdata,
       input            QUAD0_s3_axis_tlast,
       input            QUAD0_s3_axis_tvalid,
       output           QUAD0_s3_axis_tready,
       output [31:0]    QUAD0_m3_axis_tdata,
       output           QUAD0_m3_axis_tlast,
       output           QUAD0_m3_axis_tvalid,
       input            QUAD0_m3_axis_tready,
       output [1:0]     QUAD0_ch3_refdebugout,
       output           QUAD0_ch3_eyescandataerror,
       input            QUAD0_ch3_cdrfreqos,
       input            QUAD0_ch3_cdrincpctrl,
       input            QUAD0_ch3_cdrstepdir,
       input            QUAD0_ch3_cdrstepsq,
       input            QUAD0_ch3_cdrstepsx,
       input            QUAD0_ch3_eyescanreset,
       input            QUAD0_ch3_eyescantrigger,
            
       input            QUAD0_ch3_iloreset ,
       output           QUAD0_ch3_iloresetdone ,
       output           QUAD0_ch3_phystatus ,
       input            QUAD0_ch3_clkrsvd0 ,
       input            QUAD0_ch3_clkrsvd1 ,
       input            QUAD0_ch3_dmonitorclk ,
       input            QUAD0_ch3_iloresetmask ,
       input  [2:0]     QUAD0_ch3_loopback ,
       input            QUAD0_ch3_dmonfiforeset,
       input  [15:0]    QUAD0_ch3_pcsrsvdin ,
       input  [15:0]    QUAD0_ch3_gtrsvd ,
       output [15:0]    QUAD0_ch3_pcsrsvdout ,
       output [15:0]    QUAD0_ch3_pinrsvdas ,
       output           QUAD0_ch3_dmonitoroutclk,
       output           QUAD0_ch3_resetexception ,
       output [31:0]    QUAD0_ch3_dmonitorout,
       input            QUAD0_ch3_hsdppcsreset ,
       input            QUAD0_ch3_pcierstb ,
       input            QUAD0_refclk0_gtrefclkpd,
       input            QUAD0_hsclk0_rpllreset,
       input            QUAD0_hsclk0_lcpllreset,
       output           QUAD0_hsclk0_lcpllfbclklost ,
       output           QUAD0_hsclk0_lcplllock ,
       output           QUAD0_hsclk0_lcpllrefclklost ,
       output           QUAD0_hsclk0_lcpllrefclkmonitor ,
       input  [7:0]     QUAD0_hsclk0_lcpllfbdiv ,
       input            QUAD0_hsclk0_lcpllpd ,
       input  [2:0]     QUAD0_hsclk0_lcpllrefclksel ,
       input            QUAD0_hsclk0_lcpllresetbypassmode ,
       input  [1:0]     QUAD0_hsclk0_lcpllresetmask ,
       input  [25:0]    QUAD0_hsclk0_lcpllsdmdata ,
       input            QUAD0_hsclk0_lcpllsdmtoggle ,
       output           QUAD0_hsclk0_rpllfbclklost ,
       output           QUAD0_hsclk0_rplllock ,
       output           QUAD0_hsclk0_rpllrefclklost ,
       output           QUAD0_hsclk0_rpllrefclkmonitor ,
       input  [7:0]     QUAD0_hsclk0_rpllfbdiv ,
       input            QUAD0_hsclk0_rpllpd ,
       input  [2:0]     QUAD0_hsclk0_rpllrefclksel ,
       input            QUAD0_hsclk0_rpllresetbypassmode ,
       input  [1:0]     QUAD0_hsclk0_rpllresetmask ,
       input  [25:0]    QUAD0_hsclk0_rpllsdmdata ,
       input            QUAD0_hsclk0_rpllsdmtoggle ,
       output           QUAD0_hsclk0_rxrecclkout0 ,
       output           QUAD0_hsclk0_rxrecclkout1 ,
       input            QUAD0_refclk1_gtrefclkpd,
       input            QUAD0_hsclk1_rpllreset,
       input            QUAD0_hsclk1_lcpllreset,
       output           QUAD0_hsclk1_lcpllfbclklost ,
       output           QUAD0_hsclk1_lcplllock ,
       output           QUAD0_hsclk1_lcpllrefclklost ,
       output           QUAD0_hsclk1_lcpllrefclkmonitor ,
       input  [7:0]     QUAD0_hsclk1_lcpllfbdiv ,
       input            QUAD0_hsclk1_lcpllpd ,
       input  [2:0]     QUAD0_hsclk1_lcpllrefclksel ,
       input            QUAD0_hsclk1_lcpllresetbypassmode ,
       input  [1:0]     QUAD0_hsclk1_lcpllresetmask ,
       input  [25:0]    QUAD0_hsclk1_lcpllsdmdata ,
       input            QUAD0_hsclk1_lcpllsdmtoggle ,
       output           QUAD0_hsclk1_rpllfbclklost ,
       output           QUAD0_hsclk1_rplllock ,
       output           QUAD0_hsclk1_rpllrefclklost ,
       output           QUAD0_hsclk1_rpllrefclkmonitor ,
       input  [7:0]     QUAD0_hsclk1_rpllfbdiv ,
       input            QUAD0_hsclk1_rpllpd ,
       input  [2:0]     QUAD0_hsclk1_rpllrefclksel ,
       input            QUAD0_hsclk1_rpllresetbypassmode ,
       input  [1:0]     QUAD0_hsclk1_rpllresetmask ,
       input  [25:0]    QUAD0_hsclk1_rpllsdmdata ,
       input            QUAD0_hsclk1_rpllsdmtoggle ,
       output           QUAD0_hsclk1_rxrecclkout0 ,
       output           QUAD0_hsclk1_rxrecclkout1 ,
       input  [3:0]     QUAD0_rxp ,
       input  [3:0]     QUAD0_rxn ,
       output [3:0]     QUAD0_txp ,
       output [3:0]     QUAD0_txn ,

         //output         QUAD0_ch0_gtytxn,
         //output         QUAD0_ch0_gtytxp,
         //input          QUAD0_ch0_gtyrxn,
         //input          QUAD0_ch0_gtyrxp,
         //output         QUAD0_ch0_bufgtclr,
         //output         QUAD0_ch0_bufgtclrmask,
         output [2:0]   QUAD0_RX0_bufstatus,
         output         QUAD0_RX0_debugpcsout,
         input          QUAD0_RX0_perst,
         input          QUAD0_RX0_prbscntstop,
         input          QUAD0_RX0_qprbsen,
         input          QUAD0_RX0_rxrsv0,
         input          QUAD0_RX0_rxrsv1,
         input          QUAD0_RX0_rxrsv2,
         output         QUAD0_RX0_cdrlock,
         output         QUAD0_RX0_cdrphdone,
         output [255:0] QUAD0_RX0_data,
         output         QUAD0_RX0_outclk,
         output         QUAD0_RX0_pmaresetdone,
         output         QUAD0_RX0_prbserr,
         output         QUAD0_RX0_prbslocked,
         output         QUAD0_RX0_progdivresetdone,
         output         QUAD0_RX0_resetdone,

         output         QUAD0_RX0_mstresetdone,
         input          QUAD0_RX0_mstreset,
         input          QUAD0_RX0_mstdatapathreset,
         input          QUAD0_RX0_gtrxreset,
         input          QUAD0_RX0_cdrhold,
         input          QUAD0_RX0_cdrovrden,
         input          QUAD0_RX0_cdrreset,
         input          QUAD0_RX0_latclk,
         input  [2:0]   QUAD0_RX0_pcsresetmask,
         input  [7:0]   QUAD0_RX0_pmaresetmask,
         input  [1:0]   QUAD0_RX0_pd,
         input          QUAD0_RX0_polarity,
         input          QUAD0_RX0_prbscntreset,
         input  [3:0]   QUAD0_RX0_prbssel,
         input          QUAD0_RX0_progdivreset,
         input  [7:0]   QUAD0_RX0_rate,
         input  [1:0]   QUAD0_RX0_resetmode,
         input          QUAD0_RX0_userrdy,
         input          QUAD0_RX0_usrclk,

         output         QUAD0_TX0_debugpcsout,
         output [2:0]   QUAD0_TX0_bufstatus,
         output         QUAD0_TX0_dccdone,
         output         QUAD0_TX0_outclk,
         output         QUAD0_TX0_pmaresetdone,
         output         QUAD0_TX0_progdivresetdone,
         output         QUAD0_TX0_resetdone,
         output         QUAD0_TX0_mstresetdone,
         input          QUAD0_TX0_mstreset,
         input          QUAD0_TX0_mstdatapathreset,
         input          QUAD0_TX0_gttxreset,
         input  [255:0] QUAD0_TX0_data,
         input  [5:0]   QUAD0_TX0_precursor2,
         input  [5:0]   QUAD0_TX0_precursor3,
         input  [4:0]   QUAD0_TX0_txrsv0,
         input  [3:0]   QUAD0_TX0_txrsv1,
         input          QUAD0_TX0_perst,
         input          QUAD0_TX0_qprbsen,
         input          QUAD0_TX0_inhibit,
         input          QUAD0_TX0_latclk,
         input  [6:0]   QUAD0_TX0_maincursor,
         input          QUAD0_TX0_pcsresetmask,
         input  [1:0]   QUAD0_TX0_pd,
         input          QUAD0_TX0_pisopd,
         input  [1:0]   QUAD0_TX0_pmaresetmask,
         input  [5:0]   QUAD0_TX0_postcursor,
         input  [5:0]   QUAD0_TX0_precursor,
         input          QUAD0_TX0_polarity,
         input          QUAD0_TX0_prbsforceerr,
         input  [3:0]   QUAD0_TX0_prbssel,
         input          QUAD0_TX0_progdivreset,
         input  [7:0]   QUAD0_TX0_rate,
         input  [1:0]   QUAD0_TX0_resetmode,
         input          QUAD0_TX0_userrdy,
         input          QUAD0_TX0_usrclk,
         //output         QUAD0_ch1_gtytxn,
         //output         QUAD0_ch1_gtytxp,
         //input          QUAD0_ch1_gtyrxn,
         //input          QUAD0_ch1_gtyrxp,
         //output         QUAD0_ch1_bufgtclr,
         //output         QUAD0_ch1_bufgtclrmask,
         output [2:0]   QUAD0_RX1_bufstatus,
         output         QUAD0_RX1_debugpcsout,
         input          QUAD0_RX1_perst,
         input          QUAD0_RX1_prbscntstop,
         input          QUAD0_RX1_qprbsen,
         input          QUAD0_RX1_rxrsv0,
         input          QUAD0_RX1_rxrsv1,
         input          QUAD0_RX1_rxrsv2,
         output         QUAD0_RX1_cdrlock,
         output         QUAD0_RX1_cdrphdone,
         output [255:0] QUAD0_RX1_data,
         output         QUAD0_RX1_outclk,
         output         QUAD0_RX1_pmaresetdone,
         output         QUAD0_RX1_prbserr,
         output         QUAD0_RX1_prbslocked,
         output         QUAD0_RX1_progdivresetdone,
         output         QUAD0_RX1_resetdone,

         output         QUAD0_RX1_mstresetdone,
         input          QUAD0_RX1_mstreset,
         input          QUAD0_RX1_mstdatapathreset,
         input          QUAD0_RX1_gtrxreset,
         input          QUAD0_RX1_cdrhold,
         input          QUAD0_RX1_cdrovrden,
         input          QUAD0_RX1_cdrreset,
         input          QUAD0_RX1_latclk,
         input  [2:0]   QUAD0_RX1_pcsresetmask,
         input  [7:0]   QUAD0_RX1_pmaresetmask,
         input  [1:0]   QUAD0_RX1_pd,
         input          QUAD0_RX1_polarity,
         input          QUAD0_RX1_prbscntreset,
         input  [3:0]   QUAD0_RX1_prbssel,
         input          QUAD0_RX1_progdivreset,
         input  [7:0]   QUAD0_RX1_rate,
         input  [1:0]   QUAD0_RX1_resetmode,
         input          QUAD0_RX1_userrdy,
         input          QUAD0_RX1_usrclk,

         output         QUAD0_TX1_debugpcsout,
         output [2:0]   QUAD0_TX1_bufstatus,
         output         QUAD0_TX1_dccdone,
         output         QUAD0_TX1_outclk,
         output         QUAD0_TX1_pmaresetdone,
         output         QUAD0_TX1_progdivresetdone,
         output         QUAD0_TX1_resetdone,
         output         QUAD0_TX1_mstresetdone,
         input          QUAD0_TX1_mstreset,
         input          QUAD0_TX1_mstdatapathreset,
         input          QUAD0_TX1_gttxreset,
         input  [255:0] QUAD0_TX1_data,
         input  [5:0]   QUAD0_TX1_precursor2,
         input  [5:0]   QUAD0_TX1_precursor3,
         input  [4:0]   QUAD0_TX1_txrsv0,
         input  [3:0]   QUAD0_TX1_txrsv1,
         input          QUAD0_TX1_perst,
         input          QUAD0_TX1_qprbsen,
         input          QUAD0_TX1_inhibit,
         input          QUAD0_TX1_latclk,
         input  [6:0]   QUAD0_TX1_maincursor,
         input          QUAD0_TX1_pcsresetmask,
         input  [1:0]   QUAD0_TX1_pd,
         input          QUAD0_TX1_pisopd,
         input  [1:0]   QUAD0_TX1_pmaresetmask,
         input  [5:0]   QUAD0_TX1_postcursor,
         input  [5:0]   QUAD0_TX1_precursor,
         input          QUAD0_TX1_polarity,
         input          QUAD0_TX1_prbsforceerr,
         input  [3:0]   QUAD0_TX1_prbssel,
         input          QUAD0_TX1_progdivreset,
         input  [7:0]   QUAD0_TX1_rate,
         input  [1:0]   QUAD0_TX1_resetmode,
         input          QUAD0_TX1_userrdy,
         input          QUAD0_TX1_usrclk,
         //output         QUAD0_ch2_gtytxn,
         //output         QUAD0_ch2_gtytxp,
         //input          QUAD0_ch2_gtyrxn,
         //input          QUAD0_ch2_gtyrxp,
         //output         QUAD0_ch2_bufgtclr,
         //output         QUAD0_ch2_bufgtclrmask,
         output [2:0]   QUAD0_RX2_bufstatus,
         output         QUAD0_RX2_debugpcsout,
         input          QUAD0_RX2_perst,
         input          QUAD0_RX2_prbscntstop,
         input          QUAD0_RX2_qprbsen,
         input          QUAD0_RX2_rxrsv0,
         input          QUAD0_RX2_rxrsv1,
         input          QUAD0_RX2_rxrsv2,
         output         QUAD0_RX2_cdrlock,
         output         QUAD0_RX2_cdrphdone,
         output [255:0] QUAD0_RX2_data,
         output         QUAD0_RX2_outclk,
         output         QUAD0_RX2_pmaresetdone,
         output         QUAD0_RX2_prbserr,
         output         QUAD0_RX2_prbslocked,
         output         QUAD0_RX2_progdivresetdone,
         output         QUAD0_RX2_resetdone,

         output         QUAD0_RX2_mstresetdone,
         input          QUAD0_RX2_mstreset,
         input          QUAD0_RX2_mstdatapathreset,
         input          QUAD0_RX2_gtrxreset,
         input          QUAD0_RX2_cdrhold,
         input          QUAD0_RX2_cdrovrden,
         input          QUAD0_RX2_cdrreset,
         input          QUAD0_RX2_latclk,
         input  [2:0]   QUAD0_RX2_pcsresetmask,
         input  [7:0]   QUAD0_RX2_pmaresetmask,
         input  [1:0]   QUAD0_RX2_pd,
         input          QUAD0_RX2_polarity,
         input          QUAD0_RX2_prbscntreset,
         input  [3:0]   QUAD0_RX2_prbssel,
         input          QUAD0_RX2_progdivreset,
         input  [7:0]   QUAD0_RX2_rate,
         input  [1:0]   QUAD0_RX2_resetmode,
         input          QUAD0_RX2_userrdy,
         input          QUAD0_RX2_usrclk,

         output         QUAD0_TX2_debugpcsout,
         output [2:0]   QUAD0_TX2_bufstatus,
         output         QUAD0_TX2_dccdone,
         output         QUAD0_TX2_outclk,
         output         QUAD0_TX2_pmaresetdone,
         output         QUAD0_TX2_progdivresetdone,
         output         QUAD0_TX2_resetdone,
         output         QUAD0_TX2_mstresetdone,
         input          QUAD0_TX2_mstreset,
         input          QUAD0_TX2_mstdatapathreset,
         input          QUAD0_TX2_gttxreset,
         input  [255:0] QUAD0_TX2_data,
         input  [5:0]   QUAD0_TX2_precursor2,
         input  [5:0]   QUAD0_TX2_precursor3,
         input  [4:0]   QUAD0_TX2_txrsv0,
         input  [3:0]   QUAD0_TX2_txrsv1,
         input          QUAD0_TX2_perst,
         input          QUAD0_TX2_qprbsen,
         input          QUAD0_TX2_inhibit,
         input          QUAD0_TX2_latclk,
         input  [6:0]   QUAD0_TX2_maincursor,
         input          QUAD0_TX2_pcsresetmask,
         input  [1:0]   QUAD0_TX2_pd,
         input          QUAD0_TX2_pisopd,
         input  [1:0]   QUAD0_TX2_pmaresetmask,
         input  [5:0]   QUAD0_TX2_postcursor,
         input  [5:0]   QUAD0_TX2_precursor,
         input          QUAD0_TX2_polarity,
         input          QUAD0_TX2_prbsforceerr,
         input  [3:0]   QUAD0_TX2_prbssel,
         input          QUAD0_TX2_progdivreset,
         input  [7:0]   QUAD0_TX2_rate,
         input  [1:0]   QUAD0_TX2_resetmode,
         input          QUAD0_TX2_userrdy,
         input          QUAD0_TX2_usrclk,
         //output         QUAD0_ch3_gtytxn,
         //output         QUAD0_ch3_gtytxp,
         //input          QUAD0_ch3_gtyrxn,
         //input          QUAD0_ch3_gtyrxp,
         //output         QUAD0_ch3_bufgtclr,
         //output         QUAD0_ch3_bufgtclrmask,
         output [2:0]   QUAD0_RX3_bufstatus,
         output         QUAD0_RX3_debugpcsout,
         input          QUAD0_RX3_perst,
         input          QUAD0_RX3_prbscntstop,
         input          QUAD0_RX3_qprbsen,
         input          QUAD0_RX3_rxrsv0,
         input          QUAD0_RX3_rxrsv1,
         input          QUAD0_RX3_rxrsv2,
         output         QUAD0_RX3_cdrlock,
         output         QUAD0_RX3_cdrphdone,
         output [255:0] QUAD0_RX3_data,
         output         QUAD0_RX3_outclk,
         output         QUAD0_RX3_pmaresetdone,
         output         QUAD0_RX3_prbserr,
         output         QUAD0_RX3_prbslocked,
         output         QUAD0_RX3_progdivresetdone,
         output         QUAD0_RX3_resetdone,

         output         QUAD0_RX3_mstresetdone,
         input          QUAD0_RX3_mstreset,
         input          QUAD0_RX3_mstdatapathreset,
         input          QUAD0_RX3_gtrxreset,
         input          QUAD0_RX3_cdrhold,
         input          QUAD0_RX3_cdrovrden,
         input          QUAD0_RX3_cdrreset,
         input          QUAD0_RX3_latclk,
         input  [2:0]   QUAD0_RX3_pcsresetmask,
         input  [7:0]   QUAD0_RX3_pmaresetmask,
         input  [1:0]   QUAD0_RX3_pd,
         input          QUAD0_RX3_polarity,
         input          QUAD0_RX3_prbscntreset,
         input  [3:0]   QUAD0_RX3_prbssel,
         input          QUAD0_RX3_progdivreset,
         input  [7:0]   QUAD0_RX3_rate,
         input  [1:0]   QUAD0_RX3_resetmode,
         input          QUAD0_RX3_userrdy,
         input          QUAD0_RX3_usrclk,

         output         QUAD0_TX3_debugpcsout,
         output [2:0]   QUAD0_TX3_bufstatus,
         output         QUAD0_TX3_dccdone,
         output         QUAD0_TX3_outclk,
         output         QUAD0_TX3_pmaresetdone,
         output         QUAD0_TX3_progdivresetdone,
         output         QUAD0_TX3_resetdone,
         output         QUAD0_TX3_mstresetdone,
         input          QUAD0_TX3_mstreset,
         input          QUAD0_TX3_mstdatapathreset,
         input          QUAD0_TX3_gttxreset,
         input  [255:0] QUAD0_TX3_data,
         input  [5:0]   QUAD0_TX3_precursor2,
         input  [5:0]   QUAD0_TX3_precursor3,
         input  [4:0]   QUAD0_TX3_txrsv0,
         input  [3:0]   QUAD0_TX3_txrsv1,
         input          QUAD0_TX3_perst,
         input          QUAD0_TX3_qprbsen,
         input          QUAD0_TX3_inhibit,
         input          QUAD0_TX3_latclk,
         input  [6:0]   QUAD0_TX3_maincursor,
         input          QUAD0_TX3_pcsresetmask,
         input  [1:0]   QUAD0_TX3_pd,
         input          QUAD0_TX3_pisopd,
         input  [1:0]   QUAD0_TX3_pmaresetmask,
         input  [5:0]   QUAD0_TX3_postcursor,
         input  [5:0]   QUAD0_TX3_precursor,
         input          QUAD0_TX3_polarity,
         input          QUAD0_TX3_prbsforceerr,
         input  [3:0]   QUAD0_TX3_prbssel,
         input          QUAD0_TX3_progdivreset,
         input  [7:0]   QUAD0_TX3_rate,
         input  [1:0]   QUAD0_TX3_resetmode,
         input          QUAD0_TX3_userrdy,
         input          QUAD0_TX3_usrclk,

         output         QUAD0_gtpowergood

);

/*       input  */ wire           QUAD_altclk = 1'b0;
//Northout_Southin_intf for last quad
/*       output */ wire [1:0]   QUAD0_resetdone_northout;
/*       input  */ wire [1:0]   QUAD0_resetdone_southin = 2'b0;
/*       output */ wire [3:0]   QUAD0_txpinorthout;
/*       output */ wire [3:0]   QUAD0_rxpinorthout;
/*       input  */ wire [3:0]   QUAD0_txpisouthin = 4'b0;
/*       input  */ wire [3:0]   QUAD0_rxpisouthin = 4'b0;
/*       output */ wire [5:0]   QUAD0_pipenorthout;
/*       input  */ wire [5:0]   QUAD0_pipesouthin = 6'b0;
//Northin_Southout_intf for first quad
/*       input  */ wire [1:0]   QUAD0_resetdone_northin = 2'b0;
/*       output */ wire [1:0]   QUAD0_resetdone_southout;
/*       input  */ wire [3:0]   QUAD0_txpinorthin = 4'b0;
/*       input  */ wire [3:0]   QUAD0_rxpinorthin = 4'b0;
/*       output */ wire [3:0]   QUAD0_txpisouthout;
/*       output */ wire [3:0]   QUAD0_rxpisouthout;
/*       input  */ wire [5:0]   QUAD0_pipenorthin = 6'b0;
/*       output */ wire [5:0]   QUAD0_pipesouthout;





my_gtm_gt_quad_base_0 gt_quad_base_0_inst (
         .GT_REFCLK0          ( QUAD0_GTREFCLK0) ,
         .GT_REFCLK1          ( QUAD0_GTREFCLK1) ,
         .rxp                         ( QUAD0_rxp) ,
         .rxn                         ( QUAD0_rxn) ,
         .txp                         ( QUAD0_txp) ,
         .txn                         ( QUAD0_txn) ,
         .apb3clk                     ( gtwiz_freerun_clk) ,
         .apb3presetn                 ( 1'b1) ,
         .apb3paddr                   ( 16'b0),
         .apb3penable                 ( 1'b0),
         .apb3psel                    ( 1'b0),
         .apb3pwdata                  ( 32'b0),
         .apb3pwrite                  ( 1'b0),
         .apb3prdata                  ( ),
         .apb3pslverr                 ( ),
         .apb3pready                  ( ),
         .rxmarginreqack              ( QUAD0_rxmarginreqack) ,
         .rxmarginrescmd              ( QUAD0_rxmarginrescmd) ,
         .rxmarginreslanenum          ( QUAD0_rxmarginreslanenum) ,
         .rxmarginrespayld            ( QUAD0_rxmarginrespayld) ,
         .rxmarginresreq              ( QUAD0_rxmarginresreq) ,
         .rxmarginreqcmd              ( QUAD0_rxmarginreqcmd) ,
         .rxmarginreqlanenum          ( QUAD0_rxmarginreqlanenum) ,
         .rxmarginreqpayld            ( QUAD0_rxmarginreqpayld) ,
         .rxmarginreqreq              ( QUAD0_rxmarginreqreq) ,
         .rxmarginclk                 ( QUAD0_rxmarginclk) ,
         .rxmarginresack              ( QUAD0_rxmarginresack) ,
         .pcielinkreachtarget         ( QUAD0_pcielinkreachtarget) ,
         .pcieltssm                   ( QUAD0_pcieltssm) ,
         .gpi                         ( QUAD0_gpi) ,
         .gpo                         ( QUAD0_gpo) ,
         .hsclk0_rxrecclksel          ( QUAD0_hsclk0_rxrecclksel) ,
         .hsclk1_rxrecclksel          ( QUAD0_hsclk1_rxrecclksel) ,
         .correcterr                  ( QUAD0_correcterr) ,
         .debugtracetvalid            ( QUAD0_debugtracetvalid) ,
         .debugtracetdata             ( QUAD0_debugtracetdata) ,
         .refclk0_gtrefclkpdint       ( QUAD0_refclk0_gtrefclkpdint) ,
         .refclk0_clktestsigint       ( QUAD0_refclk0_clktestsigint) ,
         .refclk1_gtrefclkpdint       ( QUAD0_refclk1_gtrefclkpdint) ,
         .refclk1_clktestsigint       ( QUAD0_refclk1_clktestsigint) ,
         .trigackin0                  ( QUAD0_trigackin0) ,
         .trigout0                    ( QUAD0_trigout0) ,
         .ubinterrupt                 ( QUAD0_ubinterrupt) ,
         .ubtxuart                    ( QUAD0_ubtxuart) ,
         .uncorrecterr                ( QUAD0_uncorrecterr) ,
         .ctrlrsvdout                 ( QUAD0_ctrlrsvdout) ,
         .bgbypassb                   ( QUAD0_bgbypassb) ,
         .bgmonitorenb                ( QUAD0_bgmonitorenb) ,
         .bgpdb                       ( QUAD0_bgpdb) ,
         .bgrcalovrdenb               ( QUAD0_bgrcalovrdenb) ,
         .bgrcalovrd                  ( QUAD0_bgrcalovrd) ,
         .debugtraceready             ( QUAD0_debugtraceready) ,
         .debugtraceclk               ( QUAD0_debugtraceclk) ,
         .trigackout0                 ( QUAD0_trigackout0) ,
         .trigin0                     ( QUAD0_trigin0) ,
         .ubenable                    ( QUAD0_ubenable) ,
         .ubiolmbrst                  ( QUAD0_ubiolmbrst) ,
         .ubmbrst                     ( QUAD0_ubmbrst) ,
         .ubintr                      ( QUAD0_ubintr) ,
         .ubrxuart                    ( QUAD0_ubrxuart) ,
         .coestatusdebug              ( QUAD0_coestatusdebug) ,
         .ctrlrsvdin                  ( QUAD0_ctrlrsvdin) ,
         .refclk0_clktestsig          ( QUAD0_refclk0_clktestsig) ,
         .refclk1_clktestsig          ( QUAD0_refclk1_clktestsig) ,
         .resetdone_northin           ( QUAD0_resetdone_northin) ,
         .resetdone_southin           ( QUAD0_resetdone_southin) ,
         .resetdone_northout          ( QUAD0_resetdone_northout) ,
         .resetdone_southout          ( QUAD0_resetdone_southout) ,
         .refclk0_gtrefclkpd          ( QUAD0_refclk0_gtrefclkpd) ,
         .hsclk0_rxrecclkout0         ( QUAD0_hsclk0_rxrecclkout0) ,
         .hsclk0_rxrecclkout1         ( QUAD0_hsclk0_rxrecclkout1) ,
         .hsclk0_lcpllfbclklost       ( QUAD0_hsclk0_lcpllfbclklost) ,
         .hsclk0_lcplllock            ( QUAD0_hsclk0_lcplllock) ,
         .hsclk0_lcpllrefclklost      ( QUAD0_hsclk0_lcpllrefclklost) ,
         .hsclk0_lcpllrefclkmonitor   ( QUAD0_hsclk0_lcpllrefclkmonitor) ,
         .hsclk0_lcpllfbdiv           ( QUAD0_hsclk0_lcpllfbdiv) ,
         .hsclk0_lcpllpd              ( QUAD0_hsclk0_lcpllpd) ,
         .hsclk0_lcpllrefclksel       ( QUAD0_hsclk0_lcpllrefclksel) ,
         .hsclk0_lcpllreset           ( QUAD0_hsclk0_lcpllreset) ,
         .hsclk0_lcpllresetbypassmode ( QUAD0_hsclk0_lcpllresetbypassmode) ,
         .hsclk0_lcpllresetmask       ( QUAD0_hsclk0_lcpllresetmask) ,
         .hsclk0_lcpllsdmdata         ( QUAD0_hsclk0_lcpllsdmdata) ,
         .hsclk0_lcpllsdmtoggle       ( QUAD0_hsclk0_lcpllsdmtoggle) ,
         .hsclk0_rpllfbclklost        ( QUAD0_hsclk0_rpllfbclklost) ,
         .hsclk0_rplllock             ( QUAD0_hsclk0_rplllock) ,
         .hsclk0_rpllrefclklost       ( QUAD0_hsclk0_rpllrefclklost) ,
         .hsclk0_rpllrefclkmonitor    ( QUAD0_hsclk0_rpllrefclkmonitor) ,
         .hsclk0_rpllfbdiv            ( QUAD0_hsclk0_rpllfbdiv) ,
         .hsclk0_rpllpd               ( QUAD0_hsclk0_rpllpd) ,
         .hsclk0_rpllrefclksel        ( QUAD0_hsclk0_rpllrefclksel) ,
         .hsclk0_rpllreset            ( QUAD0_hsclk0_rpllreset) ,
         .hsclk0_rpllresetbypassmode  ( QUAD0_hsclk0_rpllresetbypassmode) ,
         .hsclk0_rpllresetmask        ( QUAD0_hsclk0_rpllresetmask) ,
         .hsclk0_rpllsdmdata          ( QUAD0_hsclk0_rpllsdmdata) ,
         .hsclk0_rpllsdmtoggle        ( QUAD0_hsclk0_rpllsdmtoggle) ,
         .refclk1_gtrefclkpd          ( QUAD0_refclk1_gtrefclkpd) ,
         .hsclk1_rxrecclkout0         ( QUAD0_hsclk1_rxrecclkout0) ,
         .hsclk1_rxrecclkout1         ( QUAD0_hsclk1_rxrecclkout1) ,
         .hsclk1_lcpllfbclklost       ( QUAD0_hsclk1_lcpllfbclklost) ,
         .hsclk1_lcplllock            ( QUAD0_hsclk1_lcplllock) ,
         .hsclk1_lcpllrefclklost      ( QUAD0_hsclk1_lcpllrefclklost) ,
         .hsclk1_lcpllrefclkmonitor   ( QUAD0_hsclk1_lcpllrefclkmonitor) ,
         .hsclk1_lcpllfbdiv           ( QUAD0_hsclk1_lcpllfbdiv) ,
         .hsclk1_lcpllpd              ( QUAD0_hsclk1_lcpllpd) ,
         .hsclk1_lcpllrefclksel       ( QUAD0_hsclk1_lcpllrefclksel) ,
         .hsclk1_lcpllreset           ( QUAD0_hsclk1_lcpllreset) ,
         .hsclk1_lcpllresetbypassmode ( QUAD0_hsclk1_lcpllresetbypassmode) ,
         .hsclk1_lcpllresetmask       ( QUAD0_hsclk1_lcpllresetmask) ,
         .hsclk1_lcpllsdmdata         ( QUAD0_hsclk1_lcpllsdmdata) ,
         .hsclk1_lcpllsdmtoggle       ( QUAD0_hsclk1_lcpllsdmtoggle) ,
         .hsclk1_rpllfbclklost        ( QUAD0_hsclk1_rpllfbclklost) ,
         .hsclk1_rplllock             ( QUAD0_hsclk1_rplllock) ,
         .hsclk1_rpllrefclklost       ( QUAD0_hsclk1_rpllrefclklost) ,
         .hsclk1_rpllrefclkmonitor    ( QUAD0_hsclk1_rpllrefclkmonitor) ,
         .hsclk1_rpllfbdiv            ( QUAD0_hsclk1_rpllfbdiv) ,
         .hsclk1_rpllpd               ( QUAD0_hsclk1_rpllpd) ,
         .hsclk1_rpllrefclksel        ( QUAD0_hsclk1_rpllrefclksel) ,
         .hsclk1_rpllreset            ( QUAD0_hsclk1_rpllreset) ,
         .hsclk1_rpllresetbypassmode  ( QUAD0_hsclk1_rpllresetbypassmode) ,
         .hsclk1_rpllresetmask        ( QUAD0_hsclk1_rpllresetmask) ,
         .hsclk1_rpllsdmdata          ( QUAD0_hsclk1_rpllsdmdata) ,
         .hsclk1_rpllsdmtoggle        ( QUAD0_hsclk1_rpllsdmtoggle) ,
         .ch0_pcsrsvdout           ( QUAD0_ch0_pcsrsvdout),
         .ch0_pinrsvdas            ( QUAD0_ch0_pinrsvdas),
         .ch0_dmonitorout          ( QUAD0_ch0_dmonitorout),
         .ch0_dmonitoroutclk       ( QUAD0_ch0_dmonitoroutclk),
         .ch0_resetexception       ( QUAD0_ch0_resetexception),
         .ch0_gtrsvd               ( QUAD0_ch0_gtrsvd),
         .ch0_pcsrsvdin            ( QUAD0_ch0_pcsrsvdin),
         .ch0_loopback             ( QUAD0_ch0_loopback),
         .ch0_iloreset             ( QUAD0_ch0_iloreset),
         .ch0_iloresetdone         ( QUAD0_ch0_iloresetdone),
         .ch0_phystatus            ( QUAD0_ch0_phystatus),
         .ch0_clkrsvd0             ( QUAD0_ch0_clkrsvd0),
         .ch0_clkrsvd1             ( QUAD0_ch0_clkrsvd1),
         .ch0_dmonfiforeset        ( QUAD0_ch0_dmonfiforeset),
         .ch0_dmonitorclk          ( QUAD0_ch0_dmonitorclk),
         .ch0_iloresetmask         ( QUAD0_ch0_iloresetmask),
         .ch0_pcierstb             ( QUAD0_ch0_pcierstb),
         .ch0_gttxreset            ( QUAD0_TX0_gttxreset),
         .ch0_txbufstatus          ( QUAD0_TX0_bufstatus),
         .ch0_txdccdone            ( QUAD0_TX0_dccdone),
         .ch0_txdebugpcsout        ( QUAD0_TX0_debugpcsout),
         .ch0_txpmaresetdone       ( QUAD0_TX0_pmaresetdone),
         .ch0_txprogdivresetdone   ( QUAD0_TX0_progdivresetdone),
         .ch0_txresetdone          ( QUAD0_TX0_resetdone),
         .ch0_txdata               ( QUAD0_TX0_data),
         .ch0_txpd                 ( QUAD0_TX0_pd),
         .ch0_txpmaresetmask       ( QUAD0_TX0_pmaresetmask),
         .ch0_txresetmode          ( QUAD0_TX0_resetmode),
         .ch0_txprbssel            ( QUAD0_TX0_prbssel),
         .ch0_txrsv0               ( QUAD0_TX0_txrsv0),
         .ch0_txrsv1               ( QUAD0_TX0_txrsv1),
         .ch0_txpostcursor         ( QUAD0_TX0_postcursor),
         .ch0_txprecursor          ( QUAD0_TX0_precursor),
         .ch0_txprecursor2         ( QUAD0_TX0_precursor2),
         .ch0_txprecursor3         ( QUAD0_TX0_precursor3),
         .ch0_txmaincursor         ( QUAD0_TX0_maincursor),
         .ch0_txrate               ( QUAD0_TX0_rate),
         .ch0_txinhibit            ( QUAD0_TX0_inhibit),
         .ch0_txlatclk             ( QUAD0_TX0_latclk),
         .ch0_txpcsresetmask       ( QUAD0_TX0_pcsresetmask),
         .ch0_txperst              ( QUAD0_TX0_perst),
         .ch0_txpisopd             ( QUAD0_TX0_pisopd),
         .ch0_txpolarity           ( QUAD0_TX0_polarity),
         .ch0_txprbsforceerr       ( QUAD0_TX0_prbsforceerr),
         .ch0_txprogdivreset       ( QUAD0_TX0_progdivreset),
         .ch0_txqprbsen            ( QUAD0_TX0_qprbsen),
         .ch0_txuserrdy            ( QUAD0_TX0_userrdy),
         .ch0_txmstresetdone       ( QUAD0_TX0_mstresetdone),
         .ch0_txmstreset           ( QUAD0_TX0_mstreset),
         .ch0_txmstdatapathreset   ( QUAD0_TX0_mstdatapathreset),
         .ch0_txoutclk             ( QUAD0_TX0_outclk),
         .ch0_txusrclk             ( QUAD0_TX0_usrclk),

         .m0_axis_tready           ( QUAD0_m0_axis_tready),
         .m0_axis_tdata            ( QUAD0_m0_axis_tdata),
         .m0_axis_tlast            ( QUAD0_m0_axis_tlast),
         .m0_axis_tvalid           ( QUAD0_m0_axis_tvalid),
         .s0_axis_tready           ( QUAD0_s0_axis_tready),
         .s0_axis_tdata            ( QUAD0_s0_axis_tdata),
         .s0_axis_tlast            ( QUAD0_s0_axis_tlast),
         .s0_axis_tvalid           ( QUAD0_s0_axis_tvalid),
         .ch0_refdebugout          ( QUAD0_ch0_refdebugout),      
         .ch0_eyescandataerror     ( QUAD0_ch0_eyescandataerror), 
         .ch0_cdrfreqos            ( QUAD0_ch0_cdrfreqos),
         .ch0_cdrincpctrl          ( QUAD0_ch0_cdrincpctrl),
         .ch0_cdrstepdir           ( QUAD0_ch0_cdrstepdir),
         .ch0_cdrstepsq            ( QUAD0_ch0_cdrstepsq),
         .ch0_cdrstepsx            ( QUAD0_ch0_cdrstepsx),
         .ch0_eyescanreset         ( QUAD0_ch0_eyescanreset),
         .ch0_eyescantrigger       ( QUAD0_ch0_eyescantrigger),
         .ch0_gtrxreset            ( QUAD0_RX0_gtrxreset),
         .ch0_rxdata               ( QUAD0_RX0_data),
         .ch0_rxbufstatus          ( QUAD0_RX0_bufstatus),
         .ch0_rxcdrlock            ( QUAD0_RX0_cdrlock),
         .ch0_rxcdrphdone          ( QUAD0_RX0_cdrphdone),
         .ch0_rxdebugpcsout        ( QUAD0_RX0_debugpcsout),
         .ch0_rxpmaresetdone       ( QUAD0_RX0_pmaresetdone),
         .ch0_rxprbserr            ( QUAD0_RX0_prbserr),
         .ch0_rxprbslocked         ( QUAD0_RX0_prbslocked),
         .ch0_rxprogdivresetdone   ( QUAD0_RX0_progdivresetdone),
         .ch0_rxresetdone          ( QUAD0_RX0_resetdone),
         .ch0_rxpd                 ( QUAD0_RX0_pd),
         .ch0_rxresetmode          ( QUAD0_RX0_resetmode),
         .ch0_rxpcsresetmask       ( QUAD0_RX0_pcsresetmask),
         .ch0_rxprbssel            ( QUAD0_RX0_prbssel),
         .ch0_rxpmaresetmask       ( QUAD0_RX0_pmaresetmask),
         .ch0_rxrate               ( QUAD0_RX0_rate),
         .ch0_rxcdrhold            ( QUAD0_RX0_cdrhold),
         .ch0_rxcdrovrden          ( QUAD0_RX0_cdrovrden),
         .ch0_rxcdrreset           ( QUAD0_RX0_cdrreset),
         .ch0_rxlatclk             ( QUAD0_RX0_latclk),
         .ch0_rxperst              ( QUAD0_RX0_perst),
         .ch0_rxpolarity           ( QUAD0_RX0_polarity),
         .ch0_rxprbscntreset       ( QUAD0_RX0_prbscntreset),
         .ch0_rxprbscntstop        ( QUAD0_RX0_prbscntstop),
         .ch0_rxprogdivreset       ( QUAD0_RX0_progdivreset),
         .ch0_rxqprbsen            ( QUAD0_RX0_qprbsen),
         .ch0_rxrsv0               ( QUAD0_RX0_rxrsv0),
         .ch0_rxrsv1               ( QUAD0_RX0_rxrsv1),
         .ch0_rxrsv2               ( QUAD0_RX0_rxrsv2),
         .ch0_rxuserrdy            ( QUAD0_RX0_userrdy),
         .ch0_rxmstresetdone       ( QUAD0_RX0_mstresetdone),
         .ch0_rxmstreset           ( QUAD0_RX0_mstreset),
         .ch0_rxmstdatapathreset   ( QUAD0_RX0_mstdatapathreset),
         .ch0_rxoutclk             ( QUAD0_RX0_outclk),
         .ch0_rxusrclk             ( QUAD0_RX0_usrclk),
         .ch1_pcsrsvdout           ( QUAD0_ch1_pcsrsvdout),
         .ch1_pinrsvdas            ( QUAD0_ch1_pinrsvdas),
         .ch1_dmonitorout          ( QUAD0_ch1_dmonitorout),
         .ch1_dmonitoroutclk       ( QUAD0_ch1_dmonitoroutclk),
         .ch1_resetexception       ( QUAD0_ch1_resetexception),
         .ch1_gtrsvd               ( QUAD0_ch1_gtrsvd),
         .ch1_pcsrsvdin            ( QUAD0_ch1_pcsrsvdin),
         .ch1_loopback             ( QUAD0_ch1_loopback),
         .ch1_iloreset             ( QUAD0_ch1_iloreset),
         .ch1_iloresetdone         ( QUAD0_ch1_iloresetdone),
         .ch1_phystatus            ( QUAD0_ch1_phystatus),
         .ch1_clkrsvd0             ( QUAD0_ch1_clkrsvd0),
         .ch1_clkrsvd1             ( QUAD0_ch1_clkrsvd1),
         .ch1_dmonfiforeset        ( QUAD0_ch1_dmonfiforeset),
         .ch1_dmonitorclk          ( QUAD0_ch1_dmonitorclk),
         .ch1_iloresetmask         ( QUAD0_ch1_iloresetmask),
         .ch1_pcierstb             ( QUAD0_ch1_pcierstb),
         .ch1_gttxreset            ( QUAD0_TX1_gttxreset),
         .ch1_txbufstatus          ( QUAD0_TX1_bufstatus),
         .ch1_txdccdone            ( QUAD0_TX1_dccdone),
         .ch1_txdebugpcsout        ( QUAD0_TX1_debugpcsout),
         .ch1_txpmaresetdone       ( QUAD0_TX1_pmaresetdone),
         .ch1_txprogdivresetdone   ( QUAD0_TX1_progdivresetdone),
         .ch1_txresetdone          ( QUAD0_TX1_resetdone),
         .ch1_txdata               ( QUAD0_TX1_data),
         .ch1_txpd                 ( QUAD0_TX1_pd),
         .ch1_txpmaresetmask       ( QUAD0_TX1_pmaresetmask),
         .ch1_txresetmode          ( QUAD0_TX1_resetmode),
         .ch1_txprbssel            ( QUAD0_TX1_prbssel),
         .ch1_txrsv0               ( QUAD0_TX1_txrsv0),
         .ch1_txrsv1               ( QUAD0_TX1_txrsv1),
         .ch1_txpostcursor         ( QUAD0_TX1_postcursor),
         .ch1_txprecursor          ( QUAD0_TX1_precursor),
         .ch1_txprecursor2         ( QUAD0_TX1_precursor2),
         .ch1_txprecursor3         ( QUAD0_TX1_precursor3),
         .ch1_txmaincursor         ( QUAD0_TX1_maincursor),
         .ch1_txrate               ( QUAD0_TX1_rate),
         .ch1_txinhibit            ( QUAD0_TX1_inhibit),
         .ch1_txlatclk             ( QUAD0_TX1_latclk),
         .ch1_txpcsresetmask       ( QUAD0_TX1_pcsresetmask),
         .ch1_txperst              ( QUAD0_TX1_perst),
         .ch1_txpisopd             ( QUAD0_TX1_pisopd),
         .ch1_txpolarity           ( QUAD0_TX1_polarity),
         .ch1_txprbsforceerr       ( QUAD0_TX1_prbsforceerr),
         .ch1_txprogdivreset       ( QUAD0_TX1_progdivreset),
         .ch1_txqprbsen            ( QUAD0_TX1_qprbsen),
         .ch1_txuserrdy            ( QUAD0_TX1_userrdy),
         .ch1_txmstresetdone       ( QUAD0_TX1_mstresetdone),
         .ch1_txmstreset           ( QUAD0_TX1_mstreset),
         .ch1_txmstdatapathreset   ( QUAD0_TX1_mstdatapathreset),
         .ch1_txoutclk             ( QUAD0_TX1_outclk),
         .ch1_txusrclk             ( QUAD0_TX1_usrclk),

         .m1_axis_tready           ( QUAD0_m1_axis_tready),
         .s1_axis_tdata            ( QUAD0_s1_axis_tdata),
         .s1_axis_tlast            ( QUAD0_s1_axis_tlast),
         .s1_axis_tvalid           ( QUAD0_s1_axis_tvalid),
         .s1_axis_tready           ( QUAD0_s1_axis_tready),
         .m1_axis_tdata            ( QUAD0_m1_axis_tdata),
         .m1_axis_tlast            ( QUAD0_m1_axis_tlast),
         .m1_axis_tvalid           ( QUAD0_m1_axis_tvalid),
         .ch1_refdebugout          ( QUAD0_ch1_refdebugout),      
         .ch1_eyescandataerror     ( QUAD0_ch1_eyescandataerror), 
         .ch1_cdrfreqos            ( QUAD0_ch1_cdrfreqos),
         .ch1_cdrincpctrl          ( QUAD0_ch1_cdrincpctrl),
         .ch1_cdrstepdir           ( QUAD0_ch1_cdrstepdir),
         .ch1_cdrstepsq            ( QUAD0_ch1_cdrstepsq),
         .ch1_cdrstepsx            ( QUAD0_ch1_cdrstepsx),
         .ch1_eyescanreset         ( QUAD0_ch1_eyescanreset),
         .ch1_eyescantrigger       ( QUAD0_ch1_eyescantrigger),
         .ch1_gtrxreset            ( QUAD0_RX1_gtrxreset),
         .ch1_rxdata               ( QUAD0_RX1_data),
         .ch1_rxbufstatus          ( QUAD0_RX1_bufstatus),
         .ch1_rxcdrlock            ( QUAD0_RX1_cdrlock),
         .ch1_rxcdrphdone          ( QUAD0_RX1_cdrphdone),
         .ch1_rxdebugpcsout        ( QUAD0_RX1_debugpcsout),
         .ch1_rxpmaresetdone       ( QUAD0_RX1_pmaresetdone),
         .ch1_rxprbserr            ( QUAD0_RX1_prbserr),
         .ch1_rxprbslocked         ( QUAD0_RX1_prbslocked),
         .ch1_rxprogdivresetdone   ( QUAD0_RX1_progdivresetdone),
         .ch1_rxresetdone          ( QUAD0_RX1_resetdone),
         .ch1_rxpd                 ( QUAD0_RX1_pd),
         .ch1_rxresetmode          ( QUAD0_RX1_resetmode),
         .ch1_rxpcsresetmask       ( QUAD0_RX1_pcsresetmask),
         .ch1_rxprbssel            ( QUAD0_RX1_prbssel),
         .ch1_rxpmaresetmask       ( QUAD0_RX1_pmaresetmask),
         .ch1_rxrate               ( QUAD0_RX1_rate),
         .ch1_rxcdrhold            ( QUAD0_RX1_cdrhold),
         .ch1_rxcdrovrden          ( QUAD0_RX1_cdrovrden),
         .ch1_rxcdrreset           ( QUAD0_RX1_cdrreset),
         .ch1_rxlatclk             ( QUAD0_RX1_latclk),
         .ch1_rxperst              ( QUAD0_RX1_perst),
         .ch1_rxpolarity           ( QUAD0_RX1_polarity),
         .ch1_rxprbscntreset       ( QUAD0_RX1_prbscntreset),
         .ch1_rxprbscntstop        ( QUAD0_RX1_prbscntstop),
         .ch1_rxprogdivreset       ( QUAD0_RX1_progdivreset),
         .ch1_rxqprbsen            ( QUAD0_RX1_qprbsen),
         .ch1_rxrsv0               ( QUAD0_RX1_rxrsv0),
         .ch1_rxrsv1               ( QUAD0_RX1_rxrsv1),
         .ch1_rxrsv2               ( QUAD0_RX1_rxrsv2),
         .ch1_rxuserrdy            ( QUAD0_RX1_userrdy),
         .ch1_rxmstresetdone       ( QUAD0_RX1_mstresetdone),
         .ch1_rxmstreset           ( QUAD0_RX1_mstreset),
         .ch1_rxmstdatapathreset   ( QUAD0_RX1_mstdatapathreset),
         .ch1_rxoutclk             ( QUAD0_RX1_outclk),
         .ch1_rxusrclk             ( QUAD0_RX1_usrclk),
         .ch2_pcsrsvdout           ( QUAD0_ch2_pcsrsvdout),
         .ch2_pinrsvdas            ( QUAD0_ch2_pinrsvdas),
         .ch2_dmonitorout          ( QUAD0_ch2_dmonitorout),
         .ch2_dmonitoroutclk       ( QUAD0_ch2_dmonitoroutclk),
         .ch2_resetexception       ( QUAD0_ch2_resetexception),
         .ch2_gtrsvd               ( QUAD0_ch2_gtrsvd),
         .ch2_pcsrsvdin            ( QUAD0_ch2_pcsrsvdin),
         .ch2_loopback             ( QUAD0_ch2_loopback),
         .ch2_iloreset             ( QUAD0_ch2_iloreset),
         .ch2_iloresetdone         ( QUAD0_ch2_iloresetdone),
         .ch2_phystatus            ( QUAD0_ch2_phystatus),
         .ch2_clkrsvd0             ( QUAD0_ch2_clkrsvd0),
         .ch2_clkrsvd1             ( QUAD0_ch2_clkrsvd1),
         .ch2_dmonfiforeset        ( QUAD0_ch2_dmonfiforeset),
         .ch2_dmonitorclk          ( QUAD0_ch2_dmonitorclk),
         .ch2_iloresetmask         ( QUAD0_ch2_iloresetmask),
         .ch2_pcierstb             ( QUAD0_ch2_pcierstb),
         .ch2_gttxreset            ( QUAD0_TX2_gttxreset),
         .ch2_txbufstatus          ( QUAD0_TX2_bufstatus),
         .ch2_txdccdone            ( QUAD0_TX2_dccdone),
         .ch2_txdebugpcsout        ( QUAD0_TX2_debugpcsout),
         .ch2_txpmaresetdone       ( QUAD0_TX2_pmaresetdone),
         .ch2_txprogdivresetdone   ( QUAD0_TX2_progdivresetdone),
         .ch2_txresetdone          ( QUAD0_TX2_resetdone),
         .ch2_txdata               ( QUAD0_TX2_data),
         .ch2_txpd                 ( QUAD0_TX2_pd),
         .ch2_txpmaresetmask       ( QUAD0_TX2_pmaresetmask),
         .ch2_txresetmode          ( QUAD0_TX2_resetmode),
         .ch2_txprbssel            ( QUAD0_TX2_prbssel),
         .ch2_txrsv0               ( QUAD0_TX2_txrsv0),
         .ch2_txrsv1               ( QUAD0_TX2_txrsv1),
         .ch2_txpostcursor         ( QUAD0_TX2_postcursor),
         .ch2_txprecursor          ( QUAD0_TX2_precursor),
         .ch2_txprecursor2         ( QUAD0_TX2_precursor2),
         .ch2_txprecursor3         ( QUAD0_TX2_precursor3),
         .ch2_txmaincursor         ( QUAD0_TX2_maincursor),
         .ch2_txrate               ( QUAD0_TX2_rate),
         .ch2_txinhibit            ( QUAD0_TX2_inhibit),
         .ch2_txlatclk             ( QUAD0_TX2_latclk),
         .ch2_txpcsresetmask       ( QUAD0_TX2_pcsresetmask),
         .ch2_txperst              ( QUAD0_TX2_perst),
         .ch2_txpisopd             ( QUAD0_TX2_pisopd),
         .ch2_txpolarity           ( QUAD0_TX2_polarity),
         .ch2_txprbsforceerr       ( QUAD0_TX2_prbsforceerr),
         .ch2_txprogdivreset       ( QUAD0_TX2_progdivreset),
         .ch2_txqprbsen            ( QUAD0_TX2_qprbsen),
         .ch2_txuserrdy            ( QUAD0_TX2_userrdy),
         .ch2_txmstresetdone       ( QUAD0_TX2_mstresetdone),
         .ch2_txmstreset           ( QUAD0_TX2_mstreset),
         .ch2_txmstdatapathreset   ( QUAD0_TX2_mstdatapathreset),
         .ch2_txoutclk             ( QUAD0_TX2_outclk),
         .ch2_txusrclk             ( QUAD0_TX2_usrclk),

         .s2_axis_tdata            ( QUAD0_s2_axis_tdata),
         .s2_axis_tlast            ( QUAD0_s2_axis_tlast),
         .s2_axis_tvalid           ( QUAD0_s2_axis_tvalid),
         .s2_axis_tready           ( QUAD0_s2_axis_tready),
         .m2_axis_tdata            ( QUAD0_m2_axis_tdata),
         .m2_axis_tlast            ( QUAD0_m2_axis_tlast),
         .m2_axis_tvalid           ( QUAD0_m2_axis_tvalid),
         .m2_axis_tready           ( QUAD0_m2_axis_tready),
         .ch2_refdebugout          ( QUAD0_ch2_refdebugout),      
         .ch2_eyescandataerror     ( QUAD0_ch2_eyescandataerror), 
         .ch2_cdrfreqos            ( QUAD0_ch2_cdrfreqos),
         .ch2_cdrincpctrl          ( QUAD0_ch2_cdrincpctrl),
         .ch2_cdrstepdir           ( QUAD0_ch2_cdrstepdir),
         .ch2_cdrstepsq            ( QUAD0_ch2_cdrstepsq),
         .ch2_cdrstepsx            ( QUAD0_ch2_cdrstepsx),
         .ch2_eyescanreset         ( QUAD0_ch2_eyescanreset),
         .ch2_eyescantrigger       ( QUAD0_ch2_eyescantrigger),
         .ch2_gtrxreset            ( QUAD0_RX2_gtrxreset),
         .ch2_rxdata               ( QUAD0_RX2_data),
         .ch2_rxbufstatus          ( QUAD0_RX2_bufstatus),
         .ch2_rxcdrlock            ( QUAD0_RX2_cdrlock),
         .ch2_rxcdrphdone          ( QUAD0_RX2_cdrphdone),
         .ch2_rxdebugpcsout        ( QUAD0_RX2_debugpcsout),
         .ch2_rxpmaresetdone       ( QUAD0_RX2_pmaresetdone),
         .ch2_rxprbserr            ( QUAD0_RX2_prbserr),
         .ch2_rxprbslocked         ( QUAD0_RX2_prbslocked),
         .ch2_rxprogdivresetdone   ( QUAD0_RX2_progdivresetdone),
         .ch2_rxresetdone          ( QUAD0_RX2_resetdone),
         .ch2_rxpd                 ( QUAD0_RX2_pd),
         .ch2_rxresetmode          ( QUAD0_RX2_resetmode),
         .ch2_rxpcsresetmask       ( QUAD0_RX2_pcsresetmask),
         .ch2_rxprbssel            ( QUAD0_RX2_prbssel),
         .ch2_rxpmaresetmask       ( QUAD0_RX2_pmaresetmask),
         .ch2_rxrate               ( QUAD0_RX2_rate),
         .ch2_rxcdrhold            ( QUAD0_RX2_cdrhold),
         .ch2_rxcdrovrden          ( QUAD0_RX2_cdrovrden),
         .ch2_rxcdrreset           ( QUAD0_RX2_cdrreset),
         .ch2_rxlatclk             ( QUAD0_RX2_latclk),
         .ch2_rxperst              ( QUAD0_RX2_perst),
         .ch2_rxpolarity           ( QUAD0_RX2_polarity),
         .ch2_rxprbscntreset       ( QUAD0_RX2_prbscntreset),
         .ch2_rxprbscntstop        ( QUAD0_RX2_prbscntstop),
         .ch2_rxprogdivreset       ( QUAD0_RX2_progdivreset),
         .ch2_rxqprbsen            ( QUAD0_RX2_qprbsen),
         .ch2_rxrsv0               ( QUAD0_RX2_rxrsv0),
         .ch2_rxrsv1               ( QUAD0_RX2_rxrsv1),
         .ch2_rxrsv2               ( QUAD0_RX2_rxrsv2),
         .ch2_rxuserrdy            ( QUAD0_RX2_userrdy),
         .ch2_rxmstresetdone       ( QUAD0_RX2_mstresetdone),
         .ch2_rxmstreset           ( QUAD0_RX2_mstreset),
         .ch2_rxmstdatapathreset   ( QUAD0_RX2_mstdatapathreset),
         .ch2_rxoutclk             ( QUAD0_RX2_outclk),
         .ch2_rxusrclk             ( QUAD0_RX2_usrclk),
         .ch3_pcsrsvdout           ( QUAD0_ch3_pcsrsvdout),
         .ch3_pinrsvdas            ( QUAD0_ch3_pinrsvdas),
         .ch3_dmonitorout          ( QUAD0_ch3_dmonitorout),
         .ch3_dmonitoroutclk       ( QUAD0_ch3_dmonitoroutclk),
         .ch3_resetexception       ( QUAD0_ch3_resetexception),
         .ch3_gtrsvd               ( QUAD0_ch3_gtrsvd),
         .ch3_pcsrsvdin            ( QUAD0_ch3_pcsrsvdin),
         .ch3_loopback             ( QUAD0_ch3_loopback),
         .ch3_iloreset             ( QUAD0_ch3_iloreset),
         .ch3_iloresetdone         ( QUAD0_ch3_iloresetdone),
         .ch3_phystatus            ( QUAD0_ch3_phystatus),
         .ch3_clkrsvd0             ( QUAD0_ch3_clkrsvd0),
         .ch3_clkrsvd1             ( QUAD0_ch3_clkrsvd1),
         .ch3_dmonfiforeset        ( QUAD0_ch3_dmonfiforeset),
         .ch3_dmonitorclk          ( QUAD0_ch3_dmonitorclk),
         .ch3_iloresetmask         ( QUAD0_ch3_iloresetmask),
         .ch3_pcierstb             ( QUAD0_ch3_pcierstb),
         .ch3_gttxreset            ( QUAD0_TX3_gttxreset),
         .ch3_txbufstatus          ( QUAD0_TX3_bufstatus),
         .ch3_txdccdone            ( QUAD0_TX3_dccdone),
         .ch3_txdebugpcsout        ( QUAD0_TX3_debugpcsout),
         .ch3_txpmaresetdone       ( QUAD0_TX3_pmaresetdone),
         .ch3_txprogdivresetdone   ( QUAD0_TX3_progdivresetdone),
         .ch3_txresetdone          ( QUAD0_TX3_resetdone),
         .ch3_txdata               ( QUAD0_TX3_data),
         .ch3_txpd                 ( QUAD0_TX3_pd),
         .ch3_txpmaresetmask       ( QUAD0_TX3_pmaresetmask),
         .ch3_txresetmode          ( QUAD0_TX3_resetmode),
         .ch3_txprbssel            ( QUAD0_TX3_prbssel),
         .ch3_txrsv0               ( QUAD0_TX3_txrsv0),
         .ch3_txrsv1               ( QUAD0_TX3_txrsv1),
         .ch3_txpostcursor         ( QUAD0_TX3_postcursor),
         .ch3_txprecursor          ( QUAD0_TX3_precursor),
         .ch3_txprecursor2         ( QUAD0_TX3_precursor2),
         .ch3_txprecursor3         ( QUAD0_TX3_precursor3),
         .ch3_txmaincursor         ( QUAD0_TX3_maincursor),
         .ch3_txrate               ( QUAD0_TX3_rate),
         .ch3_txinhibit            ( QUAD0_TX3_inhibit),
         .ch3_txlatclk             ( QUAD0_TX3_latclk),
         .ch3_txpcsresetmask       ( QUAD0_TX3_pcsresetmask),
         .ch3_txperst              ( QUAD0_TX3_perst),
         .ch3_txpisopd             ( QUAD0_TX3_pisopd),
         .ch3_txpolarity           ( QUAD0_TX3_polarity),
         .ch3_txprbsforceerr       ( QUAD0_TX3_prbsforceerr),
         .ch3_txprogdivreset       ( QUAD0_TX3_progdivreset),
         .ch3_txqprbsen            ( QUAD0_TX3_qprbsen),
         .ch3_txuserrdy            ( QUAD0_TX3_userrdy),
         .ch3_txmstresetdone       ( QUAD0_TX3_mstresetdone),
         .ch3_txmstreset           ( QUAD0_TX3_mstreset),
         .ch3_txmstdatapathreset   ( QUAD0_TX3_mstdatapathreset),
         .ch3_txoutclk             ( QUAD0_TX3_outclk),
         .ch3_txusrclk             ( QUAD0_TX3_usrclk),

         .s3_axis_tdata            ( QUAD0_s3_axis_tdata),
         .s3_axis_tlast            ( QUAD0_s3_axis_tlast),
         .s3_axis_tvalid           ( QUAD0_s3_axis_tvalid),
         .s3_axis_tready           ( QUAD0_s3_axis_tready),
         .m3_axis_tdata            ( QUAD0_m3_axis_tdata),
         .m3_axis_tlast            ( QUAD0_m3_axis_tlast),
         .m3_axis_tvalid           ( QUAD0_m3_axis_tvalid),
         .m3_axis_tready           ( QUAD0_m3_axis_tready),
         .ch3_refdebugout          ( QUAD0_ch3_refdebugout),      
         .ch3_eyescandataerror     ( QUAD0_ch3_eyescandataerror), 
         .ch3_cdrfreqos            ( QUAD0_ch3_cdrfreqos),
         .ch3_cdrincpctrl          ( QUAD0_ch3_cdrincpctrl),
         .ch3_cdrstepdir           ( QUAD0_ch3_cdrstepdir),
         .ch3_cdrstepsq            ( QUAD0_ch3_cdrstepsq),
         .ch3_cdrstepsx            ( QUAD0_ch3_cdrstepsx),
         .ch3_eyescanreset         ( QUAD0_ch3_eyescanreset),
         .ch3_eyescantrigger       ( QUAD0_ch3_eyescantrigger),
         .ch3_gtrxreset            ( QUAD0_RX3_gtrxreset),
         .ch3_rxdata               ( QUAD0_RX3_data),
         .ch3_rxbufstatus          ( QUAD0_RX3_bufstatus),
         .ch3_rxcdrlock            ( QUAD0_RX3_cdrlock),
         .ch3_rxcdrphdone          ( QUAD0_RX3_cdrphdone),
         .ch3_rxdebugpcsout        ( QUAD0_RX3_debugpcsout),
         .ch3_rxpmaresetdone       ( QUAD0_RX3_pmaresetdone),
         .ch3_rxprbserr            ( QUAD0_RX3_prbserr),
         .ch3_rxprbslocked         ( QUAD0_RX3_prbslocked),
         .ch3_rxprogdivresetdone   ( QUAD0_RX3_progdivresetdone),
         .ch3_rxresetdone          ( QUAD0_RX3_resetdone),
         .ch3_rxpd                 ( QUAD0_RX3_pd),
         .ch3_rxresetmode          ( QUAD0_RX3_resetmode),
         .ch3_rxpcsresetmask       ( QUAD0_RX3_pcsresetmask),
         .ch3_rxprbssel            ( QUAD0_RX3_prbssel),
         .ch3_rxpmaresetmask       ( QUAD0_RX3_pmaresetmask),
         .ch3_rxrate               ( QUAD0_RX3_rate),
         .ch3_rxcdrhold            ( QUAD0_RX3_cdrhold),
         .ch3_rxcdrovrden          ( QUAD0_RX3_cdrovrden),
         .ch3_rxcdrreset           ( QUAD0_RX3_cdrreset),
         .ch3_rxlatclk             ( QUAD0_RX3_latclk),
         .ch3_rxperst              ( QUAD0_RX3_perst),
         .ch3_rxpolarity           ( QUAD0_RX3_polarity),
         .ch3_rxprbscntreset       ( QUAD0_RX3_prbscntreset),
         .ch3_rxprbscntstop        ( QUAD0_RX3_prbscntstop),
         .ch3_rxprogdivreset       ( QUAD0_RX3_progdivreset),
         .ch3_rxqprbsen            ( QUAD0_RX3_qprbsen),
         .ch3_rxrsv0               ( QUAD0_RX3_rxrsv0),
         .ch3_rxrsv1               ( QUAD0_RX3_rxrsv1),
         .ch3_rxrsv2               ( QUAD0_RX3_rxrsv2),
         .ch3_rxuserrdy            ( QUAD0_RX3_userrdy),
         .ch3_rxmstresetdone       ( QUAD0_RX3_mstresetdone),
         .ch3_rxmstreset           ( QUAD0_RX3_mstreset),
         .ch3_rxmstdatapathreset   ( QUAD0_RX3_mstdatapathreset),
         .ch3_rxoutclk             ( QUAD0_RX3_outclk),
         .ch3_rxusrclk             ( QUAD0_RX3_usrclk),
         .gtpowergood                      ( QUAD0_gtpowergood)
);

endmodule
