

//------------------------------------------------------------------------------
//  (c) Copyright 2017-2018 Advanced Micro Devices, Inc. All rights reserved.
//
//  This file contains confidential and proprietary information
//  of Advanced Micro Devices, Inc. and is protected under U.S. and
//  international copyright and other intellectual property
//  laws.
//
//  DISCLAIMER
//  This disclaimer is not a license and does not grant any
//  rights to the materials distributed herewith. Except as
//  otherwise provided in a valid license issued to you by
//  AMD, and to the maximum extent permitted by applicable
//  law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
//  WITH ALL FAULTS, AND AMD HEREBY DISCLAIMS ALL WARRANTIES
//  AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
//  BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
//  INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
//  (2) AMD shall not be liable (whether in contract or tort,
//  including negligence, or under any other theory of
//  liability) for any loss or damage of any kind or nature
//  related to, arising under or in connection with these
//  materials, including for any direct, or any indirect,
//  special, incidental, or consequential loss or damage
//  (including loss of data, profits, goodwill, or any type of
//  loss or damage suffered as a result of any action brought
//  by a third party) even if such damage or loss was
//  reasonably foreseeable or AMD had been advised of the
//  possibility of the same.
//
//  CRITICAL APPLICATIONS
//  AMD products are not designed or intended to be fail-
//  safe, or for use in any application requiring fail-safe
//  performance, such as life-support or safety devices or
//  systems, Class III medical devices, nuclear facilities,
//  applications related to the deployment of airbags, or any
//  other applications that could lead to death, personal
//  injury, or severe property or environmental damage
//  (individually and collectively, "Critical
//  Applications"). Customer assumes the sole risk and
//  liability of any use of AMD products in Critical
//  Applications, subject only to applicable laws and
//  regulations governing limitations on product liability.
//
//  THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
//  PART OF THIS FILE AT ALL TIMES.
//------------------------------------------------------------------------------

`timescale 1ns / 1ps
module prbs_generator_checker_intf0_inst # (
    parameter EGW_IS_PARENT_IP   = 1,
    parameter IP_LR0_ENABLE      = 1,
    parameter IP_LR0_SETTINGS    = " ",
    parameter IP_LR10_ENABLE     = 0,
    parameter IP_LR10_SETTINGS   = " ",
    parameter IP_LR11_ENABLE     = 0,
    parameter IP_LR11_SETTINGS   = " ",
    parameter IP_LR12_ENABLE     = 0,
    parameter IP_LR12_SETTINGS   = " ",
    parameter IP_LR13_ENABLE     = 0,
    parameter IP_LR13_SETTINGS   = " ",
    parameter IP_LR14_ENABLE     = 0,
    parameter IP_LR14_SETTINGS   = " ",
    parameter IP_LR15_ENABLE     = 0,
    parameter IP_LR15_SETTINGS   = " ",
    parameter IP_LR1_ENABLE      = 0,
    parameter IP_LR1_SETTINGS    = " ",
    parameter IP_LR2_ENABLE      = 0,
    parameter IP_LR2_SETTINGS    = " ",
    parameter IP_LR3_ENABLE      = 0,
    parameter IP_LR3_SETTINGS    = " ",
    parameter IP_LR4_ENABLE      = 0,
    parameter IP_LR4_SETTINGS    = " ",
    parameter IP_LR5_ENABLE      = 0,
    parameter IP_LR5_SETTINGS    = " ",
    parameter IP_LR6_ENABLE      = 0,
    parameter IP_LR6_SETTINGS    = " ",
    parameter IP_LR7_ENABLE      = 0,
    parameter IP_LR7_SETTINGS    = " ",
    parameter IP_LR8_ENABLE      = 0,
    parameter IP_LR8_SETTINGS    = " ",
    parameter IP_LR9_ENABLE      = 0,
    parameter IP_LR9_SETTINGS    = " ",
    parameter IP_MLR_ENABLE      = 1,
    parameter IP_MULTI_LR        = 0,
    parameter IP_NO_OF_LANES     = 4,
    parameter IP_NO_OF_RX_LANES  = 0,
    parameter IP_NO_OF_TX_LANES  = 0,
    parameter IP_PRESET          = "start_from_scratch",                                
    parameter IP_SETTINGS          = " ",
    parameter IP_NO_OF_LR          = 0,
    parameter IP_GT_DIRECTION  = "DUPLEX"                                        
  )

(

       output [255:0] ch0_txdata,
       output [5:0]   ch0_txheader,
       output [6:0]   ch0_txsequence,
       output  [15:0] ch0_txctrl0,
       output  [15:0] ch0_txctrl1,
       output  [7:0]  ch0_txctrl2,
       input          ch0_txresetdone, 
       input          ch0_txuserrdy_in, 
       output  [7:0]   ch0_txrate,
       input [255:0]   ch0_rxdata,
       input [1:0]     ch0_rxdatavalid,
       input [5:0]     ch0_rxheader,
       output          ch0_rxgearboxslip,
       input [15:0]    ch0_rxctrl0,
       input [15:0]    ch0_rxctrl1,
       input [1:0]     ch0_rxheadervalid,
       input [1:0]     ch0_rxstartofseq,
       input [7:0]     ch0_rxctrl2,
       input [7:0]     ch0_rxctrl3,
       input           ch0_rxresetdone, 
			 input           ch0_rxuserrdy_in, 
       output  [7:0]   ch0_rxrate,
 

   input   [3:0]  rate_sel,

   input          apb3clk,
   (* dont_touch = "true" *) input          gt_rxusrclk,
   (* dont_touch = "true" *) input          gt_txusrclk,
   input          tx_reset_in,
   input          rx_reset_in,
   input          gt_reset_in,
  
   output         gpi_out,
   input          gpo_in,
   input          gpio_enable,
   output         link_status_out

   );

   wire [7:0] rate_sel_delayed; 
   wire [7:0] rate_sel_apb_sync = rate_sel; 
   wire [7:0] rate_sel_post_gpio; 
   wire gpi_out_int;
   wire gpi_out_d;
   reg gpi_out_q;

   wire gpo_in_sync;
   xpm_cdc_sync_rst # (
    .DEST_SYNC_FF (3),
    .INIT         (0)
  ) gpo_in_sync_inst (
    .src_rst  (gpo_in),
    .dest_rst (gpo_in_sync),
    .dest_clk (apb3clk)
  );

   assign gpi_out =  1'b0; 
   assign rate_sel_delayed =  rate_sel_apb_sync;
   assign gpi_out_int = 1'b0;
   assign rate_sel_post_gpio = rate_sel_apb_sync;





wire [7:0] rate_sel_delayed_tx;
wire [7:0] rate_sel_delayed_rx;
wire [7:0] rate_sel_tx = rate_sel_delayed_tx;
wire [7:0] rate_sel_rx = rate_sel_delayed_rx;

xpm_cdc_array_single # (
  .DEST_SYNC_FF (3),
  .INIT_SYNC_FF (1),
  .WIDTH(8)
) tx_rate_port_sync (
  .src_in   (rate_sel_delayed),
  .src_clk  (apb3clk),
  .dest_clk (gt_txusrclk),
  .dest_out (rate_sel_delayed_tx)
);

xpm_cdc_array_single # (
  .DEST_SYNC_FF (3),
  .INIT_SYNC_FF (1),
  .WIDTH(8)
) rx_rate_port_sync (
  .src_in   (rate_sel_delayed),
  .src_clk  (apb3clk),
  .dest_clk (gt_rxusrclk),
  .dest_out (rate_sel_delayed_rx)
);

wire gtwiz_reset_clk_freerun_in = apb3clk;

wire [255:0]                      txdata_int         [IP_NO_OF_TX_LANES-1:0]; 
wire [5:0]                        txheader_int       [IP_NO_OF_TX_LANES-1:0]; 
wire [6:0]                        txsequence_int     [IP_NO_OF_TX_LANES-1:0]; 
wire [15:0]                       txctrl0_int        [IP_NO_OF_TX_LANES-1:0]; 
wire [15:0]                       txctrl1_int        [IP_NO_OF_TX_LANES-1:0]; 
wire [7:0]                        txctrl2_int        [IP_NO_OF_TX_LANES-1:0]; 
(* keep = "TRUE" *)reg [255:0]                      txdata_int_reg         [IP_NO_OF_TX_LANES-1:0]; 
(* keep = "TRUE" *)reg [5:0]                        txheader_int_reg       [IP_NO_OF_TX_LANES-1:0]; 
(* keep = "TRUE" *)reg [6:0]                        txsequence_int_reg     [IP_NO_OF_TX_LANES-1:0]; 
(* keep = "TRUE" *)reg [15:0]                       txctrl0_int_reg        [IP_NO_OF_TX_LANES-1:0]; 
(* keep = "TRUE" *)reg [15:0]                       txctrl1_int_reg        [IP_NO_OF_TX_LANES-1:0]; 
(* keep = "TRUE" *)reg [7:0]                        txctrl2_int_reg        [IP_NO_OF_TX_LANES-1:0]; 

wire [255:0]                      rxdata_int         [IP_NO_OF_RX_LANES-1:0]; 
wire [1:0]                        rxdatavalid_int    [IP_NO_OF_RX_LANES-1:0]; 
wire [15:0]                       rxctrl0_int        [IP_NO_OF_RX_LANES-1:0]; 
wire [15:0]                       rxctrl1_int        [IP_NO_OF_RX_LANES-1:0]; 
wire [7:0]                        rxctrl2_int        [IP_NO_OF_RX_LANES-1:0]; 
wire [7:0]                        rxctrl3_int        [IP_NO_OF_RX_LANES-1:0]; 
wire [IP_NO_OF_RX_LANES-1:0]      rxgearboxslip_int;
wire [IP_NO_OF_RX_LANES-1:0]      prbs_match_out;

  // PRBS match and related link management
  // -------------------------------------------------------------------------------------------------------------------

  // Perform a bitwise NAND of all PRBS match indicators, creating a combinatorial indication of any PRBS mismatch
  // across all transceiver channels
  wire prbs_error_any_async = ~(&prbs_match_out);

  wire prbs_error_any_sync;
  wire prbs_error_any_sync_a;
  reg prbs_error_any_async_q;
  
  always @(posedge gt_rxusrclk) begin
    prbs_error_any_async_q <= prbs_error_any_async;
  end
  // Synchronize the PRBS mismatch indicator the free-running clock domain, using a reset synchronizer with asynchronous
  // reset and synchronous removal
  (* DONT_TOUCH = "TRUE" *)
  prbs_generator_checker_intf0_example_reset_synchronizer prbs_generator_checker_intf0_reset_synchronizer_prbs_match_all_inst (
    .clk_in (apb3clk),
    .rst_in (prbs_error_any_async_q),
    .rst_out(prbs_error_any_sync_a)
  );

   assign prbs_error_any_sync = prbs_error_any_sync_a || tx_reset_in || rx_reset_in || gt_reset_in;

  // Implement an example link status state machine using a simple leaky bucket mechanism. The link status indicates
  // the continual PRBS match status to both the top-level observer and the initialization state machine, while being
  // tolerant of occasional bit errors. This is an example and can be modified as necessary.
  localparam ST_LINK_DOWN = 1'b0;
  localparam ST_LINK_UP   = 1'b1;
  reg        sm_link      = ST_LINK_DOWN;
  reg [6:0]  link_ctr;

  always @(posedge apb3clk) begin
    case (sm_link)
      // The link is considered to be down when the link counter initially has a value less than 67. When the link is
      // down, the counter is incremented on each cycle where all PRBS bits match, but reset whenever any PRBS mismatch
      // occurs. When the link counter reaches 67, transition to the link up state.
      ST_LINK_DOWN: begin
        if (prbs_error_any_sync !== 1'b0) begin
          link_ctr <= 7'd0;
        end
        else begin
          if (link_ctr < 7'd67)
            link_ctr <= link_ctr + 7'd1;
          else
            sm_link <= ST_LINK_UP;
        end
      end

      // When the link is up, the link counter is decreased by 34 whenever any PRBS mismatch occurs, but is increased by
      // only 1 on each cycle where all PRBS bits match, up to its saturation point of 67. If the link counter reaches
      // 0 (including rollover protection), transition to the link down state.
      ST_LINK_UP: begin
        if (prbs_error_any_sync !== 1'b0) begin
          if (link_ctr > 7'd33) begin
            link_ctr <= link_ctr - 7'd34;
            if (link_ctr == 7'd34)
              sm_link  <= ST_LINK_DOWN;
          end
          else begin
            link_ctr <= 7'd0;
            sm_link  <= ST_LINK_DOWN;
          end
        end
        else begin
          if (link_ctr < 7'd67)
            link_ctr <= link_ctr + 7'd1;
        end
      end
    endcase
  end


// Assign the link status indicator to the top-level two-state output for user reference
assign link_status_out = sm_link;

assign txusrclk_out = gt_txusrclk;
assign rxusrclk_out = gt_rxusrclk;
assign ch0_txrate = rate_sel_tx;
assign ch0_rxrate = rate_sel_rx;

assign ch0_txdata               =       txdata_int_reg[0];
assign ch0_txheader             =       txheader_int_reg[0];
assign ch0_txsequence           =       txsequence_int_reg[0];
assign ch0_txctrl0              =       txctrl0_int_reg[0];
assign ch0_txctrl1              =       txctrl1_int_reg[0];
assign ch0_txctrl2              =       txctrl2_int_reg[0];


//Registering
always @(posedge gt_txusrclk) begin
   txdata_int_reg[0] <=    txdata_int[0];
   txheader_int_reg[0] <=    txheader_int[0];
   txsequence_int_reg[0] <=    txsequence_int[0];
   txctrl0_int_reg[0] <=    txctrl0_int[0];
   txctrl1_int_reg[0] <=    txctrl1_int[0];
   txctrl2_int_reg[0] <=    txctrl2_int[0];
end
assign rxdata_int[0]               =         ch0_rxdata;
assign rxdatavalid_int[0]          =         ch0_rxdatavalid;
assign rxctrl0_int[0]              =         ch0_rxctrl0;
assign rxctrl1_int[0]              =         ch0_rxctrl1;
assign rxctrl2_int[0]              =         ch0_rxctrl2;
assign rxctrl3_int[0]              =         ch0_rxctrl3;
assign ch0_rxgearboxslip           =         rxgearboxslip_int[0];


  genvar lw_tx;

  generate
  for (lw_tx = 0; lw_tx < IP_NO_OF_TX_LANES ; lw_tx = lw_tx + 1)
  begin : multilinks_tx

prbs_generator_checker_intf0_stimulus_gen_lane_top #(
      .C_TX_GEN_WIDTH (256)
   ) prbs_generator_checker_intf0_stimulus_inst (
       .gt_reset_all_in             (tx_reset_in),
       .gt_reset_clk_freerun_in     (gtwiz_reset_clk_freerun_in),
       .gt_userclk_tx_usrclk2_in    (gt_txusrclk),
       .gt_userclk_tx_active_in     (!tx_reset_in ),
       .rate_sel                    (rate_sel),
       .txheader_out                (txheader_int[lw_tx]),
       .txsequence_out              (txsequence_int[lw_tx]),
       .txctrl0_out                 (txctrl0_int[lw_tx]),
       .txctrl1_out                 (txctrl1_int[lw_tx]),
       .txctrl2_out                 (txctrl2_int[lw_tx]),
       .txdata_out                  (txdata_int[lw_tx])
     );
   
  end
  endgenerate


  genvar lw_rx;

  generate
  for (lw_rx = 0; lw_rx < IP_NO_OF_RX_LANES ; lw_rx = lw_rx + 1)
  begin : multilinks_rx
   
    prbs_generator_checker_intf0_stimulus_check_lane_top #(
      .C_RX_CHECK_WIDTH (256)
   ) prbs_generator_checker_intf0_checking_inst (
       .gt_reset_all_in             (rx_reset_in),
       .gt_reset_clk_freerun_in     (gtwiz_reset_clk_freerun_in),
       .gt_userclk_rx_usrclk2_in    (gt_rxusrclk),
       .gt_userclk_rx_active_in     (!rx_reset_in),
       .rate_sel                    (rate_sel),
       .rxdatavalid_in              (rxdatavalid_int[lw_rx]),
       .rxctrl0_in                  (rxctrl0_int[lw_rx]),
       .rxctrl1_in                  (rxctrl1_int[lw_rx]),
       .rxctrl2_in                  (rxctrl2_int[lw_rx]),
       .rxctrl3_in                  (rxctrl3_int[lw_rx]),
       .rxgearboxslip_out           (rxgearboxslip_int[lw_rx]),
       .rxdata_in                   (rxdata_int[lw_rx]),
       .prbs_match_out              (prbs_match_out[lw_rx])
     );
   
  end
  endgenerate

//GT Wizard rest of mandatory ports assignments
 

endmodule

//------}
