// (c) Copyright 2023 Advanced Micro Devices, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of AMD and is protected under U.S. and international copyright
// and other intellectual property laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// AMD, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND AMD HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) AMD shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or AMD had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// AMD products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of AMD products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
//-----------------------------------------------------------------

`timescale 1ns / 1fs
module my_gtm_example_tb ();
logic        gtwiz_freerun_clock = 1'b0;
logic [3:0]  QUAD0_rxp; 
logic [3:0]  QUAD0_rxn; 
logic [3:0]  QUAD0_txp; 
logic [3:0]  QUAD0_txn; 
logic        gt_powergood;
logic INTF0_link_status_out;
logic [3:0]  intf0_rate_sel         = 4'b0;
logic        intf0_gpio_enable      = 1'b0;
logic        gtwiz_INTF0_rst_all_in = 1'b0;
reg          intf0_clk_lr0_tx       = 1'b0;
reg          intf0_clk_lr0_rx       = 1'b0;
reg          gt_resetdone_INTF0     = 1'b0;
reg          dp_resetdone_INTF0     = 1'b0;
real         intf0_t0;
real         intf0_t1;
real         intf0_t2;
real         intf0_t3;
real         intf0_frequency;
real         intf0_frequency_int;
logic        gtwiz_INTF0_rst_tx_datapath_in =1'b0;
logic        INTF0_rst_tx_done_out;
logic        INTF0_TX_usrclk;
logic        gtwiz_INTF0_rst_rx_datapath_in = 1'b0;
logic        INTF0_rst_rx_done_out;
logic        INTF0_RX_usrclk;
logic INTF1_link_status_out;
logic [3:0]  intf1_rate_sel         = 4'b0;
logic        intf1_gpio_enable      = 1'b0;
logic        gtwiz_INTF1_rst_all_in = 1'b0;
reg          intf1_clk_lr0_tx       = 1'b0;
reg          intf1_clk_lr0_rx       = 1'b0;
reg          gt_resetdone_INTF1     = 1'b0;
reg          dp_resetdone_INTF1     = 1'b0;
real         intf1_t0;
real         intf1_t1;
real         intf1_t2;
real         intf1_t3;
real         intf1_frequency;
real         intf1_frequency_int;
reg          intf1_clk_lr1_tx = 1'b0;
reg          intf1_clk_lr1_rx = 1'b0;
logic        gtwiz_INTF1_rst_tx_datapath_in =1'b0;
logic        INTF1_rst_tx_done_out;
logic        INTF1_TX_usrclk;
logic        gtwiz_INTF1_rst_rx_datapath_in = 1'b0;
logic        INTF1_rst_rx_done_out;
logic        INTF1_RX_usrclk;

reg [2-1 : 0]INTF_PASS  = 2'h0;
reg [2-1 : 0]INTF_FAIL  = 2'h0;
reg [2-1 : 0]TB_TIMEOUT = 2'h0;
reg [2-1 : 0]INTF_TIMEOUT = 2'h0;

reg [10:0] link_up_ctr_ip0 = 11'd0;
reg        link_stable_ip0 = 1'b0;
always @(posedge gtwiz_freerun_clock) begin
  if (INTF0_link_status_out !== 1'b1) begin
    link_up_ctr_ip0 <= 11'd0;
    link_stable_ip0 <= 1'b0;
  end
  else begin
    if (&link_up_ctr_ip0)
      link_stable_ip0 <= 1'b1;
    else
      link_up_ctr_ip0 <= link_up_ctr_ip0 + 11'd1;
  end
end

initial begin
#1;
  $display("my_gtm example design simulation started");
  gtwiz_INTF0_rst_all_in = 1'b0;
  wait( gt_powergood || &(TB_TIMEOUT) );
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM gt_powergood assertion failed \n for INTERFACE_0");
    end
  end
  else begin
    $display("gt_powergood asserted for INTERFACE_0");
  end
  wait( (!INTF0_rst_tx_done_out) || &(TB_TIMEOUT) );
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX Resetdone deassertion failed \n for INTERFACE_0");
    end
  end
  wait( (!INTF0_rst_rx_done_out) || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM RX Resetdone deassertion failed \n for INTERFACE_0");
    end
  end
  wait( (INTF0_rst_tx_done_out && INTF0_rst_rx_done_out) || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX/RX Resetdone assertion failed \n for INTERFACE_0");
    end
  end
  //wait for 30 free running clock edges to reset GT
  //Note:: 30 is arbitrary no.
  repeat (30) @ (posedge gtwiz_freerun_clock);
  gtwiz_INTF0_rst_all_in = 1'b1;
  //Assert GT reset for 5 clock cycles.
  //Note:: 5 is arbitrary no.
  repeat (5) @ (posedge gtwiz_freerun_clock);
  gtwiz_INTF0_rst_all_in = 1'b0;
  wait ((!INTF0_rst_tx_done_out) || &(TB_TIMEOUT)); 
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX Resetdone deassertion failed \n for INTERFACE_0 after tb primary reset");
    end
  end
  wait ((!INTF0_rst_rx_done_out) || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM RX Resetdone deassertion failed \n for INTERFACE_0 after tb primary reset");
    end
  end
  assign intf0_t2 = 415.0390625 - (415.0390625)*0.01;
  assign intf0_t3 = 415.0390625 + (415.0390625)*0.01;
  wait( INTF0_rst_tx_done_out || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX Resetdone assertion failed \n for INTERFACE_0 after tb primary reset");
    end
  end
  $display("INTF0 CONFIG0 TX RESET DONE asserted");
  @ (posedge INTF0_TX_usrclk) intf0_t0 = $realtime;
  @ (posedge INTF0_TX_usrclk) intf0_t1 = $realtime;
  intf0_frequency = 1000 / (intf0_t1 - intf0_t0);
  $display("INTF0 CONFIG0 TXUSRCLK Frequency = %g", intf0_frequency);
  if ((intf0_frequency <= intf0_t3 ) &&  (intf0_frequency >=  intf0_t2))
    $display(" TX_USER CLOCK IS PROPER. TX_USER_CLOCK is %g and EXPECTED is [%g:%g]",intf0_frequency, intf0_t2 , intf0_t3);
  else begin
    $display("Error: GTM TX_USER CLOCK IS NOT PROPER. \n TX_USER_CLOCK is %g and EXPECTED is [%g:%g]",intf0_frequency, intf0_t2 , intf0_t3);
    INTF_FAIL[0] = 1'b1;
  end
  assign intf0_t2 =415.0390625 - (415.0390625)*0.01;
  assign intf0_t3 =415.0390625 + (415.0390625)*0.01;
  wait( INTF0_rst_rx_done_out || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM RX Resetdone assertion failed \n for INTERFACE_0 after tb primary reset");
    end
  end
  $display("INTF0 CONFIG0 RX RESET DONE asserted");
  @ (posedge INTF0_RX_usrclk) intf0_t0 = $realtime;
  @ (posedge INTF0_RX_usrclk) intf0_t1 = $realtime;
  intf0_frequency = 1000 / (intf0_t1 - intf0_t0);
  $display("INTF0 CONFIG0 RXUSRCLK Frequency = %g", intf0_frequency);
  if ((intf0_frequency <= intf0_t3 ) &&  (intf0_frequency >=  intf0_t2 ))
    $display("RX_USER CLOCK IS PROPER. RX_USER_CLOCK is %g and EXPECTED is [%g:%g]",intf0_frequency, intf0_t2 , intf0_t3);
  else begin
    $display("Error: GTM RX_USER CLOCK IS NOT PROPER. \n RX_USER_CLOCK POST PLL LOCK is %g and EXPECTED is [%g:%g]",intf0_frequency, intf0_t2 , intf0_t3);
    INTF_FAIL[0] = 1'b1;
  end
  #100;
  wait(link_stable_ip0 || &(TB_TIMEOUT)); 
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM link stable assertion failed \n for INTERFACE_0 CONFIG0");
    end
  end
  @ (posedge INTF0_TX_usrclk) intf0_t0 = $realtime;
  @ (posedge INTF0_TX_usrclk) intf0_t1 = $realtime;
  intf0_frequency = 1000 / (intf0_t1 - intf0_t0);
  $display("INTF0 CONFIG0 :::::: POST LINK STABLE TXUSRCLK Frequency = %g", intf0_frequency);
  #100ns;
  @ (posedge INTF0_RX_usrclk) intf0_t0 = $realtime;
  @ (posedge INTF0_RX_usrclk) intf0_t1 = $realtime;
  intf0_frequency = 1000 / (intf0_t1 - intf0_t0);
  $display("INTF0 CONFIG0 :::::: POST LINK STABLE RXUSRCLK Frequency = %g", intf0_frequency);
  $display("Time : %12d ns   INTF0 CONFIG0: simulation completed", $time);
  INTF_PASS[0] = 1'b1;
end

initial begin
  intf0_frequency_int = 415.0390625; 
  repeat (300000) @ (posedge gtwiz_freerun_clock);
  if (intf0_frequency_int <= 150 )
    repeat (1500000) @ (posedge gtwiz_freerun_clock);
  repeat (300000) @ (posedge gtwiz_freerun_clock);
  repeat (1000000) @ (posedge gtwiz_freerun_clock);
  repeat (1000000) @ (posedge gtwiz_freerun_clock);
  if (!INTF_PASS[0]) begin
    INTF_TIMEOUT[0] = 1'b1;
    repeat (5) @ (posedge gtwiz_freerun_clock);
    TB_TIMEOUT[0] = 1'b1;
#100us;
    $display("Time : %12d ns   INTF0 :: simulation timeout. ", $time);
  end
end

always #3.2ns intf0_clk_lr0_tx =  ~intf0_clk_lr0_tx;

 
always #3.2ns intf0_clk_lr0_rx =  ~intf0_clk_lr0_rx;

 
 
wire clk_INTF0_R0       =      
  (intf0_rate_sel == 4'd0) ? intf0_clk_lr0_rx  : 
 
  (intf0_rate_sel > 4'd0) ?  intf0_clk_lr0_rx : 
  intf0_clk_lr0_rx; 
 
 
 
 
 
 
 
 
 
 
reg [10:0] link_up_ctr_ip1 = 11'd0;
reg        link_stable_ip1 = 1'b0;
always @(posedge gtwiz_freerun_clock) begin
  if (INTF1_link_status_out !== 1'b1) begin
    link_up_ctr_ip1 <= 11'd0;
    link_stable_ip1 <= 1'b0;
  end
  else begin
    if (&link_up_ctr_ip1)
      link_stable_ip1 <= 1'b1;
    else
      link_up_ctr_ip1 <= link_up_ctr_ip1 + 11'd1;
  end
end

initial begin
#1;
  $display("my_gtm example design simulation started");
  gtwiz_INTF1_rst_all_in = 1'b0;
  wait( gt_powergood || &(TB_TIMEOUT) );
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM gt_powergood assertion failed \n for INTERFACE_1");
    end
  end
  else begin
    $display("gt_powergood asserted for INTERFACE_1");
  end
  wait( (!INTF1_rst_tx_done_out) || &(TB_TIMEOUT) );
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX Resetdone deassertion failed \n for INTERFACE_1");
    end
  end
  wait( (!INTF1_rst_rx_done_out) || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM RX Resetdone deassertion failed \n for INTERFACE_1");
    end
  end
  wait( (INTF1_rst_tx_done_out && INTF1_rst_rx_done_out) || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX/RX Resetdone assertion failed \n for INTERFACE_1");
    end
  end
  //wait for 30 free running clock edges to reset GT
  //Note:: 30 is arbitrary no.
  repeat (30) @ (posedge gtwiz_freerun_clock);
  gtwiz_INTF1_rst_all_in = 1'b1;
  //Assert GT reset for 5 clock cycles.
  //Note:: 5 is arbitrary no.
  repeat (5) @ (posedge gtwiz_freerun_clock);
  gtwiz_INTF1_rst_all_in = 1'b0;
  wait ((!INTF1_rst_tx_done_out) || &(TB_TIMEOUT)); 
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX Resetdone deassertion failed \n for INTERFACE_1 after tb primary reset");
    end
  end
  wait ((!INTF1_rst_rx_done_out) || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM RX Resetdone deassertion failed \n for INTERFACE_1 after tb primary reset");
    end
  end
  assign intf1_t2 = 415.0390625 - (415.0390625)*0.01;
  assign intf1_t3 = 415.0390625 + (415.0390625)*0.01;
  wait( INTF1_rst_tx_done_out || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX Resetdone assertion failed \n for INTERFACE_1 after tb primary reset");
    end
  end
  $display("INTF1 CONFIG0 TX RESET DONE asserted");
  @ (posedge INTF1_TX_usrclk) intf1_t0 = $realtime;
  @ (posedge INTF1_TX_usrclk) intf1_t1 = $realtime;
  intf1_frequency = 1000 / (intf1_t1 - intf1_t0);
  $display("INTF1 CONFIG0 TXUSRCLK Frequency = %g", intf1_frequency);
  if ((intf1_frequency <= intf1_t3 ) &&  (intf1_frequency >=  intf1_t2))
    $display(" TX_USER CLOCK IS PROPER. TX_USER_CLOCK is %g and EXPECTED is [%g:%g]",intf1_frequency, intf1_t2 , intf1_t3);
  else begin
    $display("Error: GTM TX_USER CLOCK IS NOT PROPER. \n TX_USER_CLOCK is %g and EXPECTED is [%g:%g]",intf1_frequency, intf1_t2 , intf1_t3);
    INTF_FAIL[1] = 1'b1;
  end
  assign intf1_t2 =415.0390625 - (415.0390625)*0.01;
  assign intf1_t3 =415.0390625 + (415.0390625)*0.01;
  wait( INTF1_rst_rx_done_out || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM RX Resetdone assertion failed \n for INTERFACE_1 after tb primary reset");
    end
  end
  $display("INTF1 CONFIG0 RX RESET DONE asserted");
  @ (posedge INTF1_RX_usrclk) intf1_t0 = $realtime;
  @ (posedge INTF1_RX_usrclk) intf1_t1 = $realtime;
  intf1_frequency = 1000 / (intf1_t1 - intf1_t0);
  $display("INTF1 CONFIG0 RXUSRCLK Frequency = %g", intf1_frequency);
  if ((intf1_frequency <= intf1_t3 ) &&  (intf1_frequency >=  intf1_t2 ))
    $display("RX_USER CLOCK IS PROPER. RX_USER_CLOCK is %g and EXPECTED is [%g:%g]",intf1_frequency, intf1_t2 , intf1_t3);
  else begin
    $display("Error: GTM RX_USER CLOCK IS NOT PROPER. \n RX_USER_CLOCK POST PLL LOCK is %g and EXPECTED is [%g:%g]",intf1_frequency, intf1_t2 , intf1_t3);
    INTF_FAIL[1] = 1'b1;
  end
  #100;
  wait(link_stable_ip1 || &(TB_TIMEOUT)); 
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM link stable assertion failed \n for INTERFACE_1 CONFIG0");
    end
  end
  @ (posedge INTF1_TX_usrclk) intf1_t0 = $realtime;
  @ (posedge INTF1_TX_usrclk) intf1_t1 = $realtime;
  intf1_frequency = 1000 / (intf1_t1 - intf1_t0);
  $display("INTF1 CONFIG0 :::::: POST LINK STABLE TXUSRCLK Frequency = %g", intf1_frequency);
  #100ns;
  @ (posedge INTF1_RX_usrclk) intf1_t0 = $realtime;
  @ (posedge INTF1_RX_usrclk) intf1_t1 = $realtime;
  intf1_frequency = 1000 / (intf1_t1 - intf1_t0);
  $display("INTF1 CONFIG0 :::::: POST LINK STABLE RXUSRCLK Frequency = %g", intf1_frequency);
  $display("Time : %12d ns   INTF1 CONFIG0: simulation completed", $time);
  intf1_gpio_enable = 1'b0;
  intf1_rate_sel = 4'd1;
  $display(" INTF1 RATE CHANGE INITIATED ");
  wait(!INTF1_rst_tx_done_out || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX Resetdone deassertion failed \n in INTERFACE_1 CONFIG1 rate change sequence");
    end
  end
  wait(!INTF1_rst_rx_done_out || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM RX Resetdone deassertion failed \n in INTERFACE_1 CONFIG1 rate change sequence");
    end
  end
  assign intf1_t2 = 322.2656250 - (322.2656250)*0.01;
  assign intf1_t3 = 322.2656250 + (322.2656250)*0.01;
  wait( INTF1_rst_tx_done_out || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM TX Resetdone assertion failed \n in INTERFACE_1 CONFIG1 rate change sequence");
    end
  end
  $display("INTF1 CONFIG1 ::::: TX RESET DONE asserted");
  @ (posedge INTF1_TX_usrclk) intf1_t0 = $realtime;
  @ (posedge INTF1_TX_usrclk) intf1_t1 = $realtime;
  intf1_frequency = 1000 / (intf1_t1 - intf1_t0);
  $display("INTF1 CONFIG1:::::::TXUSRCLK Frequency = %g", intf1_frequency);
  if ((intf1_frequency <= intf1_t3 ) &&  (intf1_frequency >=  intf1_t2 ))
    $display("INTF1 CONFIG1::::::: TX_USER CLOCK IS PROPER. TX_USER_CLOCK is %g and EXPECTED is [%g:%g]",intf1_frequency, intf1_t2 , intf1_t3);
  else begin
    $display("Error: GTM TX_USER CLOCK IS NOT PROPER. \n TX_USER_CLOCK is %g and EXPECTED is [%g:%g]",intf1_frequency, intf1_t2 , intf1_t3);
    INTF_FAIL[1] = 1'b1;
  end
  assign intf1_t2 = 322.2656250 - (322.2656250)*0.01;
  assign intf1_t3 = 322.2656250 + (322.2656250)*0.01;
  wait( INTF1_rst_rx_done_out || &(TB_TIMEOUT));
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM RX Resetdone assertion failed \n in INTERFACE_1 CONFIG1 rate change sequence");
    end
  end
  $display("INTF1 CONFIG1 ::::: RX RESET DONE asserted");
  #100;
  intf1_gpio_enable = 1'b0;
  @ (posedge INTF1_RX_usrclk) intf1_t0 = $realtime;
  @ (posedge INTF1_RX_usrclk) intf1_t1 = $realtime;
  intf1_frequency = 1000 / (intf1_t1 - intf1_t0);
  $display("INTF1 CONFIG1:::::::RXUSRCLK Frequency = %g", intf1_frequency);
if ((intf1_frequency <= intf1_t3 ) &&  (intf1_frequency >=  intf1_t2 ))
$display("INTF1 CONFIG1::::::: RX_USER CLOCK IS  PROPER RX_USER_CLOCK POST PLL LOCK is %g and EXPECTED is [%g:%g]",intf1_frequency, intf1_t2 , intf1_t3);
  else begin
    $display("Error: GTM RX_USER CLOCK IS NOT PROPER. \n RX_USER_CLOCK POST PLL LOCK is %g and EXPECTED is [%g:%g]",intf1_frequency, intf1_t2 , intf1_t3);
    INTF_FAIL[1] = 1'b1;
  end
  wait(link_stable_ip1 || &(TB_TIMEOUT)); 
  if (&(TB_TIMEOUT)) begin 
    if (!(&INTF_PASS)) begin 
      $display("Error: GTM link stable assertion failed \n for INTERFACE_1 CONFIG1");
    end
  end
  @ (posedge INTF1_TX_usrclk) intf1_t0 = $realtime;
  @ (posedge INTF1_TX_usrclk) intf1_t1 = $realtime;
  intf1_frequency = 1000 / (intf1_t1 - intf1_t0);
  $display("INTF1 POST CONFIG1 LINK STABLE TXUSRCLK Frequency = %g", intf1_frequency);
  #100ns;
  @ (posedge INTF1_RX_usrclk) intf1_t0 = $realtime;
  @ (posedge INTF1_RX_usrclk) intf1_t1 = $realtime;
  intf1_frequency = 1000 / (intf1_t1 - intf1_t0);
  $display("INTF1 CONFIG1::::: POST LINK STABLE RXUSRCLK Frequency = %g", intf1_frequency);
  INTF_PASS[1] = 1'b1;
end

initial begin
  intf1_frequency_int = 415.0390625; 
  repeat (300000) @ (posedge gtwiz_freerun_clock);
  if (intf1_frequency_int <= 150 )
    repeat (1500000) @ (posedge gtwiz_freerun_clock);
  intf1_frequency_int = 322.2656250; 
  repeat (300000) @ (posedge gtwiz_freerun_clock);
  if (intf1_frequency_int <= 150 )
    repeat (1500000) @ (posedge gtwiz_freerun_clock);
  repeat (300000) @ (posedge gtwiz_freerun_clock);
  repeat (1000000) @ (posedge gtwiz_freerun_clock);
  repeat (1000000) @ (posedge gtwiz_freerun_clock);
  if (!INTF_PASS[1]) begin
    INTF_TIMEOUT[1] = 1'b1;
    repeat (5) @ (posedge gtwiz_freerun_clock);
    TB_TIMEOUT[1] = 1'b1;
#100us;
    $display("Time : %12d ns   INTF1 :: simulation timeout. ", $time);
  end
end

always #3.2ns intf1_clk_lr0_tx =  ~intf1_clk_lr0_tx;

always #3.2ns intf1_clk_lr1_tx =  ~intf1_clk_lr1_tx;
 
always #3.2ns intf1_clk_lr0_rx =  ~intf1_clk_lr0_rx;
always #3.2ns intf1_clk_lr1_rx =  ~intf1_clk_lr1_rx;

 
 
wire clk_INTF1_R0       =      
  (intf1_rate_sel == 4'd0) ? intf1_clk_lr0_rx  : 
  (intf1_rate_sel == 4'd1) ? intf1_clk_lr1_rx  : 
  (intf1_rate_sel < 4'd1) ? intf1_clk_lr0_rx : 
 
  (intf1_rate_sel > 4'd1) ?  intf1_clk_lr1_rx : 
  intf1_clk_lr0_rx; 
 
 
 
 
 
 
 
 
 
 

initial begin
  wait(&INTF_PASS || (&(TB_TIMEOUT))); 
  if (&INTF_PASS) begin
    if(|INTF_FAIL) begin
      $display("** Time : %12d ns Error: GTM User clocks are not proper. Test failed",$time);
    end else begin
      $display("Time : %12d ns   PASS: simulation completed", $time);
      $display("**  Test completed successfully"); 
    end
  end else begin
  $display("**Error: GTM Example Design Simulation timed out"); 
  end
  $finish;
end

reg  QUAD0_GTREFCLK0_in_p = 1'b0;
always #3.2ns QUAD0_GTREFCLK0_in_p =  ~QUAD0_GTREFCLK0_in_p;
reg  QUAD0_GTREFCLK1_in_p = 1'b0;
always #3.2ns QUAD0_GTREFCLK1_in_p =  ~QUAD0_GTREFCLK1_in_p;
//APB Clock generation
//always #2.5ns gtwiz_freerun_clock = ~ gtwiz_freerun_clock;
initial begin
    wait(gt_powergood === 0); // Starting free running clock after gt_powergood reached good state. 
    #20;
    forever #2.098ns gtwiz_freerun_clock = ~ gtwiz_freerun_clock;
end 
assign	QUAD0_GTREFCLK0_in_n = !QUAD0_GTREFCLK0_in_p;
assign	QUAD0_GTREFCLK1_in_n = !QUAD0_GTREFCLK1_in_p;

integer quad0_gtm_ch0_n;
integer quad0_gtm_ch0_p;
always@(*) begin
  force quad0_gtm_ch0_n = exdes_inst.exdes_inst.dut_inst.inst.intf_quad_map_inst.quad_top_inst.gt_quad_base_0_inst.inst.quad_inst.CH0_GTMTXN_integer;
  force quad0_gtm_ch0_p = exdes_inst.exdes_inst.dut_inst.inst.intf_quad_map_inst.quad_top_inst.gt_quad_base_0_inst.inst.quad_inst.CH0_GTMTXP_integer;
  force exdes_inst.exdes_inst.dut_inst.inst.intf_quad_map_inst.quad_top_inst.gt_quad_base_0_inst.inst.quad_inst.CH0_GTMRXN_integer = quad0_gtm_ch0_n;
  force exdes_inst.exdes_inst.dut_inst.inst.intf_quad_map_inst.quad_top_inst.gt_quad_base_0_inst.inst.quad_inst.CH0_GTMRXP_integer = quad0_gtm_ch0_p;
end
integer quad0_gtm_ch3_n;
integer quad0_gtm_ch3_p;
always@(*) begin
  force quad0_gtm_ch3_n = exdes_inst.exdes_inst.dut_inst.inst.intf_quad_map_inst.quad_top_inst.gt_quad_base_0_inst.inst.quad_inst.CH3_GTMTXN_integer;
  force quad0_gtm_ch3_p = exdes_inst.exdes_inst.dut_inst.inst.intf_quad_map_inst.quad_top_inst.gt_quad_base_0_inst.inst.quad_inst.CH3_GTMTXP_integer;
  force exdes_inst.exdes_inst.dut_inst.inst.intf_quad_map_inst.quad_top_inst.gt_quad_base_0_inst.inst.quad_inst.CH3_GTMRXN_integer = quad0_gtm_ch3_n;
  force exdes_inst.exdes_inst.dut_inst.inst.intf_quad_map_inst.quad_top_inst.gt_quad_base_0_inst.inst.quad_inst.CH3_GTMRXP_integer = quad0_gtm_ch3_p;
end

my_gtm_exdes exdes_inst
(
			.QUAD0_GTREFCLK0_in_p(QUAD0_GTREFCLK0_in_p),
			.QUAD0_GTREFCLK0_in_n(QUAD0_GTREFCLK0_in_n),
			.QUAD0_GTREFCLK1_in_p(QUAD0_GTREFCLK1_in_p),
			.QUAD0_GTREFCLK1_in_n(QUAD0_GTREFCLK1_in_n),
.QUAD0_rxp   ( QUAD0_rxp ),
.QUAD0_rxn   ( QUAD0_rxn ),
.QUAD0_txp   ( QUAD0_txp ),
.QUAD0_txn   ( QUAD0_txn ),
.INTF0_link_status_out           (INTF0_link_status_out),
.intf0_rate_sel                  (intf0_rate_sel),
.intf0_gpio_enable               (intf0_gpio_enable),
.gtwiz_INTF0_rst_all_in          (gtwiz_INTF0_rst_all_in),
.gtwiz_INTF0_rst_tx_datapath_in  ( gtwiz_INTF0_rst_tx_datapath_in ),
.INTF0_rst_tx_done_out           (INTF0_rst_tx_done_out),
.INTF0_TX_usrclk				(INTF0_TX_usrclk				)	,
.gtwiz_INTF0_rst_rx_datapath_in  (gtwiz_INTF0_rst_rx_datapath_in),
.INTF0_rst_rx_done_out           (INTF0_rst_rx_done_out),
.INTF0_RX_usrclk				(INTF0_RX_usrclk				)	,
.INTF1_link_status_out           (INTF1_link_status_out),
.intf1_rate_sel                  (intf1_rate_sel),
.intf1_gpio_enable               (intf1_gpio_enable),
.gtwiz_INTF1_rst_all_in          (gtwiz_INTF1_rst_all_in),
.gtwiz_INTF1_rst_tx_datapath_in  ( gtwiz_INTF1_rst_tx_datapath_in ),
.INTF1_rst_tx_done_out           (INTF1_rst_tx_done_out),
.INTF1_TX_usrclk				(INTF1_TX_usrclk				)	,
.gtwiz_INTF1_rst_rx_datapath_in  (gtwiz_INTF1_rst_rx_datapath_in),
.INTF1_rst_rx_done_out           (INTF1_rst_rx_done_out),
.INTF1_RX_usrclk				(INTF1_RX_usrclk				)	,
.gtwiz_freerun_clock ( gtwiz_freerun_clock ),
.gt_powergood(gt_powergood)
);

endmodule
